/******************************************************************************
**
** $Header: color.h,v 1.23 92/07/06 17:36:55 fxt Exp $
**
**
** $Log:	color.h,v $
 * Revision 1.23  92/07/06  17:36:55  17:36:55  fxt (Filippo Tampieri)
 * *** empty log message ***
 * 
 * Revision 1.22  92/07/02  23:45:16  23:45:16  fxt (Filippo Tampieri)
 * *** empty log message ***
 * 
 * Revision 1.21  92/06/10  13:39:31  13:39:31  fxt (Filippo Tampieri)
 * *** empty log message ***
 * 
 * Revision 1.20  92/06/04  15:04:13  15:04:13  fxt (Filippo Tampieri)
 * *** empty log message ***
 * 
 * Revision 1.19  92/05/28  10:18:00  10:18:00  fxt (Filippo Tampieri)
 * *** empty log message ***
 * 
 * Revision 1.18  92/05/05  14:06:23  14:06:23  fxt (Filippo Tampieri)
 * *** empty log message ***
 * 
 * Revision 1.17  92/04/23  15:03:43  15:03:43  fxt (Filippo Tampieri)
 * *** empty log message ***
 * 
 * Revision 1.16  92/04/23  14:56:51  14:56:51  fxt (Filippo Tampieri)
 * new memory management using Pool
 * 
 * Revision 1.15  92/04/23  14:48:29  14:48:29  fxt (Filippo Tampieri)
 * *** empty log message ***
 * 
 * Revision 1.14  92/03/18  16:03:33  16:03:33  fxt (Filippo Tampieri)
 * faster implementation using inline functions
 * 
 * Revision 1.1  91/06/04  23:12:48  23:12:48  fxt (Filippo Tampieri)
 * Initial revision
 * 
**
******************************************************************************/

#ifndef COLOR_H
#define COLOR_H

#include <iostream.h>
#include <stdio.h>
#include <math.h>
#include <common.h>
#include <pool.h>

#define R		0
#define G		1
#define B		2
#define RGBEXP	3
#define RGBEXPOFFSET	128

#define NUM_SPECTRAL_SAMPLES	3

typedef float Sreal;
typedef double Lreal;

class RGB;

class Spectrum {
private:
	Sreal a[NUM_SPECTRAL_SAMPLES];
	static Mpool pool;
public:
	friend int operator==(const Spectrum&, const Spectrum&);
	Spectrum()									{ ; }
	Spectrum(Sreal c)							{ a[0] = a[1] = a[2] = c; }
	Spectrum(Sreal c0, Sreal c1, Sreal c2)	{ a[0] = c0, a[1] = c1, a[2] = c2; }
	Spectrum(const Spectrum&);
	Spectrum(const RGB&);
	Sreal& operator[](int i)				{ return a[i]; }
	const Sreal& operator[](int i) const	{ return a[i]; }
	void zero();
	Sreal norm() const;
	Boolean normalize();
	Boolean normalize(const Spectrum&);
	Spectrum operator+=(const Spectrum&);
	Spectrum operator-=(const Spectrum&);
	Spectrum operator*=(const Spectrum&);
	Spectrum operator*=(Sreal);
	Spectrum operator/=(const Spectrum&);
	Spectrum operator/=(Sreal);
	Spectrum powerSeries() const;
	Sreal integral() const;
	Sreal brightness() const;
	Sreal max() const;
	Sreal min() const;
	int imax() const;
	int imin() const;
	void clamp(Sreal c = 1.0);
	void write(FILE *) const;
	void read(FILE *);
	static Sreal eps;
	static Spectrum* New(unsigned int n)			{ return (Spectrum*)pool.Alloc(n); }
	static void Delete(unsigned int n, Spectrum* a)	{ pool.Free(n, (void*)a); }
	static void *operator new(size_t)				{ return pool.Alloc(1); }
	static void operator delete(void *p, size_t)	{ pool.Free(1, p); }
};

extern ostream& operator<<(ostream&, const Spectrum&);
extern istream& operator>>(istream&, Spectrum&);

/* use inline version as long as NUM_SPECTRAL_SAMPLES = 3

extern Spectrum operator+(const Spectrum&, const Spectrum&);
extern Spectrum operator-(const Spectrum&, const Spectrum&);
extern Spectrum operator*(const Spectrum&, const Spectrum&);
extern Spectrum operator*(Sreal, const Spectrum&);
extern Spectrum operator*(const Spectrum&, Sreal);
extern Spectrum operator/(const Spectrum&, const Spectrum&);
extern Spectrum operator/(const Spectrum&, Sreal);
extern Spectrum max(const Spectrum&, const Spectrum&);
extern Spectrum min(const Spectrum&, const Spectrum&);
extern Spectrum max0(const Spectrum&);
extern Spectrum abs(const Spectrum&);
*/

inline Spectrum::Spectrum(const Spectrum& s)
{
	a[0] = s.a[0];
	a[1] = s.a[1];
	a[2] = s.a[2];
}

inline void Spectrum::zero()
{
	a[0] = a[1] = a[2] = 0.0;
}

inline Spectrum Spectrum::operator+=(const Spectrum& s)
{
	a[0] += s.a[0];
	a[1] += s.a[1];
	a[2] += s.a[2];

	return *this;
}

inline Spectrum Spectrum::operator-=(const Spectrum& s)
{
	a[0] -= s.a[0];
	a[1] -= s.a[1];
	a[2] -= s.a[2];

	return *this;
}

inline Spectrum Spectrum::operator*=(const Spectrum& s)
{
	a[0] *= s.a[0];
	a[1] *= s.a[1];
	a[2] *= s.a[2];

	return *this;
}

inline Spectrum Spectrum::operator*=(Sreal c)
{
	a[0] *= c;
	a[1] *= c;
	a[2] *= c;

	return *this;
}

inline Spectrum Spectrum::operator/=(const Spectrum& s)
{
	a[0] /= s.a[0];
	a[1] /= s.a[1];
	a[2] /= s.a[2];

	return *this;
}

inline Spectrum Spectrum::operator/=(Sreal c)
{
	a[0] /= c;
	a[1] /= c;
	a[2] /= c;

	return *this;
}

inline Spectrum Spectrum::powerSeries() const
{
	return Spectrum(1.0 / (1.0 - a[0]),
					1.0 / (1.0 - a[1]),
					1.0 / (1.0 - a[2]));
}

inline Sreal Spectrum::integral() const
{
	return 0.263 * a[0] + 0.655 * a[1] + 0.082 * a[2];
}

inline Sreal Spectrum::brightness() const
{
	return 0.263 * a[0] + 0.655 * a[1] + 0.082 * a[2];
}

inline Sreal Spectrum::norm() const
{
	return Sreal(sqrt(a[0] * a[0] + a[1] * a[1] + a[2] * a[2]));
}

inline Sreal Spectrum::max() const
{
	return (a[0] > a[1]) ? (a[0] > a[2] ? a[0] : a[2]) : (a[1] > a[2] ? a[1] : a[2]);
}

inline Sreal Spectrum::min() const
{
	return (a[0] < a[1]) ? (a[0] < a[2] ? a[0] : a[2]) : (a[1] < a[2] ? a[1] : a[2]);
}

inline int Spectrum::imax() const
{
	return (a[0] > a[1]) ? (a[0] > a[2] ? 0 : 2) : (a[1] > a[2] ? 1 : 2);
}

inline int Spectrum::imin() const
{
	return (a[0] < a[1]) ? (a[0] < a[2] ? 0 : 2) : (a[1] < a[2] ? 1 : 2);
}

inline Spectrum operator+(const Spectrum& s1, const Spectrum& s2)
{
	return Spectrum(s1[0] + s2[0], s1[1] + s2[1], s1[2] + s2[2]);
}

inline Spectrum operator-(const Spectrum& s1, const Spectrum& s2)
{
	return Spectrum(s1[0] - s2[0], s1[1] - s2[1], s1[2] - s2[2]);
}

inline Spectrum operator*(const Spectrum& s1, const Spectrum& s2)
{
	return Spectrum(s1[0] * s2[0], s1[1] * s2[1], s1[2] * s2[2]);
}

inline Spectrum operator*(Sreal c, const Spectrum& s1)
{
	return Spectrum(s1[0] * c, s1[1] * c, s1[2] * c);
}

inline Spectrum operator*(const Spectrum& s1, Sreal c)
{
	return Spectrum(s1[0] * c, s1[1] * c, s1[2] * c);
}

inline Spectrum operator/(const Spectrum& s1, const Spectrum& s2)
{
	return Spectrum(s1[0] / s2[0], s1[1] / s2[1], s1[2] / s2[2]);
}

inline Spectrum operator/(const Spectrum& s1, Sreal c)
{
	return Spectrum(s1[0] / c, s1[1] / c, s1[2] / c);
}

inline Spectrum max(const Spectrum& s1, const Spectrum& s2)
{
	return Spectrum(MAX(s1[0], s2[0]),
					MAX(s1[1], s2[1]),
					MAX(s1[2], s2[2]));
}

inline Spectrum min(const Spectrum& s1, const Spectrum& s2)
{
	return Spectrum(MIN(s1[0], s2[0]),
					MIN(s1[1], s2[1]),
					MIN(s1[2], s2[2]));
}

inline Spectrum max0(const Spectrum& s)
{
	return Spectrum(MAX(s[0], 0.0),
					MAX(s[1], 0.0),
					MAX(s[2], 0.0));
}

inline Spectrum abs(const Spectrum& s)
{
	return Spectrum(ABS(s[0]),
					ABS(s[1]),
					ABS(s[2]));
}

inline int operator==(Spectrum& s1, Spectrum& s2)
{
	return ABS(s1[0] - s2[0]) <= Spectrum::eps &&
		   ABS(s1[1] - s2[1]) <= Spectrum::eps &&
		   ABS(s1[2] - s2[2]) <= Spectrum::eps;
}

inline Boolean Spectrum::normalize()
{
	Sreal d = 0;
	Boolean st;

	d = a[0] * a[0] + a[1] * a[1] + a[2] * a[2];
	if(d <= eps)
		st = FALSE;
	else {
		d = 1.0 / Sreal(sqrt((double)d));
		a[0] *= d;
		a[1] *= d;
		a[2] *= d;
		st = TRUE;
	}
	return st;
}

inline Boolean Spectrum::normalize(const Spectrum& s)
{
	Sreal d = 0;
	Boolean st;

	d = s.a[0] * s.a[0] + s.a[1] * s.a[1] + s.a[2] * s.a[2];
	if(d <= eps)
		st = FALSE;
	else {
		d = 1.0 / Sreal(sqrt((double)d));
		a[0] = s.a[0] * d;
		a[1] = s.a[1] * d;
		a[2] = s.a[2] * d;
		st = TRUE;
	}

	return st;
}

inline void Spectrum::clamp(Sreal c)
{
	if(a[0] > c) a[0] = c;
	if(a[1] > c) a[1] = c;
	if(a[2] > c) a[2] = c;
}


typedef struct {
	unsigned char a[4];
} RGB32;

class RGB {
private:
	Sreal a[3];
	static Mpool pool;
public:
	friend int operator==(const RGB&, const RGB&);
	RGB()									{ ; }
	RGB(const RGB&);
	RGB(const Spectrum&);
	RGB(Sreal, Sreal, Sreal);
	void set(Sreal, Sreal, Sreal);
	Sreal& operator[](int i)				{ return a[i]; }
	const Sreal& operator[](int i) const	{ return a[i]; }
	Sreal norm() const;
	Boolean normalize();
	Boolean normalize(const RGB&);
	RGB operator+=(const RGB&);
	RGB operator-=(const RGB&);
	RGB operator*=(const RGB&);
	RGB operator*=(Sreal);
	RGB operator/=(Sreal);
	RGB powerSeries() const;
	Sreal integral() const;
	Sreal brightness() const;
	Sreal max() const;
	Sreal min() const;
	int imax() const;
	int imin() const;
	void clamp(Sreal c = 1.0);
	void map(Sreal c = 1.0);
	void expand(const RGB32&);
	void compress(RGB32&) const;
	static Sreal eps;
	static RGB* New(unsigned int n)					{ return (RGB*)pool.Alloc(n); }
	static void Delete(unsigned int n, RGB* a)		{ pool.Free(n, (void*)a); }
	static void *operator new(size_t)				{ return pool.Alloc(1); }
	static void operator delete(void *p, size_t)	{ pool.Free(1, p); }
};

extern ostream& operator<<(ostream&, const RGB&);
extern istream& operator>>(istream&, RGB&);

inline RGB::RGB(const RGB& v)
{
	a[R] = v.a[R];
	a[G] = v.a[G];
	a[B] = v.a[B];
}

/* assumes NUM_SPECTRAL_SAMPLES = 3 */
inline RGB::RGB(const Spectrum& s)
{
	a[R] = s[0];
	a[G] = s[1];
	a[B] = s[2];
}

inline RGB::RGB(Sreal r, Sreal g, Sreal b)
{
	a[R] = r;
	a[G] = g;
	a[B] = b;
}

inline void RGB::set(Sreal r, Sreal g, Sreal b)
{
	a[R] = r;
	a[G] = g;
	a[B] = b;
}

inline RGB RGB::operator+=(const RGB& v)
{
	a[R] += v[R];
	a[G] += v[G];
	a[B] += v[B];

	return *this;
}

inline RGB RGB::operator-=(const RGB& v)
{
	a[R] -= v[R];
	a[G] -= v[G];
	a[B] -= v[B];

	return *this;
}

inline RGB RGB::operator*=(const RGB& v)
{
	a[R] *= v[R];
	a[G] *= v[G];
	a[B] *= v[B];

	return *this;
}

inline RGB RGB::operator*=(Sreal c)
{
	a[R] *= c;
	a[G] *= c;
	a[B] *= c;

	return *this;
}

inline RGB RGB::operator/=(Sreal c)
{
	a[R] /= c;
	a[G] /= c;
	a[B] /= c;

	return *this;
}

inline RGB RGB::powerSeries() const
{
	return RGB(1.0 / (1.0 - a[R]),
			   1.0 / (1.0 - a[G]),
			   1.0 / (1.0 - a[B]));
}

inline Sreal RGB::integral() const
{
	return 0.263 * a[R] + 0.655 * a[G] + 0.082 * a[B];
}

inline Sreal RGB::brightness() const
{
	return 0.263 * a[R] + 0.655 * a[G] + 0.082 * a[B];
}

inline Sreal RGB::norm() const
{
	return Sreal(sqrt(a[R] * a[R] + a[G] * a[G] + a[B] * a[B]));
}

inline Sreal RGB::max() const
{
	return (a[R] > a[G]) ? (a[R] > a[B] ? a[R] : a[B]) : (a[G] > a[B] ? a[G] : a[B]);
}

inline Sreal RGB::min() const
{
	return (a[R] < a[G]) ? (a[R] < a[B] ? a[R] : a[B]) : (a[G] < a[B] ? a[G] : a[B]);
}

inline int RGB::imax() const
{
	return (a[R] > a[G]) ? (a[R] > a[B] ? R : B) : (a[G] > a[B] ? G : B);
}

inline int RGB::imin() const
{
	return (a[R] < a[G]) ? (a[R] < a[B] ? R : B) : (a[G] < a[B] ? G : B);
}

inline RGB operator+(const RGB& u, const RGB& v)
{
	return RGB(u[R] + v[R], u[G] + v[G], u[B] + v[B]);
}

inline RGB operator-(const RGB& u, const RGB& v)
{
	return RGB(u[R] - v[R], u[G] - v[G], u[B] - v[B]);
}

inline RGB operator*(const RGB& u, const RGB& v)
{
	return RGB(u[R] * v[R], u[G] * v[G], u[B] * v[B]);
}

inline RGB operator*(Sreal c, const RGB& v)
{
	return RGB(v[R] * c, v[G] * c, v[B] * c);
}

inline RGB operator*(const RGB& v, Sreal c)
{
	return RGB(v[R] * c, v[G] * c, v[B] * c);
}

inline RGB operator/(const RGB& v, Sreal c)
{
	return RGB(v[R] / c, v[G] / c, v[B] / c);
}

inline RGB max(const RGB& u, const RGB& v)
{
	return RGB(MAX(u[R], v[R]),
			   MAX(u[G], v[G]),
			   MAX(u[B], v[B]));
}

inline RGB min(const RGB& u, const RGB& v)
{
	return RGB(MIN(u[R], v[R]),
			   MIN(u[G], v[G]),
			   MIN(u[B], v[B]));
}

inline void RGB::clamp(Sreal c)
{
	if(a[R] > c) a[R] = c;
	if(a[G] > c) a[G] = c;
	if(a[B] > c) a[B] = c;
}

inline void RGB::map(Sreal c)
{
	Sreal s = 9.0 / c;

	if(a[R] <= 0.0)
		a[R] = 0.0;
	else {
		a[R] = log10(1.0 + s * a[R]);
		if(a[R] > 1.0)
			a[R] = 1.0;
	}
	if(a[G] <= 0.0)
		a[G] = 0.0;
	else {
		a[G] = log10(1.0 + s * a[G]);
		if(a[G] > 1.0)
			a[G] = 1.0;
	}
	if(a[B] <= 0.0)
		a[B] = 0.0;
	else {
		a[B] = log10(1.0 + s * a[B]);
		if(a[B] > 1.0)
			a[B] = 1.0;
	}
}

inline RGB max0(const RGB& v)
{
	return RGB(MAX(v[R], 0.0),
			   MAX(v[G], 0.0),
			   MAX(v[B], 0.0));
}

inline RGB abs(const RGB& v)
{
	return RGB(ABS(v[R]), ABS(v[G]), ABS(v[B]));
}

/* conversion */
inline Spectrum::Spectrum(const RGB& rgb)
{
	a[0] = rgb[R];
	a[1] = rgb[G];
	a[2] = rgb[B];
}

#endif

