/******************************************************************************
**
** $Header: matrix3x3.h,v 1.1 92/07/08 11:48:01 fxt Exp $
**
** $Log:	matrix3x3.h,v $
 * Revision 1.1  92/07/08  11:48:01  11:48:01  fxt (Filippo Tampieri)
 * Initial revision
 * 
**
******************************************************************************/

#ifndef MATRIX3X3_H
#define MATRIX3X3_H

#include <stream.h>
#include <common.h>
#include <geom.h>

class Matrix3x3 {
	friend Matrix3x3 Transpose(const Matrix3x3&);
	friend Matrix3x3 Inverse(const Matrix3x3&, Boolean&);
	friend Matrix3x3 operator+(const Matrix3x3&, const Matrix3x3&);
	friend Matrix3x3 operator-(const Matrix3x3&, const Matrix3x3&);
	friend Matrix3x3 operator*(const Matrix3x3&, const Matrix3x3&);
	friend ostream& operator<<(ostream&, const Matrix3x3&);
	friend istream& operator>>(istream&, Matrix3x3&);
private:
	Real m[3][3];
	Real Cofactor(int, int) const;
public:
	Matrix3x3()									{ ; }
	Matrix3x3(const Matrix3x3&);
	Real& operator()(int i, int j)				{ return m[i][j]; }
	const Real& operator()(int i, int j) const	{ return m[i][j]; }
	Matrix3x3& operator+=(const Matrix3x3&);
	Matrix3x3& operator-=(const Matrix3x3&);
	Matrix3x3& operator*=(const Matrix3x3&);
	Real Determinant() const;
	static Matrix3x3 I();
	static Matrix3x3 T(Real, Real);
	static Matrix3x3 S(Real, Real, Real sz = 1.0);
};

inline Matrix3x3 Inverse(const Matrix3x3& m)
{
	Boolean nonSingular;

	return Inverse(m, nonSingular);
}

#endif /* MATRIX3X3_H */

