/******************************************************************************
**
** $Header: /homes/fish/danix/crater/src/lib/CC/gext/RCS/plane.h,v 1.1 1993/09/15 10:41:50 danix Exp $
**
** $Log: plane.h,v $
 * Revision 1.1  1993/09/15  10:41:50  danix
 * Initial revision
 *
**
******************************************************************************/

#ifndef PLANE_H
#define PLANE_H

#include <geometry.h>
#include <ray.h>

class Plane {
	friend ostream& operator<<(ostream&, const Plane&);
	friend int operator==(const Plane&, const Plane&);
protected:
	Normal3d _normal;
	Real _d;
	void init(Real, Real, Real, Real);
	void init(const Normal3d&, Real);
	void init(const Normal3d&, const Point3d&);
	void init(const Point3d&, const Point3d&, const Point3d&);
	void init(int, const Point3d[]);
public:
	static Real eps;
	Plane()										{ }
	Plane(Real a, Real b, Real c, Real d)		{ init(a, b, c, d); }
	Plane(const Normal3d& n, Real d)			{ init(n, d); }
	Plane(const Normal3d& n, const Point3d& p)	{ init(n, p); }
	Plane(const Point3d& a, const Point3d& b, const Point3d& c)
	                                            { init(a, b, c); }
	Plane(int numVerts, const Point3d verts[])	{ init(numVerts, verts); }
	void set(Real a, Real b, Real c, Real d)	{ init(a, b, c, d); }
	void set(const Normal3d& n, Real d)			{ init(n, d); }
	void set(const Normal3d& n, const Point3d& p)
												{ init(n, p); }
	void set(const Point3d& a, const Point3d& b, const Point3d& c)
												{ init(a, b, c); }
	void set(int numVerts, const Point3d verts[])	{ init(numVerts, verts); }
	const Normal3d& normal() const				{ return _normal; }
	Real d() const								{ return _d; }
	Real eval(const Point3d& p) const			{ return (_normal | p) + _d; }
	Boolean intersect(const Ray&, Real&) const;
	Real intersect(const Point3d&, const Point3d&, Point3d&) const;
	const Point3d intersect(const Point3d&, const Point3d&) const;
	int classify(const Point3d&) const;
	static void setEps(Real _eps)				{ eps = _eps; }
};

extern istream& operator>>(istream&, Plane&);

inline int operator!=(const Plane& p, const Plane&q)
{
	return !(p == q);
}

inline Boolean operator<(const Point3d& point, const Plane& plane)
{
	return(plane.eval(point) < -plane.eps);
}

inline Boolean operator<=(const Point3d& point, const Plane& plane)
{
	return(plane.eval(point) <= plane.eps);
}

inline Boolean operator>(const Point3d& point, const Plane& plane)
{
	return(plane.eval(point) > plane.eps);
}

inline Boolean operator>=(const Point3d& point, const Plane& plane)
{
	return(plane.eval(point) >= -plane.eps);
}

inline Boolean operator==(const Point3d& point, const Plane& plane)
{
	return(ABS(plane.eval(point)) <= plane.eps);
}

inline int Plane::classify(const Point3d& point) const
{
	Real val = eval(point);
	return ((val < -eps) ? -1 : ((val > eps) ? 1 : 0));
}
	
#endif

