/******************************************************************************
**
** $Header: /homes/fish/danix/crater/src/lib/CC/pool/RCS/pool.h,v 1.2 1995/08/23 21:55:58 danix Exp $
**
** $Log: pool.h,v $
 * Revision 1.2  1995/08/23  21:55:58  danix
 * *** empty log message ***
 *
 * Revision 1.1  92/07/04  15:04:16  15:04:16  fxt (Filippo Tampieri)
 * Initial revision
 * 
**
******************************************************************************/

#ifndef POOL_H
#define POOL_H

#include <common.h>

typedef enum { PoolMaxNumBuckets = 10, PoolMaxArraySize = 1024 } PoolConsts;

class Chunk {
	friend class Pool;
	friend class Mpool;
private:
	Chunk *next;
	void *dummy;	// placeholder to get addresses aligned on 8 byte boundaries
};

class Pool {
private:
	unsigned int esize;
	unsigned int csize;
	void *elist;
	Chunk *clist;
	void* _Alloc();
public:
	Pool(unsigned int es, unsigned int cs = PoolMaxArraySize);
	~Pool();
	void* Alloc();
	void Free(void *ep);
	void Purge();
	static Boolean nopool;
};

class MpoolMap {
	friend class Mpool;
private:
	unsigned char a[PoolMaxArraySize + 1];
	unsigned int operator()(unsigned int i)	{ return a[i]; }
public:
	MpoolMap();
};

class Mpool {
private:
	unsigned int nbuckets;
	unsigned int maxasize;
	unsigned int esize;
	unsigned int csize;
	void *elist[PoolMaxNumBuckets];
	Chunk *clist[PoolMaxNumBuckets];
	void* _Alloc(unsigned int);
	static MpoolMap map;
public:
	Mpool(unsigned int es, unsigned int cs = PoolMaxArraySize);
	~Mpool();
	void* Alloc(unsigned int n);
	void Free(unsigned int n, void *ep);
	void Purge();
	static Boolean nopool;
};

inline Pool::Pool(unsigned int es, unsigned int cs)
{
	esize = es;
	csize = cs;
	elist = NIL(void);
	clist = NIL(Chunk);
}

inline Pool::~Pool()
{
	Purge();
}

inline void* Pool::Alloc()
{
	void *ep;

	if(nopool)
		ep = malloc(esize);
	else if(elist == NIL(void))
		ep = _Alloc();
	else {
		ep = elist;
		elist = *((void **)elist);
	}

	return ep;
}

inline void Pool::Free(void* ep)
{
	if(nopool)
		free(ep);
	else {
		*((void **)ep) = elist;
		elist = ep;
	}
}

inline Mpool::~Mpool()
{
	Purge();
}

inline void* Mpool::Alloc(unsigned int n)
{
	void *ep;

	if(nopool)
		ep = malloc(n * esize);
	else if(n > maxasize)
		ep = (void *)new char[n];
	else {
		unsigned int i = map(n);

		if(elist[i] == NIL(void))
			ep = _Alloc(i);
		else {
			ep = elist[i];
			elist[i] = *((void **)(elist[i]));
		}
	}

	return ep;
}

inline void Mpool::Free(unsigned int n, void* ep)
{
	if(nopool)
		free(ep);
	else if(n > maxasize)
		::delete ep;
	else {
		unsigned int i = map(n);

		*((void **)ep) = elist[i];
		elist[i] = ep;
	}
}

#endif

