/******************************************************************************
**
** $Header: vector2d.h,v 1.11 94/03/17 15:18:20 danix Exp $
**
**
** $Log:	vector2d.h,v $
 * Revision 1.11  94/03/17  15:18:20  15:18:20  danix (Dani Lischinski)
 * Changed copy constructor to copy the entire vector at once.
 * 
 * Revision 1.10  92/07/06  17:37:53  17:37:53  fxt
 * *** empty log message ***
 * 
 * Revision 1.9  92/07/02  23:39:19  23:39:19  fxt (Filippo Tampieri)
 * *** empty log message ***
 * 
 * Revision 1.8  92/04/23  15:04:15  15:04:15  fxt (Filippo Tampieri)
 * new memory management using Pool
 * 
 * Revision 1.7  92/03/11  14:23:49  14:23:49  fxt (Filippo Tampieri)
 * changed a[...] to x,y,...
 * 
 * Revision 1.6  91/11/20  15:02:29  15:02:29  danix (Dani Lischinski)
 * Added read/write.
 * 
 * Revision 1.5  91/10/16  17:13:45  17:13:45  fxt (Filippo Tampieri)
 * unary minus
 * 
 * Revision 1.4  91/09/11  11:25:44  11:25:44  fxt (Filippo Tampieri)
 * *** empty log message ***
 * 
 * Revision 1.3  91/08/30  10:53:17  10:53:17  fxt (Filippo Tampieri)
 * added abs()
 * 
 * Revision 1.2  91/06/06  10:49:27  10:49:27  fxt (Filippo Tampieri)
 * changed enum Coordinate to int and
 * added operator==.
 * 
 * Revision 1.1  91/06/04  15:48:44  15:48:44  fxt (Filippo Tampieri)
 * Initial revision
 * 
**
******************************************************************************/

#ifndef VECTOR2D_H
#define VECTOR2D_H

#include <stdio.h>
#include <iostream.h>
#include <math.h>
#include <common.h>
#include <pool.h>
#include <geom.h>

class Vector2d {
friend Vector2d operator+(const Vector2d& u, const Vector2d& v);
friend Vector2d operator-(const Vector2d& v);
friend Vector2d operator-(const Vector2d& u, const Vector2d& v);
friend Real operator|(const Vector2d& u, const Vector2d& v);
friend Vector2d operator*(const Vector2d& u, const Vector2d& v);
friend Vector2d operator*(Real c, const Vector2d& v);
friend Vector2d operator*(const Vector2d& v, Real c);
friend Vector2d operator/(const Vector2d& v, Real c);
friend Vector2d max(const Vector2d& u, const Vector2d& v);
friend Vector2d min(const Vector2d& u, const Vector2d& v);
friend Vector2d abs(const Vector2d& v);
friend int operator==(const Vector2d&, const Vector2d&);
private:
	Real x, y;
	static Mpool pool;
public:
	Vector2d()									{ ; }
	Vector2d(const Vector2d& v)					{ *this = v; }
	Vector2d(Real, Real);
	void set(Real, Real);
	Real& operator[](int i)				{ return ((Real *)this)[i]; }
	const Real& operator[](int i) const	{ return ((Real *)this)[i]; }
	Real norm() const;
	Boolean normalize();
	Boolean normalize(const Vector2d&);
	Vector2d operator+=(const Vector2d&);
	Vector2d operator-=(const Vector2d&);
	Vector2d operator*=(const Vector2d&);
	Vector2d operator*=(Real);
	Vector2d operator/=(Real);
	int imax();
	int imin();
    void write(FILE *) const;
    void read(FILE *);
	static Real eps, eps2;
	static void setEps(Real _eps)		{ eps = _eps; eps2 = _eps * _eps; }
	static Vector2d* New(unsigned int n)			{ return (Vector2d*)pool.Alloc(n); }
	static void Delete(unsigned int n, Vector2d* a)	{ pool.Free(n, (void*)a); }
	static void *operator new(size_t)				{ return pool.Alloc(1); }
	static void operator delete(void *p, size_t)	{ pool.Free(1, p); }
};

extern ostream& operator<<(ostream&, const Vector2d&);
extern istream& operator>>(istream&, Vector2d&);

inline Vector2d::Vector2d(Real _x, Real _y)
{
	x = _x;
	y = _y;
}

inline void Vector2d::set(Real _x, Real _y)
{
	x = _x;
	y = _y;
}

inline Vector2d Vector2d::operator+=(const Vector2d& v)
{
	x += v.x;
	y += v.y;

	return *this;
}

inline Vector2d Vector2d::operator-=(const Vector2d& v)
{
	x -= v.x;
	y -= v.y;

	return *this;
}

inline Vector2d Vector2d::operator*=(const Vector2d& v)
{
	x *= v.x;
	y *= v.y;

	return *this;
}

inline Vector2d Vector2d::operator*=(Real c)
{
	x *= c;
	y *= c;

	return *this;
}

inline Vector2d Vector2d::operator/=(Real c)
{
	x /= c;
	y /= c;

	return *this;
}

inline Real Vector2d::norm() const
{
	return sqrt(x * x + y * y);
}

inline int Vector2d::imax()
{
	return (x >= y) ? X : Y;
}

inline int Vector2d::imin()
{
	return (x <= y) ? X : Y;
}

inline Vector2d operator+(const Vector2d& u, const Vector2d& v)
{
	return Vector2d(u.x + v.x, u.y + v.y);
}

inline Vector2d operator-(const Vector2d& v)
{
	return Vector2d(-v.x, -v.y);
}

inline Vector2d operator-(const Vector2d& u, const Vector2d& v)
{
	return Vector2d(u.x - v.x, u.y - v.y);
}

inline Real operator|(const Vector2d& u, const Vector2d& v)
{
	return u.x * v.x + u.y * v.y;
}

inline Vector2d operator*(const Vector2d& u, const Vector2d& v)
{
	return Vector2d(u.x * v.x, u.y * v.y);
}

inline Vector2d operator*(Real c, const Vector2d& v)
{
	return Vector2d(v.x * c, v.y * c);
}

inline Vector2d operator*(const Vector2d& v, Real c)
{
	return Vector2d(v.x * c, v.y * c);
}

inline Vector2d operator/(const Vector2d& v, Real c)
{
	return Vector2d(v.x / c, v.y / c);
}

inline Vector2d max(const Vector2d& u, const Vector2d& v)
{
	return Vector2d(max(u.x, v.x), max(u.y, v.y));
}

inline Vector2d min(const Vector2d& u, const Vector2d& v)
{
	return Vector2d(min(u.x, v.x), min(u.y, v.y));
}

inline Vector2d abs(const Vector2d& v)
{
	return Vector2d(ABS(v.x), ABS(v.y));
}

typedef Vector2d Point2d;
typedef Vector2d Normal2d;

#endif

