/******************************************************************************
**
** $Header: vector3d.h,v 1.12 94/03/17 15:18:57 danix Exp $
**
**
** $Log:	vector3d.h,v $
 * Revision 1.12  94/03/17  15:18:57  15:18:57  danix (Dani Lischinski)
 * Changed copy constructor to copy the entire vector at once.
 * 
 * Revision 1.11  92/07/06  17:37:57  17:37:57  fxt
 * *** empty log message ***
 * 
 * Revision 1.10  92/07/02  23:39:22  23:39:22  fxt (Filippo Tampieri)
 * *** empty log message ***
 * 
 * Revision 1.9  92/04/23  15:04:20  15:04:20  fxt (Filippo Tampieri)
 * new memory management using Pool
 * 
 * Revision 1.8  92/03/11  14:23:52  14:23:52  fxt (Filippo Tampieri)
 * changed a[...] to x,y,...
 * 
 * Revision 1.7  91/11/20  15:02:35  15:02:35  danix (Dani Lischinski)
 * Added read/write.
 * 
 * Revision 1.6  91/10/16  17:13:53  17:13:53  fxt (Filippo Tampieri)
 * unary minus
 * 
 * Revision 1.5  91/09/11  11:25:56  11:25:56  fxt (Filippo Tampieri)
 * *** empty log message ***
 * 
 * Revision 1.4  91/08/30  10:53:24  10:53:24  fxt (Filippo Tampieri)
 * added abs()
 * 
 * Revision 1.3  91/06/06  15:16:28  15:16:28  fxt (Filippo Tampieri)
 * added constructor to homogeneous coordinates.
 * 
 * Revision 1.2  91/06/06  10:49:32  10:49:32  fxt (Filippo Tampieri)
 * changed enum Coordinate to int and
 * added operator==.
 * 
 * Revision 1.1  91/06/04  15:48:46  15:48:46  fxt (Filippo Tampieri)
 * Initial revision
 * 
**
******************************************************************************/

#ifndef VECTOR3D_H
#define VECTOR3D_H

#include <iostream.h>
#include <math.h>
#include <common.h>
#include <pool.h>
#include <geom.h>
#include <vector2d.h>

class Vector3d {
friend Vector3d operator+(const Vector3d& u, const Vector3d& v);
friend Vector3d operator-(const Vector3d& v);
friend Vector3d operator-(const Vector3d& u, const Vector3d& v);
friend Vector3d operator&(const Vector3d& u, const Vector3d& v);
friend Real operator|(const Vector3d& u, const Vector3d& v);
friend Vector3d operator*(const Vector3d& u, const Vector3d& v);
friend Vector3d operator*(Real c, const Vector3d& v);
friend Vector3d operator*(const Vector3d& v, Real c);
friend Vector3d operator/(const Vector3d& v, Real c);
friend Vector3d max(const Vector3d& u, const Vector3d& v);
friend Vector3d min(const Vector3d& u, const Vector3d& v);
friend Vector3d abs(const Vector3d& v);
friend int operator==(const Vector3d&, const Vector3d&);
private:
	Real x, y, z;
	static Mpool pool;
public:
	Vector3d()									{ ; }
	Vector3d(const Vector3d& v)					{ *this = v; }
	Vector3d(Real, Real, Real);
	Vector3d(const Vector2d&, Real);
	void set(Real, Real, Real);
	Real& operator[](int i)				{ return ((Real *)this)[i]; }
	const Real& operator[](int i) const	{ return ((Real *)this)[i]; }
	Real norm() const;
	Boolean normalize();
	Boolean normalize(const Vector3d&);
	Vector3d operator+=(const Vector3d&);
	Vector3d operator-=(const Vector3d&);
	Vector3d operator*=(const Vector3d&);
	Vector3d operator*=(Real);
	Vector3d operator/=(Real);
	int imax();
	int imin();
    void write(FILE *) const;
    void read(FILE *);
	static Real eps, eps2;
	static void setEps(Real _eps)		{ eps = _eps; eps2 = _eps * _eps; }
	static Vector3d* New(unsigned int n)			{ return (Vector3d*)pool.Alloc(n); }
	static void Delete(unsigned int n, Vector3d* a)	{ pool.Free(n, (void*)a); }
	static void *operator new(size_t)				{ return pool.Alloc(1); }
	static void operator delete(void *p, size_t)	{ pool.Free(1, p); }
};

extern ostream& operator<<(ostream&, const Vector3d&);
extern istream& operator>>(istream&, Vector3d&);

inline Vector3d::Vector3d(Real _x, Real _y, Real _z)
{
	x = _x;
	y = _y;
	z = _z;
}

inline Vector3d::Vector3d(const Vector2d& v, Real _z)
{
	x = v[X];
	y = v[Y];
	z = _z;
}

inline void Vector3d::set(Real _x, Real _y, Real _z)
{
	x = _x;
	y = _y;
	z = _z;
}

inline Vector3d Vector3d::operator+=(const Vector3d& v)
{
	x += v.x;
	y += v.y;
	z += v.z;

	return *this;
}

inline Vector3d Vector3d::operator-=(const Vector3d& v)
{
	x -= v.x;
	y -= v.y;
	z -= v.z;

	return *this;
}

inline Vector3d Vector3d::operator*=(const Vector3d& v)
{
	x *= v.x;
	y *= v.y;
	z *= v.z;

	return *this;
}

inline Vector3d Vector3d::operator*=(Real c)
{
	x *= c;
	y *= c;
	z *= c;

	return *this;
}

inline Vector3d Vector3d::operator/=(Real c)
{
	x /= c;
	y /= c;
	z /= c;

	return *this;
}

inline Real Vector3d::norm() const
{
	return sqrt(x * x + y * y + z * z);
}

inline int Vector3d::imax()
{
	return (x >= y) ? (x >= z ? X : Z) : (y >= z ? Y : Z);
}

inline int Vector3d::imin()
{
	return (x <= y) ? (x <= z ? X : Z) : (y <= z ? Y : Z);
}

inline Vector3d operator+(const Vector3d& u, const Vector3d& v)
{
	return Vector3d(u.x + v.x, u.y + v.y, u.z + v.z);
}

inline Vector3d operator-(const Vector3d& v)
{
	return Vector3d(-v.x, -v.y, -v.z);
}

inline Vector3d operator-(const Vector3d& u, const Vector3d& v)
{
	return Vector3d(u.x - v.x, u.y - v.y, u.z - v.z);
}

inline Vector3d operator&(const Vector3d& u, const Vector3d& v)
{
	return Vector3d(u.y * v.z - u.z * v.y,
					u.z * v.x - u.x * v.z,
					u.x * v.y - u.y * v.x);
}

inline Real operator|(const Vector3d& u, const Vector3d& v)
{
	return u.x * v.x + u.y * v.y + u.z * v.z;
}

inline Vector3d operator*(const Vector3d& u, const Vector3d& v)
{
	return Vector3d(u.x * v.x, u.y * v.y, u.z * v.z);
}

inline Vector3d operator*(Real c, const Vector3d& v)
{
	return Vector3d(v.x * c, v.y * c, v.z * c);
}

inline Vector3d operator*(const Vector3d& v, Real c)
{
	return Vector3d(v.x * c, v.y * c, v.z * c);
}

inline Vector3d operator/(const Vector3d& v, Real c)
{
	return Vector3d(v.x / c, v.y / c, v.z / c);
}

inline Vector3d max(const Vector3d& u, const Vector3d& v)
{
	return Vector3d(max(u.x, v.x),
					max(u.y, v.y),
					max(u.z, v.z));
}

inline Vector3d min(const Vector3d& u, const Vector3d& v)
{
	return Vector3d(min(u.x, v.x),
					min(u.y, v.y),
					min(u.z, v.z));
}

inline Vector3d abs(const Vector3d& v)
{
	return Vector3d(ABS(v.x), ABS(v.y), ABS(v.z));
}

typedef Vector3d Point3d;
typedef Vector3d Normal3d;

#endif

