/******************************************************************************
 *
 * $Source: /homes/fish/danix/latte/bsp/RCS/list.h,v $
 * $Author: danix $
 * $Locker:  $
 *
 * $Log: list.h,v $
 * Revision 1.1  1995/09/25  18:17:05  danix
 * Initial revision
 *
 *****************************************************************************/

#ifndef LIST_H
#define LIST_H

#include <common.h>

template <class T> class List;
template <class T> class ListIterator;

template <class T> class ListItem {
	friend class List<T>;
	friend class ListIterator<T>;
private:
	T item;
	ListItem<T> *next;
	ListItem()                         : item(0), next(0) {}
	ListItem(T i, ListItem<T> *n = 0)  : item(i), next(n) {}
	~ListItem()                                           {}
};

typedef void* ListPos;
#define LPos(p) ((ListItem<T>*)(p))

template <class T> class List {
	friend class ListIterator<T>;
public:
	/* primitive operations */
	int length() const                { return len; }
	ListPos first()                   { return &_head; }
	ListPos last()                    { return _last; }
	ListPos next(ListPos p)           { return LPos(p)->next; }
	Boolean isEmpty() const           { return len == 0; }
	Boolean isEnd(ListPos p) const    { return LPos(p) == _last; }
	T retrieve(ListPos p) const       { return LPos(p)->next->item; }
	T head() const                    { return _head.next->item; }
	T tail() const                    { return _last->item; }
	void store(ListPos p, T i)        { LPos(p)->next->item = i; }
	void insert(ListPos, T);
	void remove(ListPos);
	void append(T i)                  { insert(last(), i); }
	void push(T i)                    { insert(first(), i); }
	T pop()                           { T r = head(); 
										remove(first()); 
										return r; } 
	
	/* higher level operations */
	void sort();
	void clear();
	void apply(void (*f)(T));
	void insertInOrder(T);

	/* constructors, etc. */
	List(Boolean b = FALSE)   { len = 0; _last = &_head; deleteData = b; }
	~List()			          { clear(); }
	List& operator=(List<T>&);
private:
	unsigned int   len;
	ListItem<T>  _head;
	ListItem<T> *_last;
	Boolean deleteData;
};

template <class T> class ListIterator {
private:
	List<T> &list;
	ListPos p;
public:
	ListIterator(List<T> &l)        : list(l), p(l.first()) {}
	void reset()			        { p = list.first(); }
	T operator()() {
		if (list.isEnd(p))
			reset();
		T retval = list.retrieve(p);
		p = list.next(p);
		return retval;
	}
	Boolean next(T& i) {
		if (list.isEnd(p))
			return FALSE;
		else {
			i = (*this)();
			return TRUE;
		}
	}
};

#endif

