/******************************************************************************
 *
 * $Source: /homes/fish/danix/latte/bsp/RCS/main.C,v $
 * $Author: danix $
 * $Locker:  $
 *
 * $Log: main.C,v $
 * Revision 1.1  1995/09/25  18:22:07  danix
 * Initial revision
 *
 *****************************************************************************/

#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <bsptree.h>

static char *program;
static const char *bspSuffix = "bsp";

void usage()
{
	cerr << "Usage: " << program << " filename\n";
}

void readDataBase(FILE *fp, List<ObjectPtr>& objlist)
{
	// read number of objects
	int numObjects;
	if (fscanf(fp, "%d", &numObjects) != 1)
		Error("readDataBase: failed to read number of objects");
	cerr << "Number of objects is " << numObjects << endl;

	// read number of angles
	int numAngles;
	if (fscanf(fp, "%d", &numAngles) != 1)
		Error("readDataBase: failed to read number of angles");
	setNumAngles(numAngles);
	cerr << "Number of angles is " << numAngles << endl;

	// read ground filename
	char groundName[128];
	if (fscanf(fp, "%s", groundName) != 1)
		Error("readDataBase: failed to read ground filename");
	cerr << "Ground description file is " << groundName << endl;

	// read objects
	for (; numObjects > 0; numObjects--) {
		Object *o = new Object(fp);
		objlist.append(o);
	}
	cerr << "Done reading database\n";
}

void main(int argc, char** argv)
{
	char *dbName, *bspName;

	program = strdup(argv[0]);
 
	if (argc < 2) {
		usage();
		exit(1);
	}

	dbName = strdup(argv[1]);
	bspName = (char *)malloc(strlen(dbName) + strlen(bspSuffix) + 2);
	sprintf(bspName, "%s.%s", dbName, bspSuffix);
	cerr << "Input database file is: " << dbName << endl;
	cerr << "Output BSP-tree file is: " << bspName << endl;

	FILE *fin = fopen(dbName, "r");
	if (fin == NIL(FILE))
		Error("Can't open input file");
	FILE *fout = fopen(bspName, "w");
	if (fout == NIL(FILE))
		Error("Can't open output file");

	// read in database, creating a list of objects
	List<ObjectPtr> objlist;
	readDataBase(fin, objlist);
	cerr << "Read in " << objlist.length() << " objects\n";

	// construct a BSP-tree containing these objects
	BSPtree tree;
	tree.Init(objlist);

	// output the BSP-tree into a file
	tree.write(fout, TRUE);
	tree.print();
}
