/******************************************************************************
 *
 * $Source: /homes/sgi/danix/bsp/RCS/object.C,v $
 * $Author: danix $
 * $Locker:  $
 *
 * $Log: object.C,v $
 * Revision 1.5  1996/01/04  19:31:09  danix
 * Fixed color interpolation bug when splitting a triangle.
 *
 * Revision 1.4  1995/12/15  01:05:31  danix
 * Fixed a bug in readGeom
 *
 * Revision 1.3  1995/12/14  23:19:26  danix
 * Switched to new Vertex and Face classes;
 * Added processing of CNOFF objects.
 *
 * Revision 1.2  1995/11/21  21:12:01  danix
 * Objects now can read their number of faces from the database file.
 *
 * Revision 1.1  1995/09/25  18:21:28  danix
 * Initial revision
 *
 *****************************************************************************/

#include <common.h>
#include <object.h>
#include <string.h>
#include <array.h>

unsigned short Object::Angles = 0;
unsigned long  Object::count  = 0;

Object::~Object()
{
	free(fname);
	if (verts != NIL(Vertex))
		delete[] verts;
	if (faces != NIL(Face))
		delete[] faces;
}

void setNumAngles(int num)
{
	if (Object::Angles != 0)
		Error("setNumAngles: number of angles already set");
	if (num <= 0 || num > MAX_ANGLES)
		Error("setNumAngles: a bogus number of angles");
	Object::Angles = num;
}

static char string[256];
static const int lineLength = 256;
static char line[lineLength];


static void SkipComments(FILE *fp)
{
	while (fgets(line, lineLength, fp) && line[0] == '#')
		;
}


Object::Object(FILE *fp)
// fp is a stream containing object instance descriptions
{
	init();

	if (!fp)
		return;

	// else:

	// read object filename
	if (fscanf(fp, "%s", line) != 1)
		Error("Object::Object: can't read object's filename");
	if ((fname = strdup(line)) == NIL(char))
		Error("Object::Object: new failed");
	//cerr << "Object " << fname << endl;

	// read number of polygons:
	if (fscanf(fp, "%d", &num_faces) != 1)
		Error("Object::Object: can't read number of faces");

	// read transformation matrix
	register int i;
	double a, b, c, d;
	for (i = 0; i < 4; i++) {
		if (fscanf(fp, "%lf %lf %lf %lf", &a, &b, &c, &d) != 4)
			Error("Object::Object: can't read transformation");
		transform(i, 0) = a;
		transform(i, 1) = b;
		transform(i, 2) = c;
		transform(i, 3) = d;
	}

	// read extents corresponding to the different angles
	for (i = 0; i < Object::Angles; i++) {
		if (fscanf(fp, "%f %f", &(extents[i].min), &(extents[i].max)) != 2)
			Error("Object: problems reading object file");
	}
}

void Object::readGeom()
{
	int i, junk;
	double pt[3], nr[3];
	double r, g, b, a;
	int face[3];
	float col[3];
	Matrix4x4 invT;
	Vector4d v;
	Normal3d n;
	FILE *fp = fopen(fname, "r");
	if (!fp)
		Error("Object::readGeom: Couldn't open object file");

	SkipComments(fp);

	// read file type identifier string
	//cerr << "Object type is " << line;
	if (strncmp(line, "OFF", 3) == 0)
		otype = tOFF;
	else if (strncmp(line, "COFF", 4) == 0)
		otype = tCOFF;
	else if (strncmp(line, "NOFF", 4) == 0)
		otype = tNOFF;
	else if (strncmp(line, "CNOFF", 5) == 0)
		otype = tCNOFF;
	else
		Error("Object::readGeom: unknown object type");

	if (otype == tNOFF || otype == tCNOFF)
		invT = Transpose(Inverse(transform));

	SkipComments(fp);

	// read number of vertices, faces, and edges:
	if (sscanf(line, "%d %d %d", &num_verts, &num_faces, &junk) != 3)
		Error("Object::readGeom: problems reading object file");
	cerr << "Number of vertices/faces is " << num_verts << '/' <<
		num_faces << endl;
	
	// allocate memory to store object description:
	verts = new Vertex[num_verts];
	faces = new Face[num_faces];
	if (!verts || !faces)
		Error("Object::readGeom: new failed");

	for (i = 0; i < num_verts; i++) {

		SkipComments(fp);

		// read a vertex
		switch (otype) {
		case tOFF:
			if (sscanf(line, "%lf %lf %lf", pt, pt+1, pt+2) != 3)
				Error("Object::readGeom: problems reading object file");
			v = transform * Vector4d(pt[0], pt[1], pt[2], 1);
			verts[i] = Vertex(v[X], v[Y], v[Z]);
			break;
		case tCOFF:
			if (sscanf(line, "%lf %lf %lf %lf %lf %lf %lf",
					   pt, pt+1, pt+2, &r, &g, &b, &a) != 7)
				Error("Object::readGeom: problems reading object file");
			v = transform * Vector4d(pt[0], pt[1], pt[2], 1);
			verts[i] = Vertex(v[X], v[Y], v[Z]);
			verts[i].rgb(r, g, b);
			verts[i].alpha(a);
			break;
		case tNOFF:
			if (sscanf(line, "%lf %lf %lf %lf %lf %lf",
					   pt, pt+1, pt+2, nr, nr+1, nr+2) != 6)
				Error("Object::readGeom: problems reading object file");
			v = invT * Vector4d(nr[0], nr[1], nr[2], 0);
			n = Vector3d(v[X], v[Y], v[Z]), n.normalize();
			v = transform * Vector4d(pt[0], pt[1], pt[2], 1);
			verts[i] = Vertex(v[X], v[Y], v[Z], n);
			break;
		case tCNOFF:
			if (sscanf(line, "%lf %lf %lf %lf %lf %lf %lf %lf %lf %lf",
					   pt, pt+1, pt+2, nr, nr+1, nr+2,
					   &r, &g, &b, &a) != 10)
				Error("Object::readGeom: problems reading object file");
			v = invT * Vector4d(nr[0], nr[1], nr[2], 0);
			n = Vector3d(v[X], v[Y], v[Z]), n.normalize();
			v = transform * Vector4d(pt[0], pt[1], pt[2], 1);
			verts[i] = Vertex(v[X], v[Y], v[Z], n);
			verts[i].rgb(r, g, b);
			verts[i].alpha(a);
			break;
		default:
			Error("Object::readGeom: unsupported object type");
			break;
		}
	}

	for (i = 0; i < num_faces; i++) {

		SkipComments(fp);

		// read a face
		switch (otype) {
		case tOFF:
		case tNOFF:
			if (sscanf(line, "%d %d %d %d %f %f %f", &junk,
					   face, face+1, face+2, col, col+1, col+2) != 7)
				Error("Object::readGeom: problems reading object file");
			faces[i] = Face(face[0], face[1], face[2], col);
			break;
		case tCOFF:
		case tCNOFF:
			if (sscanf(line, "%d %d %d %d", &junk,
					   face, face+1, face+2) != 4)
				Error("Object::readGeom: problems reading object file");
			faces[i].set(face[0], face[1], face[2]);
			break;
		default:
			Error("Object::readGeom: unsupported object type");
			break;
		}
	}
	fclose(fp);
}

void Object::writeGeom() const
{
	int i;
	FILE *fp = fopen(fname, "w");
	if (!fp)
		Error("Object::writeGeom: Couldn't open object file");

	// write out comments containing the extents:
	for (i = 0; i < Angles; i++)
		fprintf(fp, "# %g %g\n", extent(i).min, extent(i).max);

	// write out object type:
	switch (otype) {
	case tOFF:
		fprintf(fp, "OFF\n");
		break;
	case tCOFF:
		fprintf(fp, "COFF\n");
		break;
	case tNOFF:
		fprintf(fp, "NOFF\n");
		break;
	case tCNOFF:
		fprintf(fp, "CNOFF\n");
		break;
	default:
		Error("Object::writeGeom: unknown object type");
		break;
	}

	// write number of vertices, faces, and edges:
	fprintf(fp, "%d %d 0\n", num_verts, num_faces);

	for (i = 0; i < num_verts; i++) {
		fprintf(fp, "%g %g %g", verts[i].x(), verts[i].y(), verts[i].z());
		if (otype == tNOFF || otype == tCNOFF)
		  fprintf(fp," %g %g %g", verts[i].nx(),verts[i].ny(), verts[i].nz());
		if (otype == tCOFF || otype == tCNOFF)
		  fprintf(fp," %g %g %g %g",
				  verts[i].r(), verts[i].g(), verts[i].b(), verts[i].alpha());
		fprintf(fp, "\n");
	}

	for (i = 0; i < num_faces; i++) {
		fprintf(fp, "3 %d %d %d", faces[i][0], faces[i][1], faces[i][2]);
		if (otype == tOFF || otype == tNOFF)
			fprintf(fp, " %g %g %g", faces[i].r(), faces[i].g(),
					faces[i].b());
		fprintf(fp, "\n");
	}
	fclose(fp);
}

static int classify(const Object& obj, const Plane& plane)
{
	// Recover the angle from the plane:
	double radians = acos(plane.normal()[X]);
	double delta = M_PI / (2.0 * Object::Angles);
	for (int angle = 0; angle < Object::Angles; angle++)
		if (abs((angle*M_PI)/Object::Angles - radians) < delta)
			break;

	// Compare the extent corresponding to this angle to the plane:
	double dist = - plane.eval(Point3d(0,0,0));
	if (obj.extent(angle).max <= dist)
		return -1;
	else if (obj.extent(angle).min >= dist)
		return 1;
	else
		return 0;
}

Boolean Object::operator<=(const Plane& plane)
{
	return (classify(*this, plane) == -1);
}

Boolean Object::operator>=(const Plane& plane)
{
	return (classify(*this, plane) == 1);
}

void splitFace(const Vertex verts[3], const Plane& plane,
			   Vertex neg[4], Vertex pos[4], int& ncount, int& pcount)
{
	int c[3];
	int i, j;
	Point3d p;
	Normal3d n;
	RGB rgb;
	double t;

	for (i = 0; i < 3; i++)
		c[i] = plane.classify(verts[i].pt());

	pcount = ncount = 0;

	for (i = 0, j = 1; i < 3; i++, j = (i < 2) ? i+1 : 0) {

		switch (c[i]) {
		case -1:
			neg[ncount++] = verts[i];
			break;
		case 0:
			neg[ncount++] = verts[i];
			pos[pcount++] = verts[i];
			break;
		case 1:
			pos[pcount++] = verts[i];
			break;
		}

		if (c[i] * c[j] < 0) {
			p = plane.intersect(verts[i].pt(), verts[j].pt());
			t = (p - verts[i].pt()).norm() /
				(verts[j].pt() - verts[i].pt()).norm();
			n = (1 - t) * verts[i].nr() + t * verts[j].nr();
			n.normalize();
			rgb = (1 - t) * verts[i].rgb() + t * verts[j].rgb();
			neg[ncount++] = Vertex(p, n, rgb);
			pos[pcount++] = Vertex(p, n, rgb);
		}
	}
}

void Object::splitGeom(const Plane& plane, Object& oneg, Object& opos)
{
	Vertex v[3];
	Vertex neg[4], pos[4];
	int ncount, pcount;
	int f, i, j;

	InfArray<Vertex> nverts(num_verts), pverts(num_verts);
	InfArray<Face>   nfaces(num_faces), pfaces(num_faces);

	for (f = 0; f < num_faces; f++) {

		v[0] = verts[faces[f][0]];
		v[1] = verts[faces[f][1]];
		v[2] = verts[faces[f][2]];

		splitFace(v, plane, neg, pos, ncount, pcount);

		if (ncount == 3) {
			// add a face to negative half
			i = nverts.length();
			nverts[i++] = neg[0];
			nverts[i++] = neg[1];
			nverts[i++] = neg[2];
			j = nfaces.length();
			nfaces[j++] = Face(i-3, i-2, i-1, faces[f].rgb());
		} else if (ncount == 4) {
			// add two faces
			i = nverts.length();
			nverts[i++] = neg[0];
			nverts[i++] = neg[1];
			nverts[i++] = neg[2];
			nverts[i++] = neg[3];
			j = nfaces.length();
			nfaces[j++] = Face(i-4, i-3, i-2, faces[f].rgb());
			nfaces[j++] = Face(i-4, i-2, i-1, faces[f].rgb());
		}
		if (pcount == 3) {
			// add a face to positive half
			i = pverts.length();
			pverts[i++] = pos[0];
			pverts[i++] = pos[1];
			pverts[i++] = pos[2];
			j = pfaces.length();
			pfaces[j++] = Face(i-3, i-2, i-1, faces[f].rgb());
		} else if (pcount == 4) {
			// add two faces
			i = pverts.length();
			pverts[i++] = pos[0];
			pverts[i++] = pos[1];
			pverts[i++] = pos[2];
			pverts[i++] = pos[3];
			j = pfaces.length();
			pfaces[j++] = Face(i-4, i-3, i-2, faces[f].rgb());
			pfaces[j++] = Face(i-4, i-2, i-1, faces[f].rgb());
		}
	}

	cerr << "Splitting complete: -(" << nverts.length() << ", "
		<< nfaces.length() << ") and +(" << pverts.length() << ", "
		<< pfaces.length() << ")\n";

	oneg.setGeom(nverts.length(), nverts.getArray(),
				 nfaces.length(), nfaces.getArray());
	opos.setGeom(pverts.length(), pverts.getArray(),
				 pfaces.length(), pfaces.getArray());
}

Boolean Object::split(const Plane& plane, Object*& o1, Object*& o2)
{
	if (!verts) {
		// have to read in the geometry first
		readGeom();
	}

	o1 = new Object;
	o2 = new Object;
	if (!o1 || !o2)
		Error("Object::split: new failed");
	o1->otype = o2->otype = otype;
	o1->fname = strdup(fname);
	o2->fname = strdup(fname);
	if (!o1->fname || !o2->fname)
		Error("Object::split: new failed");

	splitGeom(plane, *o1, *o2);

	if (o1->num_faces <= 0) {
		delete o1;
		o1 = NIL(Object);
	}
	if (o2->num_faces <= 0) {
		delete o2;
		o2 = NIL(Object);
	}

	if (o1 && o2) {

		// Update bounding extents:
		o1->computeExtents();
		o2->computeExtents();

		// Mark as new objects:
		o1->original = o2->original = FALSE;

		// The new objects should inherit all the contstraints their parent
		// had, plus a new constraint should be created between the two.
		ConstrPtr cons, cons1, cons2;
		while (!clist.isEmpty()) {
			cons1 = clist.pop();
			cons2 = new Constraint(*cons1);
			if (cons1->a == this)
				cons1->a = o1, cons2->a = o2;
			else
				cons1->b = o1, cons2->b = o2;
			o1->clist.append(cons1);
			o2->clist.append(cons2);
		}

		cons = new Constraint(o1, o2);
		o1->clist.append(cons);
		o2->clist.append(cons);
		return TRUE;

	} else {
		
		Object *o = (o1) ? o1 : o2;
		o->fname = strdup(fname);
		o->clist = clist;
		for (int i = 0; i < Angles; i++)
			o->extents[i] = extents[i];
		return FALSE;

	}
}

void Object::computeExtents()
{
	register int i, j;
	Vector3d vecs[MAX_ANGLES];

	Assert(Angles > 0);

	if (!verts) {
		// have to read in the geometry first
		readGeom();
	}

	for (i = 0; i < Angles; i++) {
		extents[i].Init();
		double angle = double(i) * M_PI / double(Angles);
		vecs[i].set(cos(angle), 0, sin(angle));
	}	

	for (i = 0; i < num_verts; i++) {
		Vector3d v = verts[i].pt();
		for (j = 0; j < Angles; j++) {
			extents[j] += v | vecs[j];
		}
	}
}

static const Object *currentObj;
static FILE *_fp;

static void writeConstraints(ConstrPtr c)
{
	if (c->a == currentObj)
		fprintf(_fp, " %d", (int)c->b->Id());
	else
		fprintf(_fp, " %d", (int)c->a->Id());
}

void Object::write(FILE *fp)
{
	// assign the object a unique name:
	if (!original) {
		sprintf(string, "%s.%d", fname, id);
		free(fname);
		fname = strdup(string);
		if (fname == NIL(char))
			Error("Object::write: new failed");
	}

	// write out object id and file name:
	fprintf(fp, "%d %s\n", id, fname);

	// write out transformation:
	for (register int i = 0; i < 4; i++)
		fprintf(fp, "%g %g %g %g\n", transform(i,0), transform(i,1),
				transform(i,2), transform(i,3));

	// write out constraints:
	currentObj = this;
	_fp = fp;
	fprintf(fp, "%d", clist.length());
	clist.apply(writeConstraints);
	fprintf(fp, "\n");

	// if this is an object created while constructing the BSP-tree,
	// it's geometry must be written into a file:
	if (!original) {
		Assert(num_faces > 0);
		writeGeom();
	}
}


