/******************************************************************************
 *
 * $Source: /homes/sgi/danix/bsp/RCS/object.h,v $
 * $Author: danix $
 * $Locker:  $
 *
 * $Log: object.h,v $
 * Revision 1.3  1995/12/14  23:20:34  danix
 * Switched to new Vertex and Face classes;
 * Added CNOFF objects.
 *
 * Revision 1.2  1995/12/14  00:20:47  danix
 * *** empty log message ***
 *
 * Revision 1.1  1995/09/25  18:17:34  danix
 * Initial revision
 *
 *****************************************************************************/

#ifndef OBJECT_H
#define OBJECT_H

#include <stdlib.h>
#include <values.h>
#include <list.h>
#include <vertex.h>
#include <face.h>
#include <plane.h>

#define MAX_ANGLES 8

class Object;

class Constraint {
public:
	Object *a, *b;
	Constraint(Object* _a = 0, Object* _b = 0) : a(_a), b(_b)   {}
	Constraint(const Constraint& c)            : a(c.a), b(c.b) {}    
	~Constraint()                                               {}
};

typedef Constraint * ConstrPtr;
typedef List<ConstrPtr> ConstrList;

class Extent {
public:
	float min, max;
	void Init()  { min = MAXFLOAT; max = -MAXFLOAT; }
	Extent()     { Init(); }
	void operator+=(float val) {
		min = (val < min) ? val : min;
		max = (val > max) ? val : max;
	}
};

typedef enum {tUNKNOWN, tOFF, tCOFF, tNOFF, tCNOFF} ObjType;

class Object {
public:
	unsigned long Id() const             { return id; }
	char *fileName() const               { return fname; }
	const Extent& extent(int i) const    { return extents[i]; }
	int numFaces() const                 { return num_faces; }
	Boolean operator<=(const Plane&);
	Boolean operator>=(const Plane&);
	Boolean split(const Plane&, Object*&, Object*&);
	void splitGeom(const Plane&, Object&, Object&);
	void setGeom(int nv, Vertex *v, int nf, Face *f) {
		num_verts = nv;
		verts = v;
		num_faces = nf;
		faces = f;
	}
	void readGeom();
	void writeGeom() const;
	void computeExtents();
	void write(FILE *fp);
	Object();
	Object(FILE *fp);
	~Object();
	static unsigned short Angles;
	static unsigned long count;
private:
	unsigned long id;
	Boolean original;
	char *fname;
	ObjType otype;
	Matrix4x4 transform;
	Extent extents[MAX_ANGLES];
	int num_verts, num_faces;
	Vertex *verts;
	Face *faces;
	ConstrList clist;
	void init();
};

inline void Object::init()
// Common initialization for different constructors
{
	id = ++Object::count;
	original = TRUE;
	fname = NIL(char);
	otype = tUNKNOWN;
	transform = Matrix4x4::I();
	num_verts = num_faces = 0;
	verts = NIL(Vertex);
	faces = NIL(Face);
}

inline Object::Object()
{
	init();
}

typedef Object * ObjectPtr;

extern void setNumAngles(int);

#endif /* OBJECT_H */
