/******************************************************************************
 *
 * $Source: /homes/sgi/danix/bsp/RCS/vertex.h,v $
 * $Author: danix $
 * $Locker:  $
 *
 * $Log: vertex.h,v $
 * Revision 1.1  1995/12/14  23:20:17  danix
 * Initial revision
 *
 *****************************************************************************/

#ifndef VERTEX_H
#define VERTEX_H

#include <stdio.h>
#include <geometry.h>
#include <color.h>

class Vertex {
public:
	// Constructors:
	Vertex() : a(1)                                                     {}
	Vertex(double *xyz) : p(xyz[0], xyz[1], xyz[2]), a(1)               {}
	Vertex(const Point3d& pp, const Normal3d& nn) : p(pp), n(nn), a(1)  {}
	Vertex(const Point3d& pp) : p(pp), a(1)                             {}
	Vertex(double x, double y, double z) : p(x, y, z), a(1)             {}
	Vertex(double *xyz, double *nor)
		: p(xyz[0], xyz[1], xyz[2]), n(nor[0], nor[1], nor[2]), a(1)    {}
	Vertex(double x, double y, double z, const Normal3d& nn)
		: p(x, y, z), n(nn), a(1)                                       {}
	Vertex(const Point3d& pp, const Normal3d& nn, const RGB& rgb)
		: p(pp), n(nn), color(rgb), a(1)                                {}

	// Public access functions:
	void point(float x, float y, float z)        { p = Point3d(x, y, z); }
	Point3d& pt()                                            { return p; }
	const Point3d& pt() const                                { return p; }
	float  x() const                                      { return p[0]; }
	float  y() const                                      { return p[1]; }
	float  z() const                                      { return p[2]; }
	void normal(float x, float y, float z)      { n = Normal3d(x, y, z); }
	Normal3d& nr()                                           { return n; }
	const Normal3d& nr() const                               { return n; }
	float nx() const                                      { return n[0]; }
	float ny() const                                      { return n[1]; }
	float nz() const                                      { return n[2]; }
	void rgb(float r, float g, float b)          { color = RGB(r, g, b); }
	RGB& rgb()                                           { return color; }
	const RGB& rgb() const                               { return color; }
	float  r() const                                  { return color[0]; }
	float  g() const                                  { return color[1]; }
	float  b() const                                  { return color[2]; }
	float& alpha()                                           { return a; }
	float  alpha() const                                     { return a; }
	void alpha(float _a)                                       { a = _a; }
private:
	Point3d p;
	Normal3d n;
	RGB color;
	float a;
};


inline int printVertex(FILE *fp, const Vertex& v)
{
	int n = fprintf(fp, "%g %g %g  %g %g %g  %g %g %g %g\n",
					v.x(), v.y(), v.z(), v.nx(), v.ny(), v.nz(),
					v.r(), v.g(), v.b(), v.alpha());
	return (n > 0);
}

#endif
