//============================================================================
//
//		callbacks.c
// 
//
//		Misc Callbacks
//
//			Exit
//			Reset
//
//			Choose Superman
//			Choose Trackball
//			Toggle Bounding Boxes
//
//			Update Render
//			Grab   Image
//			Paste  Image
//
//
//
//			Jonathan Shade
//
//============================================================================

#include <stdlib.h>
#include <stdio.h>

#include <Xm/Xm.h>
#include <X11/keysym.h>

#include <GL/glx.h>
#ifdef __sgi
#include <GL/GLwMDrawA.h>
#else
#include <X11/GLw/GLwDrawA.h>
#endif


#include <ui.h>
#include "state.h"
#include "input.h"
#include "callbacks.h"



//----------------------------------------------------------------------------
//
//		exit_cb
//
//----------------------------------------------------------------------------
void
exit_cb( Widget, XtPointer, XtPointer )
	{
	exit(0);
	}

//----------------------------------------------------------------------------
//
//		update_render
//
//----------------------------------------------------------------------------
void
update_render( Widget, XtPointer client_data, XtPointer )
	{
	uiPostRender( (uiRenderWindow *) client_data );
	}

//----------------------------------------------------------------------------
//
//		expose_callback:
//
//			This function is called when the X window is exposed.
//
//----------------------------------------------------------------------------
void
expose_callback( Widget, XtPointer client_data, XtPointer )
	{
	uiPostRender( (uiRenderWindow *) client_data );
	}

//----------------------------------------------------------------------------
//
//		resize_callback:
//
//			This function is called when the X window is resized.
//
//----------------------------------------------------------------------------
void
resize_callback( Widget widget, XtPointer client_data, XtPointer )
    {
	Dimension width;
	Dimension height;

	XtVaGetValues(  widget,
					XmNwidth,  &width,
					XmNheight, &height,
					NULL );

	((uiRenderWindow *) client_data)->dirtyViewport		= 1;
	((uiRenderWindow *) client_data)->glxarea_width		= width;
	((uiRenderWindow *) client_data)->glxarea_height	= height;
    }


