//============================================================================
//
//		input.c
//
//
//			Jonathan Shade
//
//============================================================================

#include <stdio.h>
#include <Xm/Xm.h>
#include <X11/keysym.h>

#include <GL/glx.h>

#include "../viewer/Camera.h"
#include "../viewer/Trackball.h"
#include "../viewer/ui.h"
#include "state.h"
#include "input.h"

#define	DRIVE_AHEAD	  0
#define DRIVE_REVERSE 1
#define DRIVE_LEFT	  2
#define DRIVE_RIGHT   3

//
//	Globals for the DRIVE work proc.
//	This is messy, but quick.
//
static	int KeepDriving	= 0;
static	int DriveDirection;

Boolean	Drive( XtPointer client_data )
	{
	uiRenderWindow 	*ren_win  = (uiRenderWindow *) client_data;
	AppState*		app_state = (AppState *) ren_win->uiData;
	GraphicsState	*gr_state = (GraphicsState *) ren_win->grData;

	Camera*	camera = gr_state->camera;

	switch( DriveDirection )
		{		
		case DRIVE_AHEAD:
			//	Camera Dolly
			camera->Dolly( 1.0 );
			camera->Update();
			break;

		case DRIVE_REVERSE:
			//	Camera Dolly
			camera->Dolly( -1.0 );
			camera->Update();
			break;

		case DRIVE_LEFT:
			//	Camera Rotate Left
			camera->Rotate( 0.02 );
			camera->Update();
			break;

		case DRIVE_RIGHT:
			//	Camera Rotate Right
			camera->Rotate( -0.02 );
			camera->Update();
			break;
		};

	//
	//	Manually render the window, and 
	//	possibly update the skycam.
	//
	uiRenderImmediately( ren_win );

	if( app_state->activeWindowList[1] )  // this 1 thing here is bad
		{
		ren_win = app_state->uiWindowList[1];
		uiRenderImmediately( ren_win );
		}

	//
	//	Returning False keeps the work
	//	registered and running.
	//
	if( KeepDriving > 0)
		return False;
	else
		return True;
	}

//----------------------------------------------------------------------------
//
//		input_drive:
//
//			This function does the driving stuff.
//
//----------------------------------------------------------------------------
void	input_drive( uiRenderWindow *ren_win, XtPointer client_data, XtPointer call_data )
	{
	static int		driveProcId;
	AppState		*app_state = (AppState*) ren_win->uiData;

    XmDrawingAreaCallbackStruct *cd = (XmDrawingAreaCallbackStruct*)call_data;

	//++++++++++++++++++++++++++++++++++++++++
	//
	//			Button Press
	//
	//++++++++++++++++++++++++++++++++++++++++
	if ( cd->event->xany.type == ButtonPress )
		{	
		switch( cd->event->xbutton.button )
			{

			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			//	Left Mouse Button
			//
			//	Turn Left
			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			case Button1:
				DriveDirection	= DRIVE_LEFT;
				KeepDriving		+= 1;

				driveProcId = XtAppAddWorkProc(
									app_state->uiWindowList[0]->app,
									Drive,
									client_data );			
				break;

			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			//	Middle Mouse Button
			//
			//	Drive Straight Ahead
			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			case Button2:
				if( cd->event->xkey.state == ControlMask )
					{
					DriveDirection = DRIVE_REVERSE;
					fprintf(stderr,"Reverse\n");
					}
				else
					DriveDirection	= DRIVE_AHEAD;
				KeepDriving		+= 1;

				driveProcId = XtAppAddWorkProc(
									app_state->uiWindowList[0]->app,
									Drive,
									client_data );			
				break;

			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			//	Right Mouse Button
			//
			//	Turn Right
			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			case Button3:
				DriveDirection	= DRIVE_RIGHT;
				KeepDriving		+= 1;

				driveProcId = XtAppAddWorkProc(
									app_state->uiWindowList[0]->app,
									Drive,
									client_data );			
				break;

			default:
				break;
			};

		} // Button Press 


	//++++++++++++++++++++++++++++++++++++++++++
	//
	//			Button Release
	//
	//++++++++++++++++++++++++++++++++++++++++++
	if ( cd->event->xany.type == ButtonRelease )
		{
		switch( cd->event->xbutton.button )
			{
			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			case Button1:

				KeepDriving -= 1;
				// XtRemoveWorkProc ??
				break;

			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			case Button2:

				KeepDriving -= 1;
				break;

			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			case Button3:
					{
					KeepDriving -= 1;
					}
				break;

			default:
				break;
			};

		} // Button Release

	return;
	}


//----------------------------------------------------------------------------
//
//		input_trackball:
//
//			This function does the trackball stuff.
//
//----------------------------------------------------------------------------
static double	Delta[3]	= { 2.0, 2.0, 2.0 };

void	input_trackball( uiRenderWindow *ren_win, XtPointer call_data )
	{
    static int			scrX,scrY;
    static int			newX,newY;
	static int			width,height;
	static double		wcX,wcY;
	static double		owcX,owcY;
	static Trackball	trackball;

	static GraphicsState* gr_state  = (GraphicsState *) ren_win->grData;

	static Camera*	camera = gr_state->camera;
	static Camera	down_cam;

    XmDrawingAreaCallbackStruct *cd = (XmDrawingAreaCallbackStruct*)call_data;

	Matrix4d	tmp_matrix;
	Vector4d	tmp_quat;


	width	= ren_win->glxarea_width;
	height	= ren_win->glxarea_height;


	//++++++++++++++++++++++++++++++++++++++++
	//
	//	Button Press
	//
	//++++++++++++++++++++++++++++++++++++++++
	if ( cd->event->xany.type == ButtonPress )
		{
		scrX = cd->event->xbutton.x;
		scrY = cd->event->xbutton.y;

		wcX = 2.0 * ((double) scrX / (double) width) - 1.0;
		wcY = 2.0 * (1.0 - (double) scrY / (double) height) - 1.0;

		owcX = wcX;
		owcY = wcY;

		switch( cd->event->xbutton.button )
			{
			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			//	Left Mouse Button
			//
			//	Initiate the trackball drag 
			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			case Button1:
				//
				//	Trackball
				//
				trackball.initBall( camera->getQuat() );
				trackball.beginDrag( wcX, wcY );
					
				//
				//	Cache camera at mouse click down
				//					
				down_cam = *camera;

				down_cam.Print();
				break;

			default:
				break;
			};

		} // Button Press


	//+++++++++++++++++++++++++++++++++++++++++
	//
	//	Button Motion
	//
	//+++++++++++++++++++++++++++++++++++++++++
    if ( cd->event->xany.type == MotionNotify )
		{
		newX = cd->event->xbutton.x;
		newY = cd->event->xbutton.y;
	
		switch( cd->event->xbutton.state )
			{
			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			//	Left Mouse Button
			//
			//	Compute the trackball drag
			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			case Button1Mask:

				//	
				//	-1 < wcX < 1
				//	-1 < wcY < 1
				//
				wcX = 2.0 * ((double) newX / (double) width) - 1.0;
				wcY = 2.0 * (1.0 - (double) newY / (double) height) - 1.0;

				//
				// This little hack reflects the current mouse position
				// about the point where the drag started. This gives an
				// opposite motion to what would be intuitive for a trackball.
				// We are doing this because we want to rotate the camera about
				// the scene, not rotate the scene in front of the camera.
				//
				//trackball.updateDrag( 2.0*owcX-wcX, 2.0*owcY-wcY );
				//
				trackball.updateDrag( wcX, wcY );				

				camera->setModelTransform( trackball.getMatrix() );
				camera->setQuat( trackball.getQuat() );

				camera->Update();
				break;

			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			//	Middle Mouse Button
			//
			//		Dolly the Camera
			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			case Button2Mask:

				if (  newX > scrX )		camera->Move( 0.0, 0.0, Delta[2] );
				else if ( newX < scrX )	camera->Move( 0.0, 0.0, -Delta[2] );
					
				camera->Update();
				break;

			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			//	Right Mouse Button
			//
			//		Translate the camera
			//~~~~~~~~~~~~~~~~~~~~~~~~~~
			case Button3Mask:

				if ( newX > scrX )		camera->Move( -Delta[0], 0.0, 0.0 );
				else if ( newX < scrX )	camera->Move( Delta[0], 0.0, 0.0 );
					
				if (  newY > scrY )		camera->Move( 0.0, Delta[1], 0.0 );
				else if ( newY < scrY )	camera->Move( 0.0, -Delta[1], 0.0 );

				camera->Update();
				break;
		
			default:
				break;
			};

		} // Button Motion

	scrX = newX;
	scrY = newY;
	
	return;
	}





//----------------------------------------------------------------------------
//
//		input_callback:
//
//			This function is called when the X window receives input.
//
//----------------------------------------------------------------------------
void	input_callback( Widget, XtPointer client_data, XtPointer call_data )
	{
	uiRenderWindow	*ren_win   = (uiRenderWindow *) client_data;
	AppState		*app_state = (AppState *) ren_win->uiData;

	//
	//	Ignore Skycam
	//
	if(  ren_win->window_id == 1 )
		return;

	switch( app_state->fly_mode )
		{
		case INPUT_FLYMODE_TRACKBALL:
		
			input_trackball( ren_win, call_data );
			break;

		case INPUT_FLYMODE_DRIVE:
		
			input_drive( ren_win, client_data, call_data );
			break;

		default:
			break;
		};

	//
	//	Render the window just changed, and
	//	re-render the skycam & manip windows.
	//
	uiPostRender( ren_win );
	}



