//============================================================================
//
//		main.c
//
//
//			Jonathan Shade
//
//============================================================================

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeBG.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <X11/keysym.h>

#include <GL/glx.h>
#ifdef __sgi
#include <GL/GLwMDrawA.h>
#else
#include <X11/GLw/GLwDrawA.h>
#endif

#include <ui.h>
#include <glx_resources.h>
#include <Camera.h>
#include "callbacks.h"
#include "menu.h"
#include "state.h"
#include "scene.h"



/*
 *	Motif options
 */
static String   fallbackResources[] = {
#ifdef __sgi
	"*fontList:			-sgi-sorority-*-*-*-*-*-120-*-*-*-*-*-*",
#endif
	"*sgiMode:			true",
	"*useSchemes:		all",
	"*glxarea*width:	300",
	"*glxarea*height:	300",
	NULL
};



//----------------------------------------------------------------------------
//
//		List of "Visuals" to try when creating
//		the windows of this application
//
//
//----------------------------------------------------------------------------
uiVisualList	visual_list_plain[] =
								{
								VR0,VR0_desc,
								VR1,VR1_desc,
								VR_IM3,VR_IM3_desc,
								VR2,VR2_desc,
								VR3,VR3_desc,
								VR4,VR4_desc,
								NULL,
								};

uiVisualList	visual_list_multisample[] =
								{
								VR_RE1,VR_RE1_desc,
								VR_RE2,VR_RE2_desc,
								VR0,VR0_desc,
								VR1,VR1_desc,
								VR2,VR2_desc,
								VR3,VR3_desc,
								NULL,
								};

//----------------------------------------------------------------------------
//
//	app_init:
//
//		Sets up the X windows stuff, and launches into 
//		the main loop of the program.
//
//----------------------------------------------------------------------------
void	app_init(int argc, char *argv[], AppState & app_state)
    {
    Arg             args[20];

	Widget  	    toplevel;
	Widget  	    topform;
	Dimension		width,height;

	Display			*dpy;

    XtAppContext    app;

	XtWorkProcId	renderWorkId;

	uiRenderWindow	compute_win;

	Camera			camera;

	int				vis_index;

	app_state.fly_mode = 0;
	app_state.activeWindowList[0] = 1;

	//
	//	Hack for Animation
	//
	app_state.uiWindowList[0] = &compute_win;

	app_state.anim_state.win_list[0] = 0;
	app_state.anim_state.work_proc_id = -1;

	//
	//	Graphics State
	//
	app_state.gr_state.camera			= &camera;

	//
	//	Create the Application
	//
	//
    toplevel	= XtAppInitialize(&app, "Compute Cost",
					NULL, 0, &argc, argv, fallbackResources, NULL, 0);
    dpy			= XtDisplay(toplevel);

	//
	//	Create a from to manage the children of the toplevel "shell".
	//
	//
//	topform = XmCreateForm( toplevel, "topform", NULL, 0 );
//	XtManageChild( topform ); 	


	//------------------------------------------------------------
	//
	//	Create the window for geometry rendering
	//
	//		This window is the "main" app window.
	//
	//------------------------------------------------------------
	XtSetArg(args[0],XmNx, 0);
	XtSetArg(args[1],XmNy, 0);
	XtSetArg(args[2],XmNtitle, "Computeulator" );

//	compute_win.parent			= XmCreateFormDialog(topform, "main_w_compute",
//														args, 3); 
	compute_win.parent			= toplevel;
									
	compute_win.shell			= toplevel; 
	XtSetValues( compute_win.shell,args,3 );

	compute_win.app				= app; 
	compute_win.dpy				= dpy; 
	compute_win.draw				= scene_Draw_Compute;
	compute_win.init				= scene_Init_GL;
	compute_win.needsInit			= 1;
	compute_win.dirtyViewport		= 1;
	compute_win.onRenderList		= 0;
	compute_win.nextRenderWindow	= NULL;

	compute_win.uiData			= (AppState *)		&app_state;

	compute_win.glxarea_width	= 1280;
	compute_win.glxarea_height	= 960;

	compute_callbacks.exposeData	= &compute_win;
	compute_callbacks.resizeData	= &compute_win;
	compute_callbacks.inputData	= &compute_win;

	//	
	//		Button Bar
	//
	//	"Start"
	//	"Exit"
	//
	compute_ht_bar[0].callback_data = &compute_win;
	compute_ht_bar[1].callback_data = &compute_win;


	vis_index = ui_Create_GL_Rendering_Window( &compute_win, compute_callbacks,
												compute_menu, compute_ht_bar,
												visual_list_plain,
												None );

fprintf(stderr,"Compute Win:\t%s\n", visual_list_plain[vis_index].descstr );

	if ( app_state.activeWindowList[compute_win.window_id] )
		{
		//XtManageChild( compute_win.parent );
		XtPopup( compute_win.shell, XtGrabNone );

		XtVaGetValues(compute_win.glxarea,
						XmNwidth,  &width,
					 	XmNheight, &height,
					 	NULL );

		compute_win.glxarea_height = height;
		compute_win.glxarea_width  = width;
		}


	//
	//  Realize the toplevel application widget
	//
	//XtRealizeWidget(toplevel);


	//
	//	Init scene database
	//
	//	Make sure it is the Image window that actually
	//	inits the db.
	//
	scene_Init_DB(  compute_win.window_id  );

	renderWorkId = XtAppAddWorkProc( app, uiRenderIdleProc, NULL );

	XtAppMainLoop(app);
    }


//----------------------------------------------------------------------------
//
//	main:
//
//		Process the command line arguments
//
//		-h			List options
//
//----------------------------------------------------------------------------

void main(int argc, char *argv[] )
	{
	AppState	app_state;

	app_state.gr_state.multisample	= 0;
	app_state.gr_state.antialias		= 0;
	app_state.gr_state.texturing		= 0;
	app_state.gr_state.geometry		= 0;
	app_state.gr_state.tex_mem_size	= 4;
	app_state.gr_state.output_header	= 0;

	for (int i = 0; i < argc; i++)
		{
		if      (strcmp(argv[i], "-aa") == 0)
			app_state.gr_state.antialias = 1;

		else if (strcmp(argv[i], "-ms") == 0)
			app_state.gr_state.multisample = 1;

		else if (strcmp(argv[i], "-t") == 0)
			app_state.gr_state.texturing = 1;

		else if (strcmp(argv[i], "-g") == 0)
			app_state.gr_state.geometry = 1;

		else if (strcmp(argv[i], "-s") == 0)
			{
			if ( i != argc-1 )
				app_state.gr_state.tex_mem_size = atoi( argv[i+1] );
			else
				{
				fprintf(stderr,"\nNo Texture Memory Size Given\n");
				exit(0);
				}
			}
		else if (strcmp(argv[i], "-c") == 0)
			app_state.gr_state.output_header = 1;
		
		else if (strcmp(argv[i], "-h") == 0)
			{
			fprintf(stderr,"\n");
			fprintf(stderr,"\n");

			fprintf(stderr,"\t-aa\tantialiasing\t(false)\n");
			fprintf(stderr,"\t-ms\tmultisampling\t(false)\n");
			fprintf(stderr,"\t-t\ttest texturing\t(false)\n");
			fprintf(stderr,"\t-g\ttest geometry\t(false)\n");
			fprintf(stderr,"\t-s #\tsize of texture memory\t(4 MB)\n");
			fprintf(stderr,"\t-c\toutput header file\t(false)\n");

			fprintf(stderr,"\n");
			exit(0);
			}
		}

	app_init( argc,argv,app_state );
	}
