
//============================================================================
//
//	Scene.c
//
//		This file contains all of the "drawing" routines
//		that are called to update the rendering in an
//		OpenGL window of the Walkthru app.
//
//		scene_Init
//		scene_Init_DB
//
//============================================================================

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/DrawingA.h>
#include <X11/keysym.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>

#include "scene.h"
#include "state.h"

#include <ui.h>
#include <Geometry.h>
#include <BoundBox.h>
#include <Camera.h>
#include <VisualObject.h>
#include <TriangleList.h>
#include <Billboard.h>
#include <timing.h>
#include <glutstroke.h>
extern void* glutStrokeRoman;

//----------------------------------------------------------------------------
//
//	Variables Global to this file
//
//----------------------------------------------------------------------------

GLfloat		color_clear[3]	= {  0.0, 0.0, 0.0  };
GLfloat		light_col[3]	= {  1.0, 1.0, 1.0  };
GLfloat		light_pos0[4]	= { -1.0, 1.0, 1.0, 0.0 };

int		g_db_valid		 = 0;
int		g_db_window_id	 = -1;
int		g_do_multisample = 0;
double	g_radius		 = 300;

GLubyte *Texture;


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#define NUM_GEOM_SIZES	100
#define NUM_IMAGE_SIZES 11
#define NUM_REPS		10
#define NUM_DECADES		2

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

int	draw_geom_stats[NUM_GEOM_SIZES*NUM_DECADES][3];
int	geom_size_array[NUM_GEOM_SIZES*NUM_DECADES];

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

double	g_quad[4][3];
int	image_size_array[NUM_IMAGE_SIZES] = {
						256,
						512,
						1024,
						2048,
						4096,
						8192,
					    16384,
			            32768,
						65536,
						131072,
						262144 };

int image_dim_array[NUM_IMAGE_SIZES][2] = {
						16,16,
						32,16,
						32,32,
						64,32,
						64,64,
						128,64,
						128,128,
						256,128,
						256,256,
						512,256,
						512,512 };

int	draw_image_stats[NUM_IMAGE_SIZES][3];

int	create_image_stats[NUM_IMAGE_SIZES][3];


//----------------------------------------------------------------------------
//
//		scene_Init_DB
//
//			Read the object Hierarchy from disk.
//
//----------------------------------------------------------------------------
void
scene_Init_DB( int window_id )
	{
	//
	//	Read a hierarchy from disk
	//
	// sprintf( g_db_filename,"%s",db_filename);
	g_db_window_id = window_id;
	}

void
foo_init_db()
	{
	fprintf(stderr,"Initializing Database ... ");

	g_quad[0][0] = 0.0;
	g_quad[0][1] = 0.0;
	g_quad[0][2] = 0.0;

	g_quad[1][0] = 600.0;
	g_quad[1][1] = 0.0;
	g_quad[1][2] = 0.0;

	g_quad[2][0] = 600.0;
	g_quad[2][1] = 375.0;
	g_quad[2][2] = 0.0;

	g_quad[3][0] = 0.0;
	g_quad[3][1] = 375.0;
	g_quad[3][2] = 0.0;

	Texture = new GLubyte [1280*1024*4];

	fprintf(stderr,"Done.\n");
	}
 
//----------------------------------------------------------------------------
//
//		scene_Init_GL
//
//			Setup OpenGL state
//
//----------------------------------------------------------------------------
void
scene_Init_GL( uiRenderWindow *ren_win )
	{
	static AppState *app_state = (AppState*) ren_win->uiData;
	static GraphicsState *gr_state = &(app_state->gr_state);
	
	//
	//	Look for extensions
	//
	const GLubyte *extensions = glGetString( GL_EXTENSIONS );
	char *do_multisample = strstr((const char*)extensions,"SGIS_multisample");
	if( do_multisample != NULL )
		g_do_multisample = 1;

	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
//	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
//	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );

	glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

	glEnable(GL_LIGHT0);
	glEnable(GL_NORMALIZE);
	glEnable(GL_DEPTH_TEST);

	glDisable(GL_DITHER);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);

	glClearDepth(1.0);
	glDepthFunc(GL_LEQUAL);
//	glCullFace(GL_BACK);

	glLightModeli( GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE );
	glLightfv( GL_LIGHT0, GL_POSITION, light_pos0 );

	if( ren_win->window_id == g_db_window_id )
		{ 
		if( !g_db_valid )
			{
			foo_init_db();
			g_db_valid = 1;

			//
			//	Init the camera
			//
			// angle,aspect,near,far
			gr_state->camera->initFrustum( 50.0, 1.33, 0.1, g_radius*5 );
			gr_state->camera->Reset();
			gr_state->camera->Move( 0,0,3*g_radius );
			gr_state->camera->Update();
			}
		}

	return;
	}
	

//----------------------------------------------------------------------------
//
//		scene_Draw_Compute
//
//			Rendering Routine for the "Compute Costs" window
//
//----------------------------------------------------------------------------
void
scene_Draw_Compute(Widget w, Display *dpy, int *dirtyViewport,
					void *ui_data )
	{
	static AppState *app_state = (AppState *) ui_data;
	static GraphicsState *gr_state = &(app_state->gr_state);
	static Dimension	width, height;

	if( !g_db_valid )	return;

	if (*dirtyViewport)
		{
    	XtVaGetValues(w, XmNwidth, &width, XmNheight, &height, NULL);
    	*dirtyViewport = 0;
		glViewport(0, 0, (GLint) width, (GLint) height);
		}

	glClearColor(color_clear[0], color_clear[1], color_clear[2], 1.0);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	gr_state->camera->setupGLProjectionMoveCamera();
	
	int j;

	if( gr_state->geometry )
		{
		scene_Measure_Geometry( w, dpy, gr_state->camera );

		if( gr_state->output_header )
			{
			fprintf(stdout,"\nint cost_draw_geom[%d][2] = {\n",NUM_GEOM_SIZES*NUM_DECADES);
			for( j=0 ; j<NUM_GEOM_SIZES*NUM_DECADES ; j++ )
				{
				fprintf(stdout,"%8d,\t%15d",geom_size_array[j],
											draw_geom_stats[j][2]);

				if( j == NUM_IMAGE_SIZES-1 )	fprintf(stdout,"\n");
				else							fprintf(stdout,",\n");
				}
			fprintf(stdout,"};\n\n");
			}
		else
			{
			fprintf(stdout,"\nDraw Geometry\n");
			fprintf(stdout,"-------------\n");
			for( j=0 ; j<NUM_GEOM_SIZES*NUM_DECADES ; j++ )
				fprintf(stdout,"%8d:\t%15d\n",geom_size_array[j],
											  draw_geom_stats[j][2]);
			}

		}

	if( gr_state->texturing )
		{
		scene_Measure_Texturing( w, dpy, gr_state->camera );

		if( gr_state->output_header )
			{
			fprintf(stdout,"\nint cost_draw_image[%d][2] = {\n",NUM_IMAGE_SIZES);
			for( j=0 ; j<NUM_IMAGE_SIZES ; j++ )
				{
				fprintf(stdout,"%8d,\t%15d",image_size_array[j],
											  draw_image_stats[j][2]);
				if( j == NUM_IMAGE_SIZES-1 )	fprintf(stdout,"\n");
				else							fprintf(stdout,",\n");
				}
			fprintf(stdout,"};\n\n");

			fprintf(stdout,"\nint cost_create_image[%d][2] = {\n",NUM_IMAGE_SIZES);
			for( j=0 ; j<NUM_IMAGE_SIZES ; j++ )
				{
				fprintf(stdout,"%8d,\t%15d",image_size_array[j],
										  create_image_stats[j][2]);
				if( j == NUM_IMAGE_SIZES-1 )	fprintf(stdout,"\n");
				else							fprintf(stdout,",\n");
				}
			fprintf(stdout,"};\n\n");
			}
		else
			{
			fprintf(stdout,"\nDraw Image\n");
			fprintf(stdout,"-------------\n");
			for( j=0 ; j<NUM_IMAGE_SIZES ; j++ )
				fprintf(stdout,"%8d:\t%15d\n",image_size_array[j],
											  draw_image_stats[j][2]);

			fprintf(stdout,"\nCreate Image\n");
			fprintf(stdout,"-------------\n");
			for( j=0 ; j<NUM_IMAGE_SIZES ; j++ )
				fprintf(stdout,"%8d:\t%15d\n",image_size_array[j],
											  create_image_stats[j][2]);
			}
		}
	exit(0);
	}

//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
void
scene_Measure_Geometry( Widget w, Display *dpy, Camera * camera )
	{
	static Timer		draw_geom_timer;
	int size = 0;

	//
	//	Initialize Geom Sizes Array
	//	
	for( int i=0 ; i<NUM_DECADES ; i++ )
		{
		for( int j=0 ; j<NUM_GEOM_SIZES ; j++ )
			{
			if( i==0 )
				size = (j+1)*( (100000)/NUM_GEOM_SIZES );
			if( i==1 )
				size = (j+1)*( (900000)/NUM_GEOM_SIZES ) + 100000;

			geom_size_array[ i*NUM_GEOM_SIZES + j ] = size;	
			}
		}

	fprintf(stderr,"Benchmarking Geometry\n");
	for( i=0 ; i<NUM_GEOM_SIZES*NUM_DECADES ; i++ )
		{
		for( int j=0 ; j<NUM_REPS ; j++ )
			{
			size = geom_size_array[ i ];
		
			glMatrixMode( GL_MODELVIEW );
			glPushMatrix();
			glLoadIdentity();

			glClearColor( 0.0, 0.0, 0.0, 0.0 );
			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
			Point3d p =  camera->getModelPosition();
			glTranslated( p[0], p[1], p[2] );
			glMultMatrixd( camera->getModelTransform() );

			glTranslated( -300, -187.5, 0 ); 


			//
			//	Draw Geometry
			//
			glEnable(GL_LIGHTING);
			if( g_do_multisample )	glEnable(GL_MULTISAMPLE_SGIS);
			RESET_TIMER( draw_geom_timer );
			START_TIMER( draw_geom_timer );

				glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
				glEnable(GL_COLOR_MATERIAL);
				glNormal3d( 0, 0, 1 );
				glColor3d( 1, 1, 1 );

				glBegin( GL_TRIANGLES );
				int row,col;
				for( int l=0 ; l<size/1000 ; l++ )
					for( int k=0 ; k<1000 ; k++ )
					{
					row = k/40;
					col = k%40;
	
					glVertex3d( col*15,      row*15,       0 );
					glVertex3d( col*15 + 14, row*15,       0 );
					glVertex3d( col*15,      row*15 + 14,  0 );
					}
				glEnd();
				glFinish();
			STOP_TIMER( draw_geom_timer );
			if( g_do_multisample )	glDisable(GL_MULTISAMPLE_SGIS);
			glDisable(GL_LIGHTING);
		
			glPopMatrix();
			glXSwapBuffers(dpy, XtWindow(w));

			draw_geom_stats[i][0] += calc_usec_time( draw_geom_timer );
			draw_geom_stats[i][1] += 1;

//fprintf(stderr,"%7d:\t%7d\n",size,calc_usec_time( draw_geom_timer ));

			}
		}

	for( int j=0 ; j<NUM_GEOM_SIZES*NUM_DECADES ; j++ )
		{
		if( draw_geom_stats[j][1] != 0 )
			draw_geom_stats[j][2] =
				draw_geom_stats[j][0]/draw_geom_stats[j][1];
		else
			draw_geom_stats[j][2] = 0;
		}
}


//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
void
scene_Measure_Texturing( Widget w, Display *dpy, Camera * camera )
	{	
	static Timer		draw_image_timer,create_image_timer;

	fprintf(stderr,"Benchmarking Texture Mapping\n");

	//
	//	Set up Camera
	//
	glMatrixMode( GL_MODELVIEW );

		glPushMatrix();
		glLoadIdentity();

		glClearColor( 0.5, 0.5, 0.5, 0.0 );
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		Point3d p =  camera->getModelPosition();
		glTranslated( p[0], p[1], p[2] );
		glMultMatrixd( camera->getModelTransform() );
		glTranslated( -300, -187.5, 0 ); 

	glDisable(GL_LIGHTING);

	//
	//	Time Pixel Read
	//
	GLuint textureID[NUM_IMAGE_SIZES];

	glGenTexturesEXT( NUM_IMAGE_SIZES, textureID );

//	for( int i=0 ; i<NUM_IMAGE_SIZES ; i++ )
	for( int i=0 ; i<1 ; i++ )
		{
		for( int j=0 ; j<100 ; j++ )
			{		
			//
			//	Create Image
			//
			RESET_TIMER( create_image_timer );
			START_TIMER( create_image_timer );

			glEnable( GL_TEXTURE_2D );


			glBindTextureEXT( GL_TEXTURE_2D, textureID[i] );
	        glTexParameterf( GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_CLAMP );
		    glTexParameterf( GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_CLAMP );
			glTexParameterf( GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_NEAREST );
			glTexParameterf( GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_NEAREST );

			glCopyTexImage2DEXT( GL_TEXTURE_2D, 0, GL_RGBA, 0, 0,
								image_dim_array[i][0],image_dim_array[i][1],0);
			glFinish();

			STOP_TIMER( create_image_timer );

			{
			GLenum error_code;
			while((error_code=glGetError()) != GL_NO_ERROR)
				fprintf(stderr,"gl error: %s\n",gluErrorString(error_code));
			}

			create_image_stats[i][0] += calc_usec_time( create_image_timer );
			create_image_stats[i][1] += 1;
			}
		}
			
	//
	//	Texture Map draw.
	//
	glClearColor(color_clear[0],color_clear[1],color_clear[2],1.0);

	for( i=0 ; i<NUM_IMAGE_SIZES ; i++ )
//	for( i=0 ; i<1 ; i++ )
		{
		//
		//	Draw Image
		//
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

		int iw = image_dim_array[i][0];
		int ih = image_dim_array[i][1];

		int kw = 512/iw;
		int kh = 512/ih;
		int factor = 100000;
		int q = factor/(kw*kh);
	
		RESET_TIMER( draw_image_timer );
		START_TIMER( draw_image_timer );

		for( int m=0; m<q ; m++ )
			{
			for( int k=0 ; k<kh; k++ )
				{			
				for( int l=0 ; l<kw; l++ )
					{			
					glBindTextureEXT( GL_TEXTURE_2D, textureID[i] );
					glColor4f( 1.0, 1.0, 1.0, 1.0 );
					glBegin( GL_QUADS );

						glTexCoord2f( 0.0, 0.0 );
						glVertex3d  ( l*iw, k*ih, 0 );

						glTexCoord2f( 1.0, 0.0 );
						glVertex3d  ( l*iw+iw, k*ih, 0 );

						glTexCoord2f( 1.0, 1.0 );
						glVertex3d  ( l*iw+iw, k*ih + ih, 0 );

						glTexCoord2f( 0.0, 1.0 );
						glVertex3d  ( l*iw, k*ih+ih, 0 );

					glEnd();
					}
				}
			}

		glFinish();
		STOP_TIMER( draw_image_timer );

		
		glXSwapBuffers(dpy, XtWindow(w));

		draw_image_stats[i][0] += calc_usec_time( draw_image_timer )/(q*kw*kh);
		draw_image_stats[i][1] += 1;
		}

	glPopMatrix();

	for( int j=0 ; j<NUM_IMAGE_SIZES ; j++ )
		{
		if( draw_image_stats[j][1] != 0 )
			draw_image_stats[j][2] =
				draw_image_stats[j][0]/draw_image_stats[j][1];
		else
			draw_image_stats[j][2] = 0;

		if( create_image_stats[j][1] != 0 )
			create_image_stats[j][2] =
				create_image_stats[j][0]/create_image_stats[j][1];
		else
			create_image_stats[j][2] = 0;
		}
	}
