//============================================================================
//
//		Billboard.h
//
//			Jonathan Shade
//
//============================================================================

#ifndef NodeGeom_H
#define NodeGeom_H

#include <GL/gl.h>
#include <GL/glu.h>
#include "VisualObject.h"
#include "Geometry.h"
#include "BoundBox.h"
#include "Camera.h"
#include "projection.h"

class Billboard: public VisualObject {

public:
			Billboard ();
			~Billboard ();

	virtual int			draw();		// required from VO inheritance
	virtual BoundBox	getBoundBox();

	void	printTimings( int id );

	Point3d	getCorner( int i );

	int		getSize();
	int		getSizeUsed();

	void	drawColored();
	void	drawTexture();
	void	drawTextureFrame();
	void	drawTextureFrameNocolor();

	void	invalidate();
	void	kill();
	int		createTextureTO( const BoundBox & b, double ep, int tt );

	int		createTextureDL( const BoundBox & b, double ep, int tt );

	int		feasibleTexture( const BoundBox & b, const Camera & c,
							 const Matrix4d & m, double ep );

	int     textureAvailable() { return textureValid; }

private:	

	GLuint	dispListID;
	GLuint	textureID;
	GLsizei	adj_width;
	GLsizei adj_height;
	int		tex_width;
	int		tex_height;
	int		newTexture;
	int		textureValid;
	int		textureNameValid;

	double	plane[4][3];
	double	m_tc[4][2];
	BoundBox	bbox;

};

#endif
