
//============================================================================
//
//		BoundBox.h
//
//
//			Jonathan Shade
//
//============================================================================

#ifndef BoundBox_H
#define BoundBox_H

#include <GL/gl.h>
#include "Geometry.h"

class BoundBox  {

	friend	BoundBox operator * ( const Matrix4d &m, const BoundBox &bbox );

public:
			BoundBox ();
			BoundBox ( const Point3d & min, const Point3d & max ); 
			BoundBox ( const Point3f & min, const Point3f & max ); 
			~BoundBox ();

	void	Print() const;

	void	Draw();
	void	Draw( GLdouble color[3] );
	void	DrawRadiusFootprint();
	void	Draw_solid( GLdouble color[3] );

	void	setValue ( const Point3d &min, const Point3d &max );
	void	setValue ( const Point3f &min, const Point3f &max );
	void	getValue ( Point3d &min, Point3d &max );

	Point3d	getMin() const;
	Point3d	getMax() const;

	void	getRejectionPoints ( const Normal3d & norm,
								 Point3d &p1, Point3d &p2 ) const;

	int		pointInBox  ( const Point3d & p );
	int		intersect   ( const BoundBox & bbox );
	int		intersection( const BoundBox & bbox, BoundBox & int_bbox );
	int		intersect	( const Point3d &o, const Point3d &d,
						  double & _tmin, double & _tmax );

	
	Point3d	getCentroid () const;
	double	getRadius   () const;
	Point3d	getPoint    ( int i ) const;
	double	operator [] ( int i );

	BoundBox	superBox ( const BoundBox &b );
	BoundBox	transform( const Matrix4d &m );

	void	setGLClipPlanes();
	void	setGLClipPlanesEps();
	void	unsetGLClipPlanes();

private:
	
	Point3d	min;
	Point3d	max;

	double	xeps,yeps,zeps;
};

#endif
