//============================================================================
//
//	Camera.h
//
//		
//		Jonathan Shade
//
//============================================================================

#ifndef	_CAMERA_H
#define _CAMERA_H

 
#include "Geometry.h"
#include "BoundBox.h"

class	Camera
	{
	public:
				Camera();
				~Camera();

		// angle, aspect, near, far 
		void	initFrustum( double, double, double, double );
		void	getFrustumParams( double&, double&, double&, double& ) const;

		void	setViewport( int, int, int, int );
		void	getViewport( int&, int&, int&, int& ) const;

		void	Reset();
		void	Print() const;
		void	Update();

		void	setupGLProjectionMoveModel()  const;
		void	setupGLProjectionMoveCamera() const;
		void	setupGLProjectionMoveCamera( double near, double far ) const;
		void	setupGLProjectionMoveCameraAntiAlias( int jitter_index ) const;

		void	setupGLModelViewMoveModel()  const;
		void	setupGLModelViewMoveCamera() const;

		void	Look   ( double x, double y, double z );
		void	Move   ( double x, double y, double z );
		void	Place  ( double x, double y, double z );
		void	Dolly  ( double x );
		void	Rotate ( double x );

		int 	inFrustumMoveModel ( const BoundBox &bbox, const Matrix4d &m );
		int 	inFrustumMoveCamera( const BoundBox &bbox, int r[6], int f[5] ) const;
		int 	inFrustumMoveCamera( const Point3d  &beg,  const Point3d &end );

		void	DrawNorms();

		void	getSummary( Point3d &p, Vector3d &l, Vector3d &u );

		Point3d			operator () ( int i, int j );
		Point3d			operator [] ( int );

		double* 		getFrustum() const;

		Vector4d		getQuat();
		void			setQuat( const Vector4d q );

		Point3d			getPosition() const;
		Vector3d		getLook()     const;
		Vector3d		getUp()       const;

		const double*	getCameraTransform();
		void			setCameraTransform( Matrix4d matrix );

		const double*	getModelTransform();
		void			setModelTransform( Matrix4d matrix );

		Point3d 		getModelPosition() const;

	
	private:

		void		resetFrustPoints();
		void		resetFrustNorms( Normal3d norms[6], Point3d points[8] );

		int 		inside_test1( const Point3d & pos, const Normal3d & norm,
								  const BoundBox & bbox ) const;
		int 		inside_test8( const Point3d & pos, const Normal3d & norm, 
								  const BoundBox & bbox, const Matrix4d & matrix );
		int 		point_in_frustum( const Point3d & p, int results[6] );

		double		frustum[6];
		double		angle,aspect,near,far;

		//  frustum points are:
		//
		//	near, then far
		//
		// (left,top) (left,bot) (right,bot) (right,top) 

		//
		//		The "un-transformed" Camera. At the
		//		origin looking down the -Z axis.
		//	
		Point3d		pos;			/* position of the camera	*/
		Vector3d	look;			/* direction of view		*/
		Vector3d	up;				/* up vector				*/
		Normal3d	frustNorms[6];
		Point3d		frustPoints[8];

		//
		//		The Viewport
		//
		int	vx,vy,vwidth,vheight;

		//
		//		The "transformed" Camera
		//
		//
		Point3d  	curr_p;
		Vector3d	curr_l;
		Vector3d	curr_u;
		Normal3d	curr_fn[6];
		Point3d		curr_fp[8];


		//
		//		The current Camera transformation.
		//
		//		"Move the Camera"
		//
		//		Includes curr_p from above
		//
		Vector4d	quat;
		Matrix4d	camTrans;

		//
		//		The current Model transformation.
		//
		//		"Move the model"
		//
		//		The opposite of the Camtrans.
		//
		Point3d		modelPos;
		Matrix4d	modelTrans;
	};


#endif

