//============================================================================
//
//		Matrix4d
//
//============================================================================

#ifndef _MATRIX4D_H
#define _MATRIX4D_H

#include <stdio.h>
#include "Vector3d.h"
#include "Vector4d.h"

class Matrix4d
	{
	friend Vector3d	operator * ( const Matrix4d &m, const Vector3d &v );
	friend Vector4d	operator * ( const Matrix4d &m, const Vector4d &v );

public:
	
					Matrix4d();
					Matrix4d( const double v[16] );

	void			Identity();
	Matrix4d		Transpose();

	Matrix4d		preConcat ( const Matrix4d &m );
	Matrix4d		postConcat( const Matrix4d &m );

	Matrix4d		preConcatRotY ( double angle );
	Matrix4d		postConcatRotY( double angle );

	Matrix4d		preConcatTrans ( double x, double y, double z );
	Matrix4d		postConcatTrans( double x, double y, double z );

	Matrix4d		preConcatScale ( double x, double y, double z );
	Matrix4d		postConcatScale( double x, double y, double z );

	Matrix4d & 		setValue( const double v[16] );

	const double*	getValue() const;
	double*			getValue();

	double &		operator[](int i);
	const double &	operator[](int i) const;

	double &	operator() (int i, int j) const;


	void			Print();
	void			Read  ( FILE* fp );
	void			Write ( FILE* fp );

private:
		
		double m[16];
	};


#endif


