//============================================================================
//
//		Matrix4f
//
//============================================================================

#ifndef _MATRIX4F_H
#define _MATRIX4F_H

#include <stdio.h>
#include "Vector3f.h"
#include "Vector4f.h"

class Matrix4f
	{
	friend Vector3f	operator * ( const Matrix4f &m, const Vector3f &v );
	friend Vector4f	operator * ( const Matrix4f &m, const Vector4f &v );

public:
	
					Matrix4f();
					Matrix4f( const float v[16] );

	void			Identity();
	Matrix4f		Transpose();

	Matrix4f		preConcat ( const Matrix4f &m );
	Matrix4f		postConcat( const Matrix4f &m );

	Matrix4f		preConcatRotY ( float angle );
	Matrix4f		postConcatRotY( float angle );

	Matrix4f		preConcatTrans ( float x, float y, float z );
	Matrix4f		postConcatTrans( float x, float y, float z );

	Matrix4f		preConcatScale ( float x, float y, float z );
	Matrix4f		postConcatScale( float x, float y, float z );

	Matrix4f & 		setValue( const float v[16] );

	const float*	getValue() const;
	float*			getValue();

	float &		operator[](int i);
	const float &	operator[](int i) const;

	float &	operator() (int i, int j) const;


	void			Print();
	void			Read  ( FILE* fp );
	void			Write ( FILE* fp );

private:
		
		float m[16];
	};


#endif


