//============================================================================
//	
//	Node.h
//
//		Jonathan Shade
//
//============================================================================

#ifndef Node_H
#define Node_H

#include "stats.h"
#include "state.h"
#include "Geometry.h"
#include "BoundBox.h"
#include "Metric.h"
#include "Camera.h"
#include "VisualObject.h"
#include "Billboard.h"

class Node {

public:
			Node ();
			~Node ();
	
	int			getHeight();
	void		getSceneExtents( Point3d &min, Point3d &max );
	BoundBox&	getBoundBox();

	void	traversal_numBytes( int &bytes, int &extra );
	void	traversal_finalPassStats( FrameStat &stat );

	void	setMultisample ( int flag );
	void	setMetricType( MetricType );

	void	traversal_printAnnotations	();
	void	traversal_initBrickMetricWOInflation();
	void	traversal_findNeighbors		( double inflation );
	void	traversal_cullAndDelete		( FrameStat &stat );
	void	traversal_cullAndAnnotate	( const Camera &camera, int level,
										  FrameStat &stat );
	void	traversal_UIC  ( const Camera &camera, int pt, int df,
							 int aa, FrameStat &stat );

	void	traversal_init2DBoundary ();
	void	traversal_printAnnotations_Brick ();
	void	traversal_cullAndDelete_Brick	 ( FrameStat &stat );
	void	traversal_cullAndAnnotate_Brick  ( const Camera &camera,
											   FrameStat &stat );

	int		traversal_UIC_Brick ( const Camera &camera,
								  double velocity,
								  const Point3d & ray,
								  int aa, int pt,
								  FrameStat &stat );

	void	traversal_draw2DBoundary    ();	
	void	traversal_drawImageCache    ( const Camera &camera, int df,
										  int dlf, int dps, FrameStat &stat);
	void	traversal_drawImageCachePriv( const Camera &camera );
	void	traversal_drawGeometry	    ( const Camera &camera,
										  FrameStat &stat,
										  int ignore_culling );
	void	traversal_drawBoundingBoxes	( int draw_frames, int draw_id );
	void	traversal_drawBoundingBoxesSolid();
	void	traversal_drawHierarchy		( double x, double y );

	void	drawID();
	void	drawPeriphery();
	void	drawWireFrame();
	void	drawSelfSpecial    ( const Camera &camera, int draw_neighbors );
	void	drawSelfSpecialAA  ( const Camera &camera, int draw_neighbors );
	void	drawChildrenSpecial( const Camera &camera );

	int		Read( char* filename, double pixel_eps, int inflate, const Camera &camera );

private:	

	//
	//	Private Methods
	//
	void	findNeighbors( Node &target );
	int		setupFrustum ( const BoundBox &bbox, const Camera &camera, double pixel_eps );
	void	setupRotatedLookat ( const Camera &camera, Matrix4d & matirx,
									 int cf ) const;
	int		frustumStraddlesPlane( const Camera &camera );

	int		intersect_with_parents( const Point3d & p, const Vector3d & v,
										Point3d &q, Node* seed );

	int		intersect_with_boundary( const Point3d & p, const Vector3d & v,
										Point3d &q );

	void	read_VisualObjects( FILE* fp );
	int		read_subtree( FILE* fp, double pixel_eps, const Camera &camera );

	//
	//	Various State
	//
	int			m_height;
	int			m_id;
	int			m_tag;				// front or back
	int			m_annotation;
	int			m_force_update_image;

	//
	//	local   - originiate in this node
	//	total	- including neighboring triangles
	//	stored  - total tris stored at this node.
	//				(stored-local)/local = % over-representation
	//
	int			m_num_tris_local;
	int			m_num_tris_total;
	int			m_num_tris_stored;

	//	Graphics Library State
	static int	m_multisample;

	// partitioning info
	Node		*m_parent;
	Node		*m_back,*m_front;
	Plane		m_plane;

	// Error Metric
	Metric		*m_metric;
	static MetricType m_metric_type; // Must be set before a Read()

	//	Stuff for drawing sky cam view (planes as lines)
	double		m_M,m_B;
	Point3d		m_line_beg,m_line_end;

	//	Visual objects (geometry and billboard)
	BoundBox	 m_bbox;
	BoundBox	 m_bloated_bbox;
	Billboard	 m_billboard;

	int			m_num_vobjs;
	int			m_num_nb_vobjs;
	int			m_num_nb_leafs;
	int			m_num_nb_whole;
	VisualObject **m_vobj_list;	
	VisualObject **m_nb_vobj_list;
	VisualObject **m_nb_whole_list;
	int			 **m_nb_tlist;
	int			 *m_nb_num_tris;
	int			 *m_vobj_inst_tag;
	Node		 **m_nb_leaf_list;
};

#endif

