//============================================================================
//
//		Plane
//
//============================================================================

#include <stdio.h>
#include <math.h>
#include "Plane.h"



//----------------------------------------------------------------------------
//
//		Constructor
//
//----------------------------------------------------------------------------
Plane::Plane()
	{
	}

//----------------------------------------------------------------------------
//
//		Constructor: from 3 points
//
//----------------------------------------------------------------------------
Plane::Plane( Point3d pp, Point3d qq, Point3d rr )
	{
	setValue( pp,qq,rr );
	}


//----------------------------------------------------------------------------
//
//		Constructor: from A B C D
//
//----------------------------------------------------------------------------
Plane::Plane( double a, double b, double c, double d )
	{
	setValue( a,b,c,d );
	}


//----------------------------------------------------------------------------
//
//		setValue
//
//----------------------------------------------------------------------------
void	Plane::setValue( Point3d pp, Point3d qq, Point3d rr )
	{
	Vector3d u = pp - qq;
	Vector3d v = rr - qq;

	n = v.cross(u);
	n.normalize();

	A = n[0];
	B = n[1];
	C = n[2];
	D = -( A*pp[0] + B*pp[1] + C*pp[2] );
	}

void	Plane::setValue( double a, double b, double c, double d )
	{
	double length;

	n[0] = a;
	n[1] = b;
	n[2] = c;

	length = n.normalize();

	A=n[0]; B=n[1]; C=n[2]; D=d/length;
	}

void	Plane::setValue( Point3d pp, Normal3d nn )
	{
	n = nn;
	n.normalize();

	A = n[0];
	B = n[1];
	C = n[2];
	D = -( A*pp[0] + B*pp[1] + C*pp[2] );
	}


//----------------------------------------------------------------------------
//
//		getNormal
//
//----------------------------------------------------------------------------
Normal3d	Plane::getNormal3d()
	{
	return n;
	}

//----------------------------------------------------------------------------
//
//		getPoint
//
//		Returns the point on the plane closest to the origin.
//		Ignores the Y direction.
//
//		The abs() function could be removed with a little
//		cleverness.
//
//----------------------------------------------------------------------------
Point3d	Plane::getPoint3d()
	{
	Vector3d X_axis( 1, 0, 0 );
	Vector3d Z_axis( 0, 0, 1 );

	Normal3d norm = n;

	double dist = abs( this->distanceTo( Point3d(0,0,0) ) );

	if( isInFront( Point3d(0,0,0) ) )
		norm = -n;

	return( Point3d( dist*norm.dot(X_axis), 0, dist*norm.dot(Z_axis) ) );
	}

//----------------------------------------------------------------------------
//
//		distanceTO
//
//			The normal is of unit length, so no
//			division is necessary.
//
//----------------------------------------------------------------------------
double		Plane::distanceTo( const Point3d & pp )
	{
	return( A*pp[0] + B*pp[1] + C*pp[2] + D );
	}

//----------------------------------------------------------------------------
//
//		isInFront
//
//----------------------------------------------------------------------------
int			Plane::isInFront( const Point3d & pp )
	{
	if ( distanceTo(pp) < 0.0 )
		return 0;

	return 1;
	}

//----------------------------------------------------------------------------
//
//		isBehind
//
//----------------------------------------------------------------------------
int			Plane::isBehind( const Point3d & pp )
	{
	if ( distanceTo(pp) > 0.0 )
		return 0;

	return 1;
	}

//----------------------------------------------------------------------------
//
//		Print
//
//----------------------------------------------------------------------------
void		Plane::Print()
	{
	fprintf( stderr,"ABCD:\t%6.2f %6.2f %6.2f %6.2f\n", A,B,C,D );
	}

void	Plane::Write( FILE* fp )
	{
	fprintf( fp, "%f %f %f %f\n", A,B,C,D ); 
	}
