//============================================================================
//
//		Quat.c
//
//
//			Jonathan Shade
//
//============================================================================

#include "math.h"
#include "Geometry.h"
#include "Quat.h"

//----------------------------------------------------------------------------
//
//		Constructor
//
//----------------------------------------------------------------------------
Quat::Quat()
	{
	w = 1.0;
	x = 0.0;
	y = 0.0;
	z = 0.0;
	}

//----------------------------------------------------------------------------
//
//		Destructor
//
//----------------------------------------------------------------------------
Quat::~Quat()
	{
	}	

//----------------------------------------------------------------------------
//
//		normalize
//
//----------------------------------------------------------------------------
void	Quat::normalize()
	{
	double	mag;

	mag = sqrt( w*w + x*x + y*y + z*z );
	
	w = w / mag;
	x = x / mag;
	y = y / mag;
	z = z / mag;
	}

//----------------------------------------------------------------------------
//
//		mult
//
//----------------------------------------------------------------------------
void	Quat::mult( Quat qL, Quat qR )
	{
	Quat qu;
	qu.w	= qL.w*qR.w - qL.x*qR.x - qL.y*qR.y - qL.z*qR.z;
	qu.x	= qL.w*qR.x + qL.x*qR.w + qL.y*qR.z - qL.z*qR.y;
	qu.y	= qL.w*qR.y + qL.y*qR.w + qL.z*qR.x - qL.x*qR.z;
	qu.z	= qL.w*qR.z + qL.z*qR.w + qL.x*qR.y - qL.y*qR.x;

	w = qu.w;
	x = qu.x;
	y = qu.y;
	z = qu.z;
	}


//----------------------------------------------------------------------------
//
//		toMatrix
//
//----------------------------------------------------------------------------
Matrix4d	Quat::toMatrix()
	{
	Matrix4d	matrix;
	double	Nq,s,xs,ys,zs,wx,wy,wz,xx,xy,xz,yy,yz,zz;

	Nq = x*x + y*y + z*z + w*w;

	if( Nq > 0.0 )
		s = 2.0/Nq;
	else
		s = 0.0;

	xs = x*s;	ys = y*s;	zs = z*s;

	wx = w*xs;	wy = w*ys;	wz = w*zs;
	xx = x*xs;	xy = x*ys;	xz = x*zs;
	yy = y*ys;	yz = y*zs;	zz = z*zs;

	matrix[0]	= 1.0 - (yy + zz);
	matrix[1]	= xy + wz;
	matrix[2]	= xz - wy;
	matrix[3]	= 0.0;

	matrix[4]	= xy - wz;
	matrix[5]	= 1.0 - (xx + zz);
	matrix[6]	= yz + wx;
	matrix[7]	= 0.0;

	matrix[8]	= xz + wy;
	matrix[9]	= yz - wx;
	matrix[10]	= 1.0 - (xx + yy);
	matrix[11]	= 0.0;
	
	matrix[12]	= 0.0;
	matrix[13]	= 0.0;
	matrix[14]	= 0.0;
	matrix[15]	= 1.0;


	return( matrix );
	}


//----------------------------------------------------------------------------
//
//		fromVectorAngle
//
//----------------------------------------------------------------------------
void	Quat::fromVectorAngle( Vector3d v, double angle )
	{
	double	len	 = v.length();
	double	sine = sin( angle/2.0 );

	w = cos( angle/2.0 );

	//
	//	Compute	normalized quaternion
	//
	x = v[0] * sine / len;
	y = v[1] * sine / len;
	z = v[2] * sine / len;
	}

