//============================================================================
//
//		Trackball.c
//
//
//			Jonathan Shade
//
//============================================================================

#include <math.h>
#include "Geometry.h"
#include "Trackball.h"


//----------------------------------------------------------------------------
//
//		Constructor
//
//----------------------------------------------------------------------------
Trackball::Trackball()
	{
	ball_center[0] = 0.0;
	ball_center[1] = 0.0;
	ball_center[2] = 0.0;
	
	ball_radius		= 2.0;	// Entire window is covered by the
							//  face of the trackball since
							//  mouse x,y are mapped to -1,1
	}

Trackball::Trackball( Point3d c, double r )
	{
	ball_center[0] = c[0];
	ball_center[1] = c[1];
	ball_center[2] = c[2];
	
	ball_radius	= r;
	}


//----------------------------------------------------------------------------
//
//		Destructor
//
//----------------------------------------------------------------------------
Trackball::~Trackball()
	{
	}

//----------------------------------------------------------------------------
//
//		initBall
//
//
//----------------------------------------------------------------------------
void	Trackball::initBall( double quat[4] )
	{
	track_quat.x = quat[0];
	track_quat.y = quat[1];
	track_quat.z = quat[2];
	track_quat.w = quat[3];
	}

void	Trackball::initBall( Vector4d q )
	{
	track_quat.x = q[0];
	track_quat.y = q[1];
	track_quat.z = q[2];
	track_quat.w = q[3];
	}


//----------------------------------------------------------------------------
//
//	MouseClickToVector
//
//		Private function to compute a spot on the _UNIT_
//		sphere from a mouse click.
//
//----------------------------------------------------------------------------
void	Trackball::MouseClickToVector( double mx, double my, double vec[3] )
	{
	double	mag;
	double	scale;

	vec[0] = (mx - ball_center[0]) / ball_radius;
	vec[1] = (my - ball_center[1]) / ball_radius;

	mag = vec[0]*vec[0] + vec[1]*vec[1];

	if( mag	> 1.0 )
		{
		scale	= 1.0 / sqrt(mag);
		vec[0] *= scale;
		vec[1] *= scale;
		vec[2]	= 0.0;
		}
	else
		{
		vec[2]	= sqrt(1 - mag);
		}
	}


//----------------------------------------------------------------------------
//
//		beginDrag
//
//----------------------------------------------------------------------------
void	Trackball::beginDrag( double mx, double my )
	{
	MouseClickToVector( mx, my, begin_vec );

	begin_quat.x = track_quat.x;
	begin_quat.y = track_quat.y;
	begin_quat.z = track_quat.z;
	begin_quat.w = track_quat.w;
	}



//----------------------------------------------------------------------------
//
//		updateDrag
//
//----------------------------------------------------------------------------
void	Trackball::updateDrag( double mx, double my )
	{
	Quat	new_quat;

	MouseClickToVector( mx, my, update_vec );

	//
	//	Find  axis about which to rotate
	//
	//	Homespun cross product.
	//
	new_quat.x =	begin_vec[1]*update_vec[2] - begin_vec[2]*update_vec[1];
	new_quat.y =	begin_vec[2]*update_vec[0] - begin_vec[0]*update_vec[2];
	new_quat.z =	begin_vec[0]*update_vec[1] - begin_vec[1]*update_vec[0];

	new_quat.w =    begin_vec[0]*update_vec[0] +
		  			begin_vec[1]*update_vec[1] +
					begin_vec[2]*update_vec[2];

	track_quat.mult( new_quat, begin_quat );
	//track_quat.normalize();
	}

//----------------------------------------------------------------------------
//
//		getMatrix
//
//----------------------------------------------------------------------------
Matrix4d	Trackball::getMatrix()
	{
	return( track_quat.toMatrix() );
	}

//----------------------------------------------------------------------------
//
//		getQuat
//
//----------------------------------------------------------------------------
void	Trackball::getQuat( double q[4] )
	{
	q[0] = track_quat.x;
	q[1] = track_quat.y;
	q[2] = track_quat.z;
	q[3] = track_quat.w;
	}

Vector4d	Trackball::getQuat()
	{
	Vector4d v;

	v[0] = track_quat.x;
	v[1] = track_quat.y;
	v[2] = track_quat.z;
	v[3] = track_quat.w;

	return( v );
	}





