#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <GL/gl.h>
#include <GL/glu.h>

//#include <il/ilGenericImgFile.h>
//#include <il/ilFileImg.h>

//#include <ifl/iflFile.h>
#include "TriangleList.h"

#define TL_OFF		0
#define TL_NOFF		1
#define TL_COFF		2
#define TL_CNOFF	3
#define TL_TCOFF	4
#define TL_TNOFF	5


TriangleList::TriangleList()
	{
	m_geomDispList = 0;
	m_compDispList = 0;
	m_transform.Identity();
	initMaterial();

	m_floff	 = 0;
	m_type	 = -1;
	m_nlist  = NULL;
	m_vlist  = NULL;
	m_clist  = NULL;
	m_tlist  = NULL;
	m_stlist = NULL;
	m_num_tris  = 0;
	m_num_verts = 0;
	m_per_vertex_colors  = 0;
	m_per_vertex_normals = 0;
	}

TriangleList::TriangleList( int num_tris, int num_vertices,
							int per_vertex_colors, int per_vertex_normals )
	{
	m_geomDispList = 0;
	m_compDispList = 0;
	m_transform.Identity();
	initMaterial();

	m_per_vertex_colors  = per_vertex_colors;
	m_per_vertex_normals = per_vertex_normals;
	setArrayBounds( num_tris, num_vertices );
	}

TriangleList::TriangleList( int num_tris, int num_verts, Point3f *va,
							float *stlist, Normal3f *na, Color *ca, Triangle *ta  )
	{
	m_geomDispList = 0;
	m_compDispList = 0;
	m_transform.Identity();
	initMaterial();
	
	m_vlist = va;
	m_nlist = na;
	m_clist = ca;
	m_tlist = ta;
	m_stlist = stlist;

	m_num_tris = num_tris;
	m_num_verts = num_verts;
	m_per_vertex_colors  = ( ca != NULL );
	m_per_vertex_normals = ( na != NULL );
	}

TriangleList::TriangleList( const TriangleList &t )
	{
	m_geomDispList = 0;
	m_compDispList = 0;
	m_transform.Identity();
	initMaterial();

	m_floff = t.m_floff;
	m_type	= t.m_type;	
	m_vlist = t.m_vlist;
	m_nlist = t.m_nlist;
	m_clist = t.m_clist;
	m_tlist = t.m_tlist;
	m_list  = t.m_list;

	m_num_tris  = t.m_num_tris;
	m_num_verts = t.m_num_verts;
	m_per_vertex_colors  = t.m_per_vertex_colors;
	m_per_vertex_normals = t.m_per_vertex_normals;
	}

//
//	This may not work for objects with per-face stuff.
//
void
TriangleList::initFLOFF( const TriangleList &t, char *filename )
	{
	m_geomDispList = 0;
	m_compDispList = 0;
	initMaterial();

	m_floff = 1;
	m_type	= t.m_type;	
	m_vlist = t.m_vlist;
	m_nlist = t.m_nlist;
	m_clist = t.m_clist;

	m_num_verts = t.m_num_verts;
	m_per_vertex_colors  = t.m_per_vertex_colors;
	m_per_vertex_normals = t.m_per_vertex_normals;


	//
	//	Read the list of tris (faces) from the file
	//
	char	scratch[256];
	FILE *fp = fopen( filename, "r" );

	if( fp == NULL )
		{
		fprintf( stderr, "Cant read FLOFF: %s\n", filename );
		exit(1);
		}

	fscanf( fp, "%s", scratch );
	fscanf( fp, "%s", scratch );
	fscanf( fp, "%d", &m_num_tris );

	//fprintf(stderr,"reading FLOFF: %s %d\n", filename,m_num_tris );

	m_list  = new      int [m_num_tris];
	m_tlist	= new Triangle [m_num_tris];

	int junk,t1,t2,t3;
	for( int i=0 ; i<m_num_tris ; i++ )
		{
		m_list[i] = i;

		fscanf( fp, "%d %d %d %d", &junk, &t1 ,&t2, &t3 );
	
		this->setTriangle( i, t1, t2, t3 );
		}

	fclose(fp);
	}

//
//
//
void
TriangleList::setArrayBounds( int num_tris, int num_vertices )
	{
	m_num_tris  = num_tris;
	m_num_verts = num_vertices;

	m_tlist = new Triangle[m_num_tris];
	m_vlist = new Point3f[m_num_verts];

	if( m_type == TL_TCOFF || TL_TNOFF )
		m_stlist = new float [m_num_verts*2];

	if( m_per_vertex_normals ) m_nlist = new Normal3f[m_num_verts];
	else					   m_nlist = new Normal3f[m_num_tris];

	if( m_per_vertex_colors )  m_clist = new Color[m_num_verts];
	else					   m_clist = new Color[m_num_tris];

	m_list = new int [m_num_tris];

	for( int i=0; i<m_num_tris ; i++ )
		m_list[i] = i;
	}


//	ka			0.9
//	kd			0.79
//	ks			0.3
//	shiny		6.0
//	ambient		0.191 0.643 0.288
//	diffuse		0.191 0.643 0.288
//	specular	1.0   1.0   1.0
void
TriangleList::initMaterial()
	{
    m_mat.ka = 0.9;
    m_mat.kd = 0.79;
    m_mat.ks = 0.3;
    m_mat.shiny = 6.0;
   
    m_mat.ambient[0]     = 0.5 * m_mat.ka;
    m_mat.ambient[1]     = 0.5 * m_mat.ka;
    m_mat.ambient[2]     = 0.5 * m_mat.ka;
    m_mat.ambient[3]     = 1.0;

    m_mat.diffuse[0]     = 0.5 * m_mat.kd;
    m_mat.diffuse[1]     = 0.5 * m_mat.kd;
    m_mat.diffuse[2]     = 0.5 * m_mat.kd;
    m_mat.diffuse[3]     = 1.0;

    m_mat.specular[0]    = 1.0   * m_mat.ks;
    m_mat.specular[1]    = 1.0   * m_mat.ks;
    m_mat.specular[2]    = 1.0   * m_mat.ks;
    m_mat.specular[3]    = 1.0;
	}

TriangleList::~TriangleList()
	{
	if( m_vlist != NULL )	delete [] m_vlist;
	if( m_nlist != NULL )	delete [] m_nlist;
	if( m_clist != NULL )	delete [] m_clist;
	if( m_tlist != NULL )	delete [] m_stlist;
	}

void
TriangleList::deallocRawData()
	{
	delete [] m_vlist;
	delete [] m_nlist;
	delete [] m_clist;
	delete [] m_tlist;
	delete [] m_stlist;
	}

int
TriangleList::getBytes()
	{
/*
	int bytes = 24*m_num_verts + 12*m_num_tris;

	if( m_per_vertex_colors)	bytes += 32*m_num_verts;
	else						bytes += 32*m_num_tris;

	if( m_per_vertex_normals )	bytes += 24*m_num_verts;
	else						bytes += 24*m_num_tris;

	if( m_stlist != NULL )
		bytes += 8*2*m_num_verts;
*/
	int bytes = 12*m_num_verts + 12*m_num_tris;

	if( m_per_vertex_colors)	bytes += 16*m_num_verts;
	else						bytes += 16*m_num_tris;

	if( m_per_vertex_normals )	bytes += 12*m_num_verts;
	else						bytes += 12*m_num_tris;

	if( m_stlist != NULL )
		bytes += 4*2*m_num_verts;

	if( m_floff )
		{
		bytes =  12*m_num_tris;
		//fprintf(stderr,"floff: %d\n",bytes);
		}

	return( bytes );
	}

int
TriangleList::getNumTriangles()
	{
	return( m_num_tris );
	}

int
TriangleList::getNumVertices()
	{
	return( m_num_verts );
	}

void
TriangleList::getTriangle( int i, Point3d &p1, Point3d &p2, Point3d &p3 )
	{
	if( i<0 || i>=m_num_tris )
	fprintf(stderr,"TriangleList::getTriangle -- Error: Index out of range\n");

	//
	//	Assumes transform has already been set !!!!
	//
	Point4f p14,p24,p34;

	//	Convert to homogneous coordinates
	p14.setValue( m_vlist[ m_tlist[i].index[0] ], 1 );
	p24.setValue( m_vlist[ m_tlist[i].index[1] ], 1 );
	p34.setValue( m_vlist[ m_tlist[i].index[2] ], 1 );

	//	Transform
	p14 = m_transform * p14;
	p24 = m_transform * p24;
	p34 = m_transform * p34;

	//	Convert back to 3d coordinates
	p1.setValue( (double) p14[0], (double) p14[1], (double) p14[2] );
	p2.setValue( (double) p24[0], (double) p24[1], (double) p24[2] );
	p3.setValue( (double) p34[0], (double) p34[1], (double) p34[2] );
	}

void
TriangleList::getTriangle( int i, Point3f &p1, Point3f &p2, Point3f &p3 )
	{
	if( i<0 || i>=m_num_tris )
	fprintf(stderr,"TriangleList::getTriangle -- Error: Index out of range\n");

	//
	//	Assumes transform has already been set !!!!
	//
	Point4f p14,p24,p34;

	//	Convert to homogneous coordinates
	p14.setValue( m_vlist[ m_tlist[i].index[0] ], 1 );
	p24.setValue( m_vlist[ m_tlist[i].index[1] ], 1 );
	p34.setValue( m_vlist[ m_tlist[i].index[2] ], 1 );

	//	Transform
	p14 = m_transform * p14;
	p24 = m_transform * p24;
	p34 = m_transform * p34;

	//	Convert back to 3d coordinates
	p1.setValue( p14[0], p14[1], p14[2] );
	p2.setValue( p24[0], p24[1], p24[2] );
	p3.setValue( p34[0], p34[1], p34[2] );
	}

Point3f&
TriangleList::operator[]( int index )
	{
	return( m_vlist[index] );
	}

void
TriangleList::setVertex( int i, const Point3f & point,
								const Normal3f & normal, float s, float t )
	{
	m_vlist[i] = point;

	//
	//	VERTEX Normals
	//
	if( m_per_vertex_normals )
		m_nlist[i] = normal;

	if( m_stlist != NULL )
		{
		m_stlist[i*2]   = s;
		m_stlist[i*2+1] = t;
		}
	}

void
TriangleList::setTriangle( int i, int v0, int v1, int v2 )
	{
	Vector3f p,q,r;
	Normal3f n;

	m_tlist[i].index[0] = v0;
	m_tlist[i].index[1] = v1;
	m_tlist[i].index[2] = v2;

	//
	//	FACE Normals
	//
	if( !m_per_vertex_normals )
		{
		p = m_vlist[ v2 ];
		q = m_vlist[ v0 ] - p;
		r = m_vlist[ v1 ] - p;

		n = q.cross(r);
		n.normalize();

		m_nlist[i]= n;
		}

	}

void
TriangleList::setTriangleColor(int i, float red,  float green,
									  float blue, float alpha)
{
	if( red   < 0.0 ) red   = 0.0;
	if( red   > 1.0 ) red   = 1.0;

	if( green < 0.0 ) green = 0.0;
	if( green > 1.0 ) green = 1.0;

	if( blue  < 0.0 ) blue  = 0.0;
	if( blue  > 1.0 ) blue  = 1.0;

	if( alpha < 0.0 ) alpha = 0.0;
	if( alpha > 1.0 ) alpha = 1.0;

    m_clist[i].value[0] = red   * m_mat.kd;
    m_clist[i].value[1] = green * m_mat.kd;
    m_clist[i].value[2] = blue  * m_mat.kd;
    m_clist[i].value[3] = alpha;

//fprintf(stderr,"%d: %f %f %f %f\n",i,red,green,blue,alpha );
}


void
TriangleList::setMaterial( Material material )
	{
	m_mat = material;
	}

void
TriangleList::setMaterial( float m[16] )
	{
    m_mat.ka = m[0];
    m_mat.kd = m[1];
    m_mat.ks = m[2];
    m_mat.shiny = m[3];
   
    m_mat.ambient[0]     = m[4]  * m_mat.ka;
    m_mat.ambient[1]     = m[5]  * m_mat.ka;
    m_mat.ambient[2]     = m[6]  * m_mat.ka;
    m_mat.ambient[3]     = m[7];

    m_mat.diffuse[0]     = m[8]  * m_mat.kd;
    m_mat.diffuse[1]     = m[9]  * m_mat.kd;
    m_mat.diffuse[2]     = m[10] * m_mat.kd;
    m_mat.diffuse[3]     = m[11];

    m_mat.specular[0]    = m[12] * m_mat.ks;
    m_mat.specular[1]    = m[13] * m_mat.ks;
    m_mat.specular[2]    = m[14] * m_mat.ks;
    m_mat.specular[3]    = m[15];
	}

void
TriangleList::setTransform( Matrix4f matrix )
	{
	m_transform = matrix;
	}


void
TriangleList::setGeomDispList( int disp_list )
	{
	m_geomDispList = disp_list;
	}

int
TriangleList::getGeomDispList()
	{
	return( m_geomDispList );
	}

//---------------------------------------------------------------
//
//		draw	- vertices
//
//---------------------------------------------------------------
void
TriangleList::drawV( int list[], int num_tris )
{
	glPushMatrix();
	glMultMatrixf( m_transform.getValue() );

	glBegin( GL_TRIANGLES );
		int i;
		for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];

			glVertex3fv( m_vlist[ m_tlist[i].index[0] ].getValue() );
			glVertex3fv( m_vlist[ m_tlist[i].index[1] ].getValue() );
			glVertex3fv( m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }
	glEnd();

	glPopMatrix();
}

//---------------------------------------------------------------
//
//		drawT		- texture coordinates
//					  vertices
//
//---------------------------------------------------------------
void
TriangleList::drawT( int list[], int num_tris )
{
	glBegin( GL_TRIANGLES );
		int i;
		for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[0]*2 ] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[0] ].getValue() );
			glTexCoord2fv( &m_stlist[ m_tlist[i].index[1]*2 ] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[1] ].getValue() );
			glTexCoord2fv( &m_stlist[ m_tlist[i].index[2]*2 ] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }
	glEnd();
}

//---------------------------------------------------------------
//
//		drawTC		- texture coordinates
//					  color
//					  vertices
//
//---------------------------------------------------------------
void
TriangleList::drawTC( int list[], int num_tris )
{
	glBegin( GL_TRIANGLES );
		int i;

		if( m_per_vertex_colors )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];

			glColor4fv (     m_clist[ m_tlist[i].index[0] ].value );
			glTexCoord2fv( &m_stlist[ m_tlist[i].index[0]*2 ] );
//			glTexCoord2d( m_stlist[ m_tlist[i].index[0]*2  ],
//						  m_stlist[ m_tlist[i].index[0]*2+1] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[0] ].getValue() );

			glColor4fv (     m_clist[ m_tlist[i].index[1] ].value );
			glTexCoord2fv( &m_stlist[ m_tlist[i].index[1]*2 ] );
//			glTexCoord2d( m_stlist[ m_tlist[i].index[1]*2   ],
//						  m_stlist[ m_tlist[i].index[1]*2+1 ] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[1] ].getValue() );

			glColor4fv (     m_clist[ m_tlist[i].index[2] ].value );
			glTexCoord2fv( &m_stlist[ m_tlist[i].index[2]*2 ] );
//			glTexCoord2d( m_stlist[ m_tlist[i].index[2]*2   ],
//						  m_stlist[ m_tlist[i].index[2]*2+1 ] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }
		else
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
			glColor4fv(  m_clist[i].value );
	
			glTexCoord2fv( &m_stlist[ m_tlist[i].index[0]*2 ] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[0] ].getValue() );

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[1]*2 ] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[1] ].getValue() );

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[2]*2 ] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }

	glEnd();
}

//---------------------------------------------------------------
//
//		drawTN		- texture coordinates
//					  normal
//					  vertices
//
//---------------------------------------------------------------
void
TriangleList::drawTN( int list[], int num_tris )
{
	glEnable( GL_LIGHTING );

	glMaterialfv( GL_FRONT, GL_AMBIENT,     m_mat.ambient    );
	glMaterialfv( GL_FRONT, GL_DIFFUSE,     m_mat.diffuse    );
	glMaterialfv( GL_FRONT, GL_SPECULAR,    m_mat.specular   );
	glMaterialf ( GL_FRONT, GL_SHININESS,   m_mat.shiny      );

	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	glBegin( GL_TRIANGLES );
		int i;

		if( m_per_vertex_normals )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
	
			glTexCoord2fv( &m_stlist[ m_tlist[i].index[0]*2 ] );
			glNormal3fv(     m_nlist[ m_tlist[i].index[0] ].getValue() );
			glVertex3fv(     m_vlist[ m_tlist[i].index[0] ].getValue() );

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[1]*2 ] );
			glNormal3fv(     m_nlist[ m_tlist[i].index[1] ].getValue() );
			glVertex3fv(     m_vlist[ m_tlist[i].index[1] ].getValue() );

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[2]*2 ] );
			glNormal3fv(     m_nlist[ m_tlist[i].index[2] ].getValue() );
			glVertex3fv(     m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }
		else
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
			glNormal3fv( m_nlist[i].getValue() );
	
			glTexCoord2fv( &m_stlist[ m_tlist[i].index[0]*2 ] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[0] ].getValue() );

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[1]*2 ] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[1] ].getValue() );

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[2]*2 ] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }

	glEnd();
	glDisable( GL_LIGHTING );
}

//---------------------------------------------------------------
//
//		drawTNC		- texture coordinates
//					  normal
//					  color
//					  vertices
//
//---------------------------------------------------------------
void
TriangleList::drawTNC( int list[], int num_tris )
{
	glEnable( GL_LIGHTING );
	glMaterialfv( GL_FRONT, GL_AMBIENT,     m_mat.ambient    );
	glMaterialfv( GL_FRONT, GL_DIFFUSE,     m_mat.diffuse    );
	glMaterialfv( GL_FRONT, GL_SPECULAR,    m_mat.specular   );
	glMaterialf ( GL_FRONT, GL_SHININESS,   m_mat.shiny      );

	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	glBegin( GL_TRIANGLES );
		int i;

		if( m_per_vertex_normals && m_per_vertex_colors )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
	
			glTexCoord2fv( &m_stlist[ m_tlist[i].index[0]*2 ] );
			glNormal3fv(     m_nlist[ m_tlist[i].index[0] ].getValue() );
			glColor4fv (     m_clist[ m_tlist[i].index[0] ].value );
			glVertex3fv(     m_vlist[ m_tlist[i].index[0] ].getValue() );

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[1]*2 ] );
			glNormal3fv(     m_nlist[ m_tlist[i].index[1] ].getValue() );
			glColor4fv (     m_clist[ m_tlist[i].index[1] ].value );
			glVertex3fv(     m_vlist[ m_tlist[i].index[1] ].getValue() );

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[2]*2 ] );
			glNormal3fv(     m_nlist[ m_tlist[i].index[2] ].getValue() );
			glColor4fv (     m_clist[ m_tlist[i].index[2] ].value );
			glVertex3fv(     m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }

	else if( m_per_vertex_normals && !m_per_vertex_colors )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
			glColor4fv(  m_clist[i].value );
	
			glTexCoord2fv( &m_stlist[ m_tlist[i].index[0]*2 ] );
			glNormal3fv(     m_nlist[ m_tlist[i].index[0] ].getValue() );
			glVertex3fv(     m_vlist[ m_tlist[i].index[0] ].getValue() );

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[1]*2 ] );
			glNormal3fv(     m_nlist[ m_tlist[i].index[1] ].getValue() );
			glVertex3fv(     m_vlist[ m_tlist[i].index[1] ].getValue() );

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[2]*2 ] );
			glNormal3fv(     m_nlist[ m_tlist[i].index[2] ].getValue() );
			glVertex3fv(     m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }


	else if( !m_per_vertex_normals && m_per_vertex_colors )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
			glNormal3fv( m_nlist[i].getValue() );
	
			glTexCoord2fv( &m_stlist[ m_tlist[i].index[0]*2 ] );
			glColor4fv (     m_clist[ m_tlist[i].index[0] ].value );
			glVertex3fv(     m_vlist[ m_tlist[i].index[0] ].getValue() );

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[1]*2 ] );
			glColor4fv (     m_clist[ m_tlist[i].index[1] ].value );
			glVertex3fv(     m_vlist[ m_tlist[i].index[1] ].getValue() );

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[2]*2 ] );
			glColor4fv (     m_clist[ m_tlist[i].index[2] ].value );
			glVertex3fv(     m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }


	else if( !m_per_vertex_normals && !m_per_vertex_colors )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
			glNormal3fv( m_nlist[i].getValue() );
			glColor4fv(  m_clist[i].value );
	
			glTexCoord2fv( &m_stlist[ m_tlist[i].index[0]*2 ] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[0] ].getValue() );

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[1]*2 ] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[1] ].getValue() );

			glTexCoord2fv( &m_stlist[ m_tlist[i].index[2]*2 ] );
			glVertex3fv(     m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }

	glEnd();
	glDisable( GL_LIGHTING );
}

//---------------------------------------------------------------
//
//		drawC
//					  color
//					  vertices
//
//---------------------------------------------------------------
void
TriangleList::drawC( int list[], int num_tris )
{
	glBegin( GL_TRIANGLES );
		int i;

		if( m_per_vertex_colors )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
	
			glColor4fv ( m_clist[ m_tlist[i].index[0] ].value );
			glVertex3fv( m_vlist[ m_tlist[i].index[0] ].getValue() );

			glColor4fv ( m_clist[ m_tlist[i].index[1] ].value );
			glVertex3fv( m_vlist[ m_tlist[i].index[1] ].getValue() );

			glColor4fv ( m_clist[ m_tlist[i].index[2] ].value );
			glVertex3fv( m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }
		else
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
			glColor4fv(  m_clist[i].value );
	
			glVertex3fv( m_vlist[ m_tlist[i].index[0] ].getValue() );
			glVertex3fv( m_vlist[ m_tlist[i].index[1] ].getValue() );
			glVertex3fv( m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }

	glEnd();
}

//---------------------------------------------------------------
//
//		drawNC
//					  normal
//					  color
//					  vertices
//
//---------------------------------------------------------------
void
TriangleList::drawNC( int list[], int num_tris )
{
	glEnable( GL_LIGHTING );
	glMaterialfv( GL_FRONT, GL_AMBIENT,     m_mat.ambient    );
	glMaterialfv( GL_FRONT, GL_DIFFUSE,     m_mat.diffuse    );
	glMaterialfv( GL_FRONT, GL_SPECULAR,    m_mat.specular   );
	glMaterialf ( GL_FRONT, GL_SHININESS,   m_mat.shiny      );

	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	glBegin( GL_TRIANGLES );
		int i;

		if( m_per_vertex_normals && m_per_vertex_colors )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
	
			glNormal3fv(    m_nlist[ m_tlist[i].index[0] ].getValue() );
			glColor4fv (    m_clist[ m_tlist[i].index[0] ].value );
			glVertex3fv(    m_vlist[ m_tlist[i].index[0] ].getValue() );

			glNormal3fv(    m_nlist[ m_tlist[i].index[1] ].getValue() );
			glColor4fv (    m_clist[ m_tlist[i].index[1] ].value );
			glVertex3fv(    m_vlist[ m_tlist[i].index[1] ].getValue() );

			glNormal3fv(    m_nlist[ m_tlist[i].index[2] ].getValue() );
			glColor4fv (    m_clist[ m_tlist[i].index[2] ].value );
			glVertex3fv(    m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }

	else if( m_per_vertex_normals && !m_per_vertex_colors )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
			glColor4fv(  m_clist[i].value );
	
			glNormal3fv(    m_nlist[ m_tlist[i].index[0] ].getValue() );
			glVertex3fv(    m_vlist[ m_tlist[i].index[0] ].getValue() );

			glNormal3fv(    m_nlist[ m_tlist[i].index[1] ].getValue() );
			glVertex3fv(    m_vlist[ m_tlist[i].index[1] ].getValue() );

			glNormal3fv(    m_nlist[ m_tlist[i].index[2] ].getValue() );
			glVertex3fv(    m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }


	else if( !m_per_vertex_normals && m_per_vertex_colors )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
			glNormal3fv( m_nlist[i].getValue() );
	
			glColor4fv (    m_clist[ m_tlist[i].index[0] ].value );
			glVertex3fv(    m_vlist[ m_tlist[i].index[0] ].getValue() );

			glColor4fv (    m_clist[ m_tlist[i].index[1] ].value );
			glVertex3fv(    m_vlist[ m_tlist[i].index[1] ].getValue() );

			glColor4fv (    m_clist[ m_tlist[i].index[2] ].value );
			glVertex3fv(    m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }


	else if( !m_per_vertex_normals && !m_per_vertex_colors )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
			glNormal3fv( m_nlist[i].getValue() );
			glColor4fv(  m_clist[i].value );
	
			glVertex3fv(    m_vlist[ m_tlist[i].index[0] ].getValue() );
			glVertex3fv(    m_vlist[ m_tlist[i].index[1] ].getValue() );
			glVertex3fv(    m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }
	glEnd();
	glDisable( GL_LIGHTING );
}

//---------------------------------------------------------------
//
//		drawNC
//					  normal
//					  color
//					  vertices
//
//---------------------------------------------------------------
void
TriangleList::drawYELLOW( int list[], int num_tris )
{
	glEnable( GL_LIGHTING );
	glMaterialfv( GL_FRONT, GL_AMBIENT,     m_mat.ambient    );
	glMaterialfv( GL_FRONT, GL_DIFFUSE,     m_mat.diffuse    );
	glMaterialfv( GL_FRONT, GL_SPECULAR,    m_mat.specular   );
	glMaterialf ( GL_FRONT, GL_SHININESS,   m_mat.shiny      );

	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	GLfloat cy[4] = { .35, .35, 0, 1 };
	GLfloat *ct;
	float F = 0.4;

	glBegin( GL_TRIANGLES );
		int i;

		if( m_per_vertex_normals && m_per_vertex_colors )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
	
			glNormal3fv(    m_nlist[ m_tlist[i].index[0] ].getValue() );
			ct = m_clist[ m_tlist[i].index[0] ].value ;
			glColor4f  ( cy[0]+F*ct[0], cy[1]+F*ct[1], cy[2]+F*ct[2], cy[3] );
			glVertex3fv(    m_vlist[ m_tlist[i].index[0] ].getValue() );

			glNormal3fv(    m_nlist[ m_tlist[i].index[1] ].getValue() );
			ct = m_clist[ m_tlist[i].index[1] ].value ;
			glColor4f  ( cy[0]+F*ct[0], cy[1]+F*ct[1], cy[2]+F*ct[2], cy[3] );
			glVertex3fv(    m_vlist[ m_tlist[i].index[1] ].getValue() );

			glNormal3fv(    m_nlist[ m_tlist[i].index[2] ].getValue() );
			ct = m_clist[ m_tlist[i].index[2] ].value ;
			glColor4f  ( cy[0]+F*ct[0], cy[1]+F*ct[1], cy[2]+F*ct[2], cy[3] );
			glVertex3fv(    m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }

	else if( m_per_vertex_normals && !m_per_vertex_colors )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
			ct = m_clist[i].value ;
			glColor4f  ( cy[0]+F*ct[0], cy[1]+F*ct[1], cy[2]+F*ct[2], cy[3] );
	
			glNormal3fv(    m_nlist[ m_tlist[i].index[0] ].getValue() );
			glVertex3fv(    m_vlist[ m_tlist[i].index[0] ].getValue() );

			glNormal3fv(    m_nlist[ m_tlist[i].index[1] ].getValue() );
			glVertex3fv(    m_vlist[ m_tlist[i].index[1] ].getValue() );

			glNormal3fv(    m_nlist[ m_tlist[i].index[2] ].getValue() );
			glVertex3fv(    m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }


	else if( !m_per_vertex_normals && m_per_vertex_colors )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
			glNormal3fv( m_nlist[i].getValue() );
	
			ct = m_clist[ m_tlist[i].index[0] ].value ;
			glColor4f  ( cy[0]+F*ct[0], cy[1]+F*ct[1], cy[2]+F*ct[2], cy[3] );
			glVertex3fv(    m_vlist[ m_tlist[i].index[0] ].getValue() );

			ct = m_clist[ m_tlist[i].index[1] ].value ;
			glColor4f  ( cy[0]+F*ct[0], cy[1]+F*ct[1], cy[2]+F*ct[2], cy[3] );
			glVertex3fv(    m_vlist[ m_tlist[i].index[1] ].getValue() );

			ct = m_clist[ m_tlist[i].index[2] ].value ;
			glColor4f  ( cy[0]+F*ct[0], cy[1]+F*ct[1], cy[2]+F*ct[2], cy[3] );
			glVertex3fv(    m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }


	else if( !m_per_vertex_normals && !m_per_vertex_colors )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
			glNormal3fv( m_nlist[i].getValue() );
			ct = m_clist[i].value ;
			glColor4f  ( cy[0]+F*ct[0], cy[1]+F*ct[1], cy[2]+F*ct[2], cy[3] );
	
			glVertex3fv(    m_vlist[ m_tlist[i].index[0] ].getValue() );
			glVertex3fv(    m_vlist[ m_tlist[i].index[1] ].getValue() );
			glVertex3fv(    m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }
	glEnd();
	glDisable( GL_LIGHTING );
}

//---------------------------------------------------------------
//
//		drawN
//					  normal
//					  vertices
//
//---------------------------------------------------------------
void
TriangleList::drawN( int list[], int num_tris )
{
	glEnable( GL_LIGHTING );
	glMaterialfv( GL_FRONT, GL_AMBIENT,     m_mat.ambient    );
	glMaterialfv( GL_FRONT, GL_DIFFUSE,     m_mat.diffuse    );
	glMaterialfv( GL_FRONT, GL_SPECULAR,    m_mat.specular   );
	glMaterialf ( GL_FRONT, GL_SHININESS,   m_mat.shiny      );

	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	glBegin( GL_TRIANGLES );
		int i;

		if( m_per_vertex_normals )
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
	
			glNormal3fv( m_nlist[ m_tlist[i].index[0] ].getValue() );
			glVertex3fv( m_vlist[ m_tlist[i].index[0] ].getValue() );

			glNormal3fv( m_nlist[ m_tlist[i].index[1] ].getValue() );
			glVertex3fv( m_vlist[ m_tlist[i].index[1] ].getValue() );

			glNormal3fv( m_nlist[ m_tlist[i].index[0] ].getValue() );
			glVertex3fv( m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }
		else
			for( int j=0 ; j<num_tris ; j++ )
			{
			i = list[j];
			glNormal3fv( m_nlist[i].getValue() );
	
			glVertex3fv( m_vlist[ m_tlist[i].index[0] ].getValue() );
			glVertex3fv( m_vlist[ m_tlist[i].index[1] ].getValue() );
			glVertex3fv( m_vlist[ m_tlist[i].index[2] ].getValue() );
		    }
	glEnd();
	glDisable( GL_LIGHTING );
}


//---------------------------------------------------------------
//---------------------------------------------------------------
void
TriangleList::createDisplayList()
	{
	if( m_geomDispList == 0 )
		{
		m_geomDispList = glGenLists( 1 );

		glNewList( m_geomDispList, GL_COMPILE );

		switch( m_type )
			{
			case TL_OFF:
				drawC( m_list, m_num_tris );
				break;

			case TL_NOFF:
				drawN( m_list, m_num_tris );
				break;

			case TL_CNOFF:
				drawNC( m_list, m_num_tris );
				break;

			case TL_COFF:
				drawC( m_list, m_num_tris );
				break;

			case TL_TCOFF:
				glEnable( GL_TEXTURE_2D );
				glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, m_tfunc );
				glBindTextureEXT( GL_TEXTURE_2D, m_texobj );
				drawTC( m_list, m_num_tris );
				glDisable( GL_TEXTURE_2D );
				break;

			case TL_TNOFF:
				glEnable( GL_TEXTURE_2D );
				glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, m_tfunc );
				glBindTextureEXT( GL_TEXTURE_2D, m_texobj );
				drawTN( m_list, m_num_tris );
				glDisable( GL_TEXTURE_2D );
				break;

			};

		glEndList();
		}

	if( m_compDispList == 0 )
		{
		m_compDispList = glGenLists( 1 );
		
		glNewList( m_compDispList, GL_COMPILE );

		glMultMatrixf( m_transform.getValue() );

		glCallList( m_geomDispList );

		glEndList();
		}
	}


//---------------------------------------------------------------
//---------------------------------------------------------------
int
TriangleList::drawDL()
	{
	if( m_compDispList == 0 )	return 0;	// Generate an error?

	glPushMatrix();
	glCallList( m_compDispList );
	glPopMatrix();

	return 1;
	}


//---------------------------------------------------------------
//---------------------------------------------------------------
void
TriangleList::draw_wire()
	{
	//this->drawN( m_list, m_num_tris );
	//return;

	glPushMatrix();
	glMultMatrixf( m_transform.getValue() );

		int i;
		for( int j=0 ; j<m_num_tris ; j++ )
			{
			i = m_list[j];

			glBegin( GL_LINE_LOOP );
			glVertex3fv( m_vlist[ m_tlist[i].index[0] ].getValue() );
			glVertex3fv( m_vlist[ m_tlist[i].index[1] ].getValue() );
			glVertex3fv( m_vlist[ m_tlist[i].index[2] ].getValue() );
			glEnd();
		    }

	glPopMatrix();
	}

//---------------------------------------------------------------
//---------------------------------------------------------------
void
TriangleList::draw_psuedoColor()
	{
	glPushMatrix();
	glMultMatrixf( m_transform.getValue() );
	this->drawYELLOW( m_list, m_num_tris );
	glPopMatrix();
	}

//---------------------------------------------------------------
//---------------------------------------------------------------
int
TriangleList::draw()
	{
	this->draw_list( m_list, m_num_tris );
	return 1;
	}

//---------------------------------------------------------------
//---------------------------------------------------------------
void
TriangleList::draw_list( int list[], int num_tris )
	{
	glPushMatrix();
	glMultMatrixf( m_transform.getValue() );

	switch( m_type )
		{
		case TL_OFF:
			drawNC( list, num_tris );
			break;

		case TL_NOFF:
			drawN( list, num_tris );
			break;

		case TL_CNOFF:
			drawNC( list, num_tris );
			break;

		case TL_COFF:
			drawC( list, num_tris );
			break;

		case TL_TCOFF:
			glEnable( GL_TEXTURE_2D );
			glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, m_tfunc );
			glBindTextureEXT( GL_TEXTURE_2D, m_texobj );
			drawTC( list, num_tris );
			glDisable( GL_TEXTURE_2D );
			break;

		case TL_TNOFF:
			glEnable( GL_TEXTURE_2D );
			glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, m_tfunc );
			glBindTextureEXT( GL_TEXTURE_2D, m_texobj );
			drawTN( list, num_tris );
			glDisable( GL_TEXTURE_2D );
			break;

		default:
		fprintf(stderr,"Dont know how to draw this type of OFF: %d\n",m_type);
			break;
		};

	glPopMatrix();
	}


//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
BoundBox
TriangleList::getBoundBox()
	{
	return m_bbox;
	}


//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
BoundBox
TriangleList::getObjectSpaceBoundBox()
	{
	Point3f pt,min,max;

	//
	//	Loop over the vertices
	//
	for( int i=0 ; i<m_num_verts ; i++ )
		{
		//
		//	Assumes transform has already been set !!!!
		//
		pt.setValue(m_vlist[i]);

		if( i==0 )
			{
			min = pt;
			max = pt;
			}
		else
			{
			if      (pt[0] < min[0]) min[0] = pt[0];
			else if (pt[0] > max[0]) max[0] = pt[0]; 

			if      (pt[1] < min[1]) min[1] = pt[1];
			else if (pt[1] > max[1]) max[1] = pt[1]; 

			if      (pt[2] < min[2]) min[2] = pt[2];
			else if (pt[2] > max[2]) max[2] = pt[2]; 
			}
        }
	return( BoundBox(min,max) );
	}


//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
void
TriangleList::computeWSBB_From_OSBB( const BoundBox & os_bbox )
	{
	Point4f	pt;
	Point3f min,max;

	//
	//	Loop over the vertices
	//
	for( int i=0 ; i<8 ; i++ )
		{
		//
		//	Assumes transform has already been set !!!!
		//
		pt.setValue( os_bbox.getPoint(i), 1 );
		pt = m_transform * pt;

		if( i==0 )
			{
			min = pt;
			max = pt;
			}
		else
			{
			if      (pt[0] < min[0]) min[0] = pt[0];
			else if (pt[0] > max[0]) max[0] = pt[0]; 

			if      (pt[1] < min[1]) min[1] = pt[1];
			else if (pt[1] > max[1]) max[1] = pt[1]; 

			if      (pt[2] < min[2]) min[2] = pt[2];
			else if (pt[2] > max[2]) max[2] = pt[2]; 
			}
        }

	m_bbox.setValue( min, max );
	}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
void
TriangleList::computeBoundBox()
	{
	Point4f	pt;
	Point3f min,max;

	//
	//	Loop over the vertices
	//
	for( int i=0 ; i<m_num_verts ; i++ )
		{
		//
		//	Assumes transform has already been set !!!!
		//
		pt.setValue(m_vlist[i],1);
		pt = m_transform * pt;

		if( i==0 )
			{
			min = pt;
			max = pt;
			}
		else
			{
			if      (pt[0] < min[0]) min[0] = pt[0];
			else if (pt[0] > max[0]) max[0] = pt[0]; 

			if      (pt[1] < min[1]) min[1] = pt[1];
			else if (pt[1] > max[1]) max[1] = pt[1]; 

			if      (pt[2] < min[2]) min[2] = pt[2];
			else if (pt[2] > max[2]) max[2] = pt[2]; 
			}
        }

	m_bbox.setValue( min, max );
	}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
void
TriangleList::computeBoundBox_FLOFF()
	{
	Point3f	pt;
	Point3f min,max;
	Point3f p[3];

	//
	//	Loop over the triangles
	//
	for( int i=0 ; i<m_num_tris ; i++ )
		{
		getTriangle( i, p[0], p[1], p[2] );

		for( int j=0 ; j<3 ; j++ )
			{
			//
			//	Assumes transform has already been set !!!!
			//
			pt.setValue( p[j] );

			if( i==0 && j==0 )
				{
				min = pt;
				max = pt;
				}
			else
				{
				if      (pt[0] < min[0]) min[0] = pt[0];
				else if (pt[0] > max[0]) max[0] = pt[0]; 

				if      (pt[1] < min[1]) min[1] = pt[1];
				else if (pt[1] > max[1]) max[1] = pt[1]; 

				if      (pt[2] < min[2]) min[2] = pt[2];
				else if (pt[2] > max[2]) max[2] = pt[2]; 
				}
			}
        }

	m_bbox.setValue( min, max );
	}

//----------------------------------------------------------------------------
//----------------------------------------------------------------------------
void
FinishLine( FILE *fp )
	{
	char achar;

	do
		{
		fscanf(fp,"%c",&achar);
		}
	while (achar != '\n');
	}

//----------------------------------------------------------------------------
//
//      Read an OFF,NOFF,CNOFF file from disk.
//
//----------------------------------------------------------------------------
int	
TriangleList::Read( char *file_name )
    {
    int     num_points;
    int     num_edges;
    int     num_tris;
    int     i,t1,t2,t3,junk;
    double	red, green, blue;
    char    off_type_string[80];
	char	line_buffer[256];
	char	image_file_name[256];
	char	tex_function[80];
    FILE    *dbfile;

    dbfile = fopen( file_name, "r" );

    if( dbfile == NULL )
        {
        fprintf(stderr," error: cant open %s\n", file_name );
        return( 0 );
        }

	//
	//	Eat comments
	//
	fgets( line_buffer, 256, dbfile );
	while( line_buffer[0] == '#' )
		fgets( line_buffer, 256, dbfile );

	//
	//	Read file TYPE
	//
    sscanf( line_buffer, "%s", off_type_string );   // FinishLine(dbfile);
    if(      strcmp( off_type_string, "OFF"   ) == 0 ) m_type = TL_OFF;
    else if( strcmp( off_type_string, "NOFF"  ) == 0 ) m_type = TL_NOFF;
    else if( strcmp( off_type_string, "COFF"  ) == 0 ) m_type = TL_COFF;
    else if( strcmp( off_type_string, "CNOFF" ) == 0 ) m_type = TL_CNOFF;
    else if( strcmp( off_type_string, "TCOFF" ) == 0 ) m_type = TL_TCOFF;
    else if( strcmp( off_type_string, "TNOFF" ) == 0 ) m_type = TL_TNOFF;

	if( m_type == TL_TCOFF || m_type == TL_TNOFF )
		{
		fscanf( dbfile,"%s", image_file_name );
		fscanf( dbfile,"%s", tex_function );
		readTexture( image_file_name, tex_function );
		}

	//
	//	Read triangle list parameters
	//
    fscanf( dbfile, "%d %d %d", &num_points, &num_tris, &num_edges );

	switch( m_type )
		{
		case TL_OFF:
			m_per_vertex_colors  = 0;
			m_per_vertex_normals = 0;
			break;

		case TL_NOFF:
			m_per_vertex_colors  = 0;
			m_per_vertex_normals = 1;
			break;

		case TL_COFF:
			m_per_vertex_colors  = 1;
			m_per_vertex_normals = 0;
			break;

		case TL_CNOFF:
			m_per_vertex_colors  = 1;
			m_per_vertex_normals = 1;
			break;

		case TL_TCOFF:
			m_per_vertex_colors  = 1;
			m_per_vertex_normals = 0;
			break;

		case TL_TNOFF:
			m_per_vertex_colors  = 0;
			m_per_vertex_normals = 1;
			break;

		}

    this->setArrayBounds( num_tris, num_points );


    //
    //  Read the Vertices
    //
    Point3f     p;
    Normal3f    n;
	Point3f		min,max;
	float		r,g,b,a,s,t;

    n.setValue(0.0,1.0,0.0);

    for( i=0 ; i<num_points ; i++ )
        {
        switch( m_type )
            {
            case TL_OFF:
                fscanf(dbfile,"%f %f %f", &p[0],&p[1],&p[2] );
                break;

            case TL_NOFF:
                fscanf(dbfile,"%f %f %f %f %f %f",
                                &p[0],&p[1],&p[2],
                                &n[0],&n[1],&n[2] );
				break;

            case TL_COFF:
                fscanf(dbfile,"%f %f %f %f %f %f %f",
                                &p[0],&p[1],&p[2],
                                &r,&g,&b,&a);
				break;

            //  CNOFF
			//
			//	This may not always work since it
			//	expects a 4-vector color.
			//
            case TL_CNOFF:
                fscanf(dbfile,"%f %f %f  %f %f %f  %f %f %f %f",
                                &p[0],&p[1],&p[2],
                                &n[0],&n[1],&n[2],
								&r,&g,&b,&a);
                break;


            case TL_TCOFF:
                fscanf(dbfile,"%f %f %f  %f %f  %f %f %f %f",
                                &p[0],&p[1],&p[2],
								&s,&t,
                                &r,&g,&b,&a);
				break;


            case TL_TNOFF:
                fscanf(dbfile,"%f %f %f  %f %f %f  %f %f",
                                &p[0],&p[1],&p[2],
                                &n[0],&n[1],&n[2],
								&s,&t );
				break;

            };

        this->setVertex( i, p, n, s, t );
		if( m_per_vertex_colors )
			this->setTriangleColor( i, r,g,b,a );

		//
		//	Assumes transform has already been set !!!!
		//
		Point4f	pt;
		pt.setValue( p, 1 );
		pt = m_transform * pt;

		if( i==0 )
			{
			min = pt;
			max = pt;
			}
		else
			{
			if      (pt[0] < min[0]) min[0] = pt[0];
			else if (pt[0] > max[0]) max[0] = pt[0]; 

			if      (pt[1] < min[1]) min[1] = pt[1];
			else if (pt[1] > max[1]) max[1] = pt[1]; 

			if      (pt[2] < min[2]) min[2] = pt[2];
			else if (pt[2] > max[2]) max[2] = pt[2]; 
			}
        }

	m_bbox.setValue( min, max );

    //
    //  Read the Triangles
    //
    for( i=0 ; i<num_tris ; i++ )
		{
        fscanf(dbfile,"%d %d %d %d",&junk,&t1,&t2,&t3 );

        this->setTriangle( i, t1, t2, t3 );

		//
		//	Read a face color
		//
		if( m_type != TL_CNOFF  &&  m_type != TL_COFF && m_type != TL_TCOFF )
			{
			if ((int)fgetc(dbfile) != 10)
				{
				fscanf(dbfile,"%lf %lf %lf",&red,&green,&blue);
				this->setTriangleColor(i, red, green, blue, 1.0);
				}
			else
				{
				this->setTriangleColor( i, 0.5, 0.5, 0.5, 1.0 );
				}
			}
		}

	fclose(dbfile);

	return( num_tris );
	}


void
TriangleList::readTexture( char *filename, char *tex_function )
{
         if( strcmp( tex_function, "BLEND"    ) == 0 ) m_tfunc = GL_BLEND;
    else if( strcmp( tex_function, "DECAL"    ) == 0 ) m_tfunc = GL_DECAL;
    else if( strcmp( tex_function, "MODULATE" ) == 0 ) m_tfunc = GL_MODULATE;

/*
	ilFileImg *imgfile = ilOpenImgFile( filename, "r" );
	if( imgfile == NULL )
		{
		fprintf(stderr,"Could not open %s\n", filename);
		exit(1);
		}

	ilType datatype = imgfile->getDataType();
	int xdim = imgfile->getXsize();
	int ydim = imgfile->getYsize();
	int numchan = imgfile->getNumChans();
	int size = ilDataSize(datatype,xdim*ydim*numchan);
	GLubyte *imgdata = new GLubyte [size];
	fprintf(stderr,"%d %d %d %d\n", xdim,ydim,numchan,size );
	imgfile->getTile(0,0,xdim,ydim,imgdata);
*/


/*
	// open the file named by 'filename'
	iflStatus sts;
	iflFile* file	= iflFile::open(filename, O_RDONLY, &sts);
	if (sts != iflOKAY) { }

	// read the entire image (just the first plane in z if image has depth)
	// into a buffer of unsiged chars

	iflSize dims;
	file->getDimensions(dims);
	unsigned char* imgdata =	new unsigned char[dims.x*dims.y*dims.c];
	iflConfig cfg(iflUChar, iflInterleaved);
	sts =	file->getTile(0, 0, 0, dims.x, dims.y, 1, imgdata,	&cfg);
	if (sts != iflOKAY) { }

	// close the file
	file->close();
*/

	glEnable( GL_TEXTURE_2D );

	glGenTexturesEXT( 1, &m_texobj );

	glBindTextureEXT( GL_TEXTURE_2D, m_texobj );

	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
//	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
//	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
/*
	if( dims.c == 1 )
		glTexImage2D( GL_TEXTURE_2D, 0, GL_LUMINANCE, dims.x, dims.y,
					0, GL_LUMINANCE, GL_UNSIGNED_BYTE, imgdata );

	else if( dims.c == 2 )
		glTexImage2D( GL_TEXTURE_2D, 0, GL_LUMINANCE_ALPHA, dims.x, dims.y,
					0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, imgdata );

	else if( dims.c == 3 )
		glTexImage2D( GL_TEXTURE_2D, 0, GL_RGB, dims.x, dims.y,
					0, GL_RGB, GL_UNSIGNED_BYTE, imgdata );

	else if( dims.c == 4 )
		glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, dims.x, dims.y,
					0, GL_RGBA, GL_UNSIGNED_BYTE, imgdata );
*/

	glDisable( GL_TEXTURE_2D );

//	delete imgdata;

	{
	GLenum error_code;
		while((error_code=glGetError()) != GL_NO_ERROR)
			fprintf(stderr,"gl error: %s\n",gluErrorString(error_code));
	}

}
