#ifndef _TRIANGLELIST_H
#define _TRIANGLELIST_H

#include "Geometry.h"
#include "VisualObject.h"

class TriangleList : public VisualObject {

public:

					TriangleList ();

					TriangleList ( int num_tris, int num_vertices,
								   int per_vertex_colors,
					               int per_vertex_normals );

					TriangleList ( int num_tris, int num_verts,
									Point3f *va, float *stlist,
									Normal3f *na, Color *ca, Triangle *ta );

					TriangleList ( const TriangleList &t);

					~TriangleList();

			void	initFLOFF( const TriangleList &t, char *filename );

			void	initMaterial();

	virtual int		draw();
			int		drawDL();
			void	draw_list( int list[], int num_tris );
			void	draw_wire();
			void	draw_psuedoColor();

			void	setVertex        (int i, const Point3f & point,
						                     const Normal3f & normal,
											 float s, float t );
			void	setTriangle      (int i, int v0, int v1, int v2 ); 
			void    setTriangleColor (int i, float red,  float green,
				                             float blue, float alpha);

			void	setArrayBounds   ( int num_tris, int num_vertices );
			void	setMaterial      ( Material mat );
			void	setMaterial      ( float mat[16] );
			void	setTransform     ( Matrix4f matrix );
			void	setCreateNormals ( int flag );

			void	setGeomDispList( int disp_list );
			int		getGeomDispList();

			int		getBytes();
			int		getNumTriangles();
			int		getNumVertices();
			void	getTriangle( int i, Point3d &p1, Point3d &p2, Point3d &p3 );
			void	getTriangle( int i, Point3f &p1, Point3f &p2, Point3f &p3 );

	virtual	BoundBox	getBoundBox();
			BoundBox	getObjectSpaceBoundBox();

			Point3f&	operator [] ( int i );	

			void	computeBoundBox();
			void	computeBoundBox_FLOFF();
			void	computeWSBB_From_OSBB( const BoundBox & os_bbox );

			void	createDisplayList();
			void	deallocRawData();

			int		Read( char *filename );

private:

			void	drawV  ( int list[], int num_tris );
			void	drawT  ( int list[], int num_tris );
			void	drawTC ( int list[], int num_tris );
			void	drawTN ( int list[], int num_tris );
			void	drawTNC( int list[], int num_tris );
			void	drawC  ( int list[], int num_tris );
			void	drawNC ( int list[], int num_tris );
			void	drawN  ( int list[], int num_tris );

			void	drawYELLOW ( int list[], int num_tris );

			void	readTexture( char *filename, char *tex_function );

	GLuint		m_geomDispList;		// Just the triangles
	GLuint		m_compDispList;		// Includes material,transform
	GLuint		m_texobj;
	GLfloat		m_tfunc;

	int			m_floff;
	int			m_type;
	int			m_num_tris;
	int			m_num_verts;
	int			m_per_vertex_colors;
	int			m_per_vertex_normals;
	int			*m_list;

	Matrix4f	m_transform;
	BoundBox	m_bbox;
	Material	m_mat;

	float		*m_stlist;
	Point3f		*m_vlist;
	Normal3f	*m_nlist;
	Color		*m_clist;
	Triangle	*m_tlist;
};


#endif


