//============================================================================
//
//		Vector3d
//
//============================================================================

#ifndef _VECTOR3D_H
#define _VECTOR3D_H

class Vector3f;
class Vector4d;

class Vector3d
	{

	friend Vector3d operator *  (const Vector3d &v1, const Vector3d &v2 );
	friend Vector3d operator *  (const Vector3d &v, double d);
	friend Vector3d operator *  ( double d, const Vector3d &v );
	friend Vector3d operator /  (const Vector3d &v, double d);
	friend Vector3d operator +  (const Vector3d &v1, const Vector3d &v2);
	friend Vector3d operator -  (const Vector3d &v1, const Vector3d &v2);
	friend int	    operator == (const Vector3d &v1, const Vector3d &v2);

public:
	
					Vector3d();
					Vector3d( const double v[3] );
					Vector3d( double x, double y, double z );
//					Vector3d( const Vector3f &v );
					Vector3d( const Vector4d &v );

	Vector3d & 		setValue( const double v[3] );
	Vector3d & 		setValue( double x, double y, double z );

	const double*	getValue() const;
	void			getValue( double &x, double &y, double &z ) const;

	double &		operator[](int i);
	const double &	operator[](int i) const;

	double			dot  ( const Vector3d &v ) const;
	Vector3d		cross( const Vector3d &v ) const;

	void			rotateY( double angle );

	double			length() const;
	void			negate();
	double			normalize();

	Vector3d		lesserComponents ( const Vector3d &v );
	Vector3d		greaterComponents( const Vector3d &v );

	Vector3d &		operator += ( const Vector3d &v );
	Vector3d &		operator -= ( const Vector3d &v );
	Vector3d &		operator *= ( const Vector3d &v );
	Vector3d &		operator *= ( double d );
	Vector3d &		operator /= ( double d );

	Vector3d		operator - () const;

	void			Print() const;

private:
		
		double x,y,z;
	};


#endif


