//============================================================================
//
//		Vector3f
//
//============================================================================

#ifndef _VECTOR3F_H
#define _VECTOR3F_H

class Vector3d;
class Vector4f;

class Vector3f
	{

	friend Vector3f operator *  (const Vector3f &v1, const Vector3f &v2 );
	friend Vector3f operator *  (const Vector3f &v, float d);
	friend Vector3f operator *  ( float d, const Vector3f &v );
	friend Vector3f operator /  (const Vector3f &v, float d);
	friend Vector3f operator +  (const Vector3f &v1, const Vector3f &v2);
	friend Vector3f operator -  (const Vector3f &v1, const Vector3f &v2);
	friend int	    operator == (const Vector3f &v1, const Vector3f &v2);

public:
	
					Vector3f();
					Vector3f( const float v[3] );
					Vector3f( float x, float y, float z );
					Vector3f( const Vector3f &v );
					Vector3f( const Vector4f &v );

	Vector3f & 		setValue( const Vector3f &v );
	Vector3f & 		setValue( const float v[3] );
	Vector3f & 		setValue( float x, float y, float z );

	const float*	getValue() const;
	void			getValue( float &x, float &y, float &z ) const;

	float &		operator[](int i);
	const float &	operator[](int i) const;

	float			dot  ( const Vector3f &v ) const;
	Vector3f		cross( const Vector3f &v ) const;

	void			rotateY( float angle );

	float			length() const;
	void			negate();
	float			normalize();

	Vector3f		lesserComponents ( const Vector3f &v );
	Vector3f		greaterComponents( const Vector3f &v );

	Vector3f &		operator += ( const Vector3f &v );
	Vector3f &		operator -= ( const Vector3f &v );
	Vector3f &		operator *= ( const Vector3f &v );
	Vector3f &		operator *= ( float d );
	Vector3f &		operator /= ( float d );

	Vector3f		operator - () const;

	void			Print() const;

private:
		
		float x,y,z;
	};


#endif


