//============================================================================
//
//		Vector4d
//
//============================================================================

#include <stdio.h>
#include <math.h>
#include "Vector4d.h"

//----------------------------------------------------------------------------
//
//		Constructors !!
//
//
//----------------------------------------------------------------------------
Vector4d::Vector4d()
	{
	x = y = z = w = 0.0;
	}

Vector4d::Vector4d( const double v[4] )
	{
	x = v[0]; y = v[1]; z = v[2]; w = v[3];
	}

Vector4d::Vector4d( const double v[3], double ww )
	{
	x = v[0]; y = v[1]; z = v[2]; w = ww;
	}

Vector4d::Vector4d( Vector3d v, double ww )
	{
	x = v[0]; y = v[1]; z = v[2]; w = ww;
	}

Vector4d::Vector4d( double xx, double yy, double zz, double ww )
	{
	x = xx; y = yy; z = zz; w = ww;
	}

//----------------------------------------------------------------------------
//
//	Set Value
//
//----------------------------------------------------------------------------
Vector4d &	Vector4d::setValue( const double v[4] )
	{
	x = v[0]; y = v[1]; z = v[2]; w = v[3];

	return *this;
	}

Vector4d &	Vector4d::setValue( const double v[3], double ww )
	{
	x = v[0]; y = v[1]; z = v[2]; w = ww;

	return *this;
	}

Vector4d &	Vector4d::setValue( Vector3d v, double ww )
	{
	x = v[0]; y = v[1]; z = v[2]; w = ww;

	return *this;
	}

Vector4d &	Vector4d::setValue( double xx, double yy, double zz, double ww )
	{
	x = xx; y = yy; z = zz; w = ww;

	return *this;
	}

//----------------------------------------------------------------------------
//
//		getValue
//
//----------------------------------------------------------------------------
const double*	Vector4d::getValue() const
	{
	return( (double *)this );
	}


void	Vector4d::getValue( double &xx, double &yy,
							double &zz, double &ww ) const
	{
	xx = x; yy = y; zz = z; ww = w;
	}

double &	Vector4d::operator[]( int i )
	{
	return( ((double *) this)[i] );
	}

const double &	Vector4d::operator[]( int i ) const
	{
	return( ((double *) this)[i] );
	}


//----------------------------------------------------------------------------
//
//		dot
//
//----------------------------------------------------------------------------
double	Vector4d::dot( const Vector4d &v ) const
	{
	return( x*v.x + y*v.y + z*v.z + w*v.w );
	}




//----------------------------------------------------------------------------
//
//		rotateY
//
//----------------------------------------------------------------------------
void	Vector4d::rotateY( double angle )
	{
	double tx,ty,tz;

	tx =  x * cos( angle ) + z * sin( angle );
	ty =  y;
	tz = -x * sin( angle ) + z * cos( angle );

	x = tx; y = ty; z = tz;
	}


//----------------------------------------------------------------------------
//
//		length
//
//----------------------------------------------------------------------------
double	Vector4d::length() const
	{
	return( sqrt( x*x + y*y + z*z + w*w ) );
	}

//----------------------------------------------------------------------------
//
//		negate
//
//----------------------------------------------------------------------------
void	Vector4d::negate()
	{
	x = -x; y = -y; z = -z; w = -w;
	}

//----------------------------------------------------------------------------
//
//		normalize
//
//----------------------------------------------------------------------------
double	Vector4d::normalize()
	{
	double length = sqrt( x*x + y*y + z*z + w*w );

	x /= length;
	y /= length;
	z /= length;
	w /= length;

	return( length );
	}


//----------------------------------------------------------------------------
//
//		+=
//
//----------------------------------------------------------------------------
Vector4d &	Vector4d::operator += ( const Vector4d &v )
	{
	x += v.x;
	y += v.y;
	z += v.z;
	w += v.w;

	return *this;
	}

//----------------------------------------------------------------------------
//
//		-=
//
//----------------------------------------------------------------------------
Vector4d &	Vector4d::operator -= ( const Vector4d &v )
	{
	x -= v.x;
	y -= v.y;
	z -= v.z;
	w -= v.w;

	return *this;
	}

//----------------------------------------------------------------------------
//
//		*=
//
//----------------------------------------------------------------------------
Vector4d &	Vector4d::operator *= ( const Vector4d &v )
	{
	x *= v.x;
	y *= v.y;
	z *= v.z;
	w *= v.w;

	return *this;
	}

Vector4d &	Vector4d::operator *= ( double d )
	{
	x *= d;
	y *= d;
	z *= d;
	w *= d;

	return *this;
	}

//----------------------------------------------------------------------------
//
//		/=
//
//----------------------------------------------------------------------------
Vector4d &	Vector4d::operator /= ( double d )
	{
	x /= d;
	y /= d;
	z /= d;
	w /= d;

	return *this;
	}

//----------------------------------------------------------------------------
//
//		Non-destructive unary negation
//
//----------------------------------------------------------------------------
Vector4d	Vector4d::operator - () const
	{
	Vector4d vec( -x, -y, -z, -w );
	
	return vec;
	}

//----------------------------------------------------------------------------
//
//		Print
//
//----------------------------------------------------------------------------
void	Vector4d::Print() const
	{
	fprintf( stderr,"%6.2f %6.2f %6.2f %6.2f\n", x,y,z,w );
	}

//----------------------------------------------------------------------------
//
//		Friend operators
//
//----------------------------------------------------------------------------

Vector4d	operator *( const Vector4d &v1, const Vector4d &v2 )
	{
	Vector4d vec( v1.x*v2.x, v1.y*v2.y, v1.z*v2.z, v1.w*v2.w );

	return vec;
	}

Vector4d	operator *( const Vector4d &v, double d )
	{
	Vector4d vec( v.x*d, v.y*d, v.z*d, v.w*d );

	return vec;
	}

Vector4d	operator *( double d, const Vector4d &v )
	{
	Vector4d vec( v.x*d, v.y*d, v.z*d, v.w*d );

	return vec;
	}

Vector4d	operator /( const Vector4d &v, double d )
	{
	Vector4d vec( v.x/d, v.y/d, v.z/d, v.w/d );

	return vec;
	}

Vector4d	operator +( const Vector4d &v1, const Vector4d &v2  )
	{
	Vector4d vec( v1.x+v2.x, v1.y+v2.y, v1.z+v2.z, v1.w+v2.w );

	return vec;
	}

Vector4d	operator -( const Vector4d &v1, const Vector4d &v2  )
	{
	Vector4d vec( v1.x-v2.x, v1.y-v2.y, v1.z-v2.z, v1.w-v2.w );

	return vec;
	}


