//============================================================================
//
//		Vector4d
//
//============================================================================

#ifndef _VECTOR4D_H
#define _VECTOR4D_H

#include "Vector3d.h"

class Vector4d
	{

	friend Vector4d operator * (const Vector4d &v1, const Vector4d &v2 );
	friend Vector4d operator * (const Vector4d &v, double d);
	friend Vector4d operator * ( double d, const Vector4d &v );
	friend Vector4d operator / (const Vector4d &v, double d);
	friend Vector4d operator + (const Vector4d &v1, const Vector4d &v2);
	friend Vector4d operator - (const Vector4d &v1, const Vector4d &v2);

public:
	
					Vector4d();
					Vector4d( const double v[4] );
					Vector4d( const double v[3], double w );
					Vector4d( Vector3d v, double w );
					Vector4d( double x, double y, double z, double w );

	Vector4d & 		setValue( const double v[4] );
	Vector4d & 		setValue( const double v[3], double w );
	Vector4d & 		setValue( Vector3d v, double w );
	Vector4d & 		setValue( double x, double y, double z, double w );

	const double*	getValue() const;
	void			getValue( double &x, double &y, double &z, double &w ) const;

	double &		operator[](int i);
	const double &	operator[](int i) const;

	double			dot  ( const Vector4d &v ) const;

	void			rotateY( double angle );

	double			length() const;
	void			negate();
	double			normalize();

	Vector4d &		operator += ( const Vector4d &v );
	Vector4d &		operator -= ( const Vector4d &v );
	Vector4d &		operator *= ( const Vector4d &v );
	Vector4d &		operator *= ( double d );
	Vector4d &		operator /= ( double d );

	Vector4d		operator - () const;

	void			Print() const;

private:
		
		double x,y,z,w;
	};


#endif


