//============================================================================
//
//		Vector4f
//
//============================================================================

#include <stdio.h>
#include <math.h>
#include "Vector4f.h"

//----------------------------------------------------------------------------
//
//		Constructors !!
//
//
//----------------------------------------------------------------------------
Vector4f::Vector4f()
	{
	x = y = z = w = 0.0;
	}

Vector4f::Vector4f( const float v[4] )
	{
	x = v[0]; y = v[1]; z = v[2]; w = v[3];
	}

Vector4f::Vector4f( const float v[3], float ww )
	{
	x = v[0]; y = v[1]; z = v[2]; w = ww;
	}

Vector4f::Vector4f( Vector3f v, float ww )
	{
	x = v[0]; y = v[1]; z = v[2]; w = ww;
	}

Vector4f::Vector4f( float xx, float yy, float zz, float ww )
	{
	x = xx; y = yy; z = zz; w = ww;
	}

//----------------------------------------------------------------------------
//
//	Set Value
//
//----------------------------------------------------------------------------
Vector4f &	Vector4f::setValue( const float v[4] )
	{
	x = v[0]; y = v[1]; z = v[2]; w = v[3];

	return *this;
	}

Vector4f &	Vector4f::setValue( const float v[3], float ww )
	{
	x = v[0]; y = v[1]; z = v[2]; w = ww;

	return *this;
	}

Vector4f &	Vector4f::setValue( Vector3f v, float ww )
	{
	x = v[0]; y = v[1]; z = v[2]; w = ww;

	return *this;
	}

Vector4f &	Vector4f::setValue( Vector3d v, float ww )
	{
	x = (float)v[0]; y = (float)v[1]; z = (float)v[2]; w = ww;

	return *this;
	}

Vector4f &	Vector4f::setValue( float xx, float yy, float zz, float ww )
	{
	x = xx; y = yy; z = zz; w = ww;

	return *this;
	}

//----------------------------------------------------------------------------
//
//		getValue
//
//----------------------------------------------------------------------------
const float*	Vector4f::getValue() const
	{
	return( (float *)this );
	}


void	Vector4f::getValue( float &xx, float &yy,
							float &zz, float &ww ) const
	{
	xx = x; yy = y; zz = z; ww = w;
	}

float &	Vector4f::operator[]( int i )
	{
	return( ((float *) this)[i] );
	}

const float &	Vector4f::operator[]( int i ) const
	{
	return( ((float *) this)[i] );
	}


//----------------------------------------------------------------------------
//
//		dot
//
//----------------------------------------------------------------------------
float	Vector4f::dot( const Vector4f &v ) const
	{
	return( x*v.x + y*v.y + z*v.z + w*v.w );
	}




//----------------------------------------------------------------------------
//
//		rotateY
//
//----------------------------------------------------------------------------
void	Vector4f::rotateY( float angle )
	{
	float tx,ty,tz;

	tx =  x * cos( angle ) + z * sin( angle );
	ty =  y;
	tz = -x * sin( angle ) + z * cos( angle );

	x = tx; y = ty; z = tz;
	}


//----------------------------------------------------------------------------
//
//		length
//
//----------------------------------------------------------------------------
float	Vector4f::length() const
	{
	return( sqrt( x*x + y*y + z*z + w*w ) );
	}

//----------------------------------------------------------------------------
//
//		negate
//
//----------------------------------------------------------------------------
void	Vector4f::negate()
	{
	x = -x; y = -y; z = -z; w = -w;
	}

//----------------------------------------------------------------------------
//
//		normalize
//
//----------------------------------------------------------------------------
float	Vector4f::normalize()
	{
	float length = sqrt( x*x + y*y + z*z + w*w );

	x /= length;
	y /= length;
	z /= length;
	w /= length;

	return( length );
	}


//----------------------------------------------------------------------------
//
//		+=
//
//----------------------------------------------------------------------------
Vector4f &	Vector4f::operator += ( const Vector4f &v )
	{
	x += v.x;
	y += v.y;
	z += v.z;
	w += v.w;

	return *this;
	}

//----------------------------------------------------------------------------
//
//		-=
//
//----------------------------------------------------------------------------
Vector4f &	Vector4f::operator -= ( const Vector4f &v )
	{
	x -= v.x;
	y -= v.y;
	z -= v.z;
	w -= v.w;

	return *this;
	}

//----------------------------------------------------------------------------
//
//		*=
//
//----------------------------------------------------------------------------
Vector4f &	Vector4f::operator *= ( const Vector4f &v )
	{
	x *= v.x;
	y *= v.y;
	z *= v.z;
	w *= v.w;

	return *this;
	}

Vector4f &	Vector4f::operator *= ( float d )
	{
	x *= d;
	y *= d;
	z *= d;
	w *= d;

	return *this;
	}

//----------------------------------------------------------------------------
//
//		/=
//
//----------------------------------------------------------------------------
Vector4f &	Vector4f::operator /= ( float d )
	{
	x /= d;
	y /= d;
	z /= d;
	w /= d;

	return *this;
	}

//----------------------------------------------------------------------------
//
//		Non-destructive unary negation
//
//----------------------------------------------------------------------------
Vector4f	Vector4f::operator - () const
	{
	Vector4f vec( -x, -y, -z, -w );
	
	return vec;
	}

//----------------------------------------------------------------------------
//
//		Print
//
//----------------------------------------------------------------------------
void	Vector4f::Print() const
	{
	fprintf( stderr,"%6.2f %6.2f %6.2f %6.2f\n", x,y,z,w );
	}

//----------------------------------------------------------------------------
//
//		Friend operators
//
//----------------------------------------------------------------------------

Vector4f	operator *( const Vector4f &v1, const Vector4f &v2 )
	{
	Vector4f vec( v1.x*v2.x, v1.y*v2.y, v1.z*v2.z, v1.w*v2.w );

	return vec;
	}

Vector4f	operator *( const Vector4f &v, float d )
	{
	Vector4f vec( v.x*d, v.y*d, v.z*d, v.w*d );

	return vec;
	}

Vector4f	operator *( float d, const Vector4f &v )
	{
	Vector4f vec( v.x*d, v.y*d, v.z*d, v.w*d );

	return vec;
	}

Vector4f	operator /( const Vector4f &v, float d )
	{
	Vector4f vec( v.x/d, v.y/d, v.z/d, v.w/d );

	return vec;
	}

Vector4f	operator +( const Vector4f &v1, const Vector4f &v2  )
	{
	Vector4f vec( v1.x+v2.x, v1.y+v2.y, v1.z+v2.z, v1.w+v2.w );

	return vec;
	}

Vector4f	operator -( const Vector4f &v1, const Vector4f &v2  )
	{
	Vector4f vec( v1.x-v2.x, v1.y-v2.y, v1.z-v2.z, v1.w-v2.w );

	return vec;
	}


