//============================================================================
//
//		Vector4f
//
//============================================================================

#ifndef _VECTOR4F_H
#define _VECTOR4F_H

#include "Vector3f.h"
#include "Vector3d.h"

class Vector4f
	{

	friend Vector4f operator * (const Vector4f &v1, const Vector4f &v2 );
	friend Vector4f operator * (const Vector4f &v, float d);
	friend Vector4f operator * ( float d, const Vector4f &v );
	friend Vector4f operator / (const Vector4f &v, float d);
	friend Vector4f operator + (const Vector4f &v1, const Vector4f &v2);
	friend Vector4f operator - (const Vector4f &v1, const Vector4f &v2);

public:
	
					Vector4f();
					Vector4f( const float v[4] );
					Vector4f( const float v[3], float w );
					Vector4f( Vector3f v, float w );
					Vector4f( float x, float y, float z, float w );

	Vector4f & 		setValue( const float v[4] );
	Vector4f & 		setValue( const float v[3], float w );
	Vector4f & 		setValue( Vector3f v, float w );
	Vector4f & 		setValue( Vector3d v, float w );
	Vector4f & 		setValue( float x, float y, float z, float w );

	const float*	getValue() const;
	void			getValue( float &x, float &y, float &z, float &w ) const;

	float &		operator[](int i);
	const float &	operator[](int i) const;

	float			dot  ( const Vector4f &v ) const;

	void			rotateY( float angle );

	float			length() const;
	void			negate();
	float			normalize();

	Vector4f &		operator += ( const Vector4f &v );
	Vector4f &		operator -= ( const Vector4f &v );
	Vector4f &		operator *= ( const Vector4f &v );
	Vector4f &		operator *= ( float d );
	Vector4f &		operator /= ( float d );

	Vector4f		operator - () const;

	void			Print() const;

private:
		
		float x,y,z,w;
	};


#endif


