//============================================================================
//
//		animate.c
//
//		Animation Callbacks
//
//			Choose Image
//			Choose Geometry
//			Choose Record
//			Go
//			Stop
//
//
//			Jonathan Shade
//
//============================================================================

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>

#include <iostream.h>

#include <Xm/Xm.h>

#include <GL/glx.h>
#ifdef __sgi
#include <GL/GLwMDrawA.h>
#else
#include <X11/GLw/GLwDrawA.h>
#endif

//#include <il/ilGenericImgFile.h>
//#include <il/ilMemoryImg.h>
#  include <il/ilError.h>
#  include <il/ilFileImg.h>
#  include <il/ilGrayImg.h>
#  include <il/ilMemoryImg.h>
#  include <il/ilRGBImg.h>

#include "Bspline.h"
#include "Camera.h"
#include "state.h"
#include "stats.h"
#include "ui.h"
#include "animate.h"

double	DELTA_T;
double	DELTA_ROT;

#define TREE_HEIGHT		120.0

GLubyte	*screen_buffer;
char	datafile[256];
double	height_field[129][129];	// [Z][X]
double	hf_width = 7680.0;

//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
void
animate_init( AppState & app_state )
	{
	sprintf( datafile,"%s", app_state.anim_state.curve_file );
	app_state.gr_state.bspline = new Bspline( datafile );

	DELTA_T		= 1.0/(double)(app_state.anim_state.num_frames-1);
	DELTA_ROT	= 3.14159/360.0;
	}


//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
void
animate_read_curve( Widget, XtPointer client_data, XtPointer )
	{
	AppState *app_state = (AppState *) ((uiRenderWindow *)client_data)->uiData;
	GraphicsState & gr_state = app_state->gr_state;

	if( gr_state.bspline != NULL )
		delete gr_state.bspline;

	gr_state.bspline = new Bspline( datafile );
	}

//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
void
animate_choose_preview( Widget, XtPointer client_data, XtPointer )
	{
	AppState *state = (AppState *) ((uiRenderWindow *)client_data)->uiData;

	if( state->anim_state.win_list[0] )
		state->anim_state.win_list[0] = 0;
	else
		state->anim_state.win_list[0] = 1;
	}

//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
void
animate_choose_geometry( Widget, XtPointer client_data, XtPointer )
	{
	AppState *state = (AppState *) ((uiRenderWindow *)client_data)->uiData;

	if( state->anim_state.win_list[1] )
		state->anim_state.win_list[1] = 0;
	else
		state->anim_state.win_list[1] = 1;
	}

//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
void
animate_choose_image( Widget, XtPointer client_data, XtPointer )
	{
	AppState *state = (AppState *) ((uiRenderWindow *)client_data)->uiData;

	if( state->anim_state.win_list[2] )
		state->anim_state.win_list[2] = 0;
	else
		state->anim_state.win_list[2] = 1;
	}

//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
void
animate_choose_record( Widget, XtPointer client_data, XtPointer )
	{
	AppState *state = (AppState *) ((uiRenderWindow *)client_data)->uiData;

	if( state->anim_state.record )
		state->anim_state.record = 0;
	else
		state->anim_state.record = 1;
	}

//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
void
animate_print_camera( Widget, XtPointer client_data, XtPointer )
	{
	AppState *app_state = (AppState *) ((uiRenderWindow *)client_data)->uiData;

	app_state->gr_state.camera->Print();
	}


//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
void
animate_start( Widget, XtPointer client_data, XtPointer )
	{
	animate_start_batch( (uiRenderWindow *)client_data  );
	}

//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
void
animate_start_batch( uiRenderWindow *ren_win )
	{
	AppState *app_state = (AppState *) ren_win->uiData;
	FILE *fp =  app_state->stats.fp;

	if( fp != NULL )
		{
		fclose( fp );
		fp = NULL;
		}

	fp = fopen( "stats.dat", "w" );
	if( fp == NULL )
		{
		fprintf(stderr,"Could not open stats.dat for writing.\n");
		exit(0);
		}

	app_state->stats.num_frames = app_state->anim_state.num_frames;
	app_state->stats.fp = fp;

	//
	//	Register the animation procedure
	//
	app_state->anim_state.param = 0.0;
	app_state->anim_state.animating = 1;
	app_state->anim_state.current_frame = app_state->anim_state.starting_frame;
	app_state->anim_state.work_proc_id = XtAppAddWorkProc( 
										ren_win->app,
										animate_render,
										ren_win ); 
	}

//----------------------------------------------------------------------------
//
//
// 
//----------------------------------------------------------------------------
void
animate_stop( Widget, XtPointer client_data, XtPointer )
	{
	AppState *app_state = (AppState *) ((uiRenderWindow *)client_data)->uiData;

	//
	//	Unregister the animation procedure
	//
	XtRemoveWorkProc( app_state->anim_state.work_proc_id );

	app_state->anim_state.param = 1.0;
	app_state->anim_state.animating = 0;
	app_state->anim_state.speed = app_state->gr_state.zoomSpeed;

	fclose( app_state->stats.fp );
	app_state->stats.fp = NULL;
	}

//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
void
animate_reset_cam( Widget, XtPointer client_data, XtPointer )
	{
	AppState *app_state =
		(AppState *) ((uiRenderWindow *)client_data)->uiData;

	GraphicsState & gr_state	= app_state->gr_state;

	uiRenderWindow	*ren_win   = (uiRenderWindow *) client_data;

	if( ren_win->needsInit )
		return;

	//
	//	This updates the cameras of all the windows
	//	since all of the camera pointers are aliased
	//	to one actual camera instance.
	//
	gr_state.camera->Reset();
	gr_state.camera->Place( gr_state.ic_x, gr_state.ic_y, gr_state.ic_z );
	gr_state.camera->Update();

	for ( int i=0 ; i<3 ; i++ )
		{
		if( app_state->anim_state.win_list[i] )
			{
			ren_win = app_state->uiWindowList[i]; 
			//
			//	Render the window just changed, and
			//
			uiPostRender( ren_win );

			//
			//	Check Constraint List for other Windows to Render
			//
			for( int j=0 ; j<ren_win->num_constraints ; j++ )
				{
				uiRenderWindow *cstr = &(ren_win->constraint_list[j]);

				if( app_state->activeWindowList[cstr->window_id] ) 
				uiRenderImmediately( cstr );
				}
			}
		}
	}

//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
Boolean
animate_render( XtPointer client_data )
	{
	uiRenderWindow	*ren_win = (uiRenderWindow *) client_data;

	AppState *app_state = (AppState *) ren_win->uiData;

	GraphicsState & gr_state	= app_state->gr_state;

	if( ren_win->needsInit )
		{
		glXMakeCurrent( ren_win->dpy,
						XtWindow(ren_win->glxarea),
						ren_win->cx);

		(ren_win->init)( ren_win );
		ren_win->needsInit = 0;
		}

	if( screen_buffer == NULL )
		screen_buffer = new GLubyte [640*480*4];
	
	if( app_state->anim_state.current_frame <
			app_state->anim_state.ending_frame )
		{
		//
		//	Update the camera
		//
		Point3d  c = gr_state.camera->getPosition();
		Vector3d l = gr_state.camera->getLook();
		Vector3d ta; 
	
		float t = DELTA_T * (float) app_state->anim_state.current_frame;

		//
		//	position
		//	
		double x = gr_state.bspline->eval( t, 0 );
		double y = gr_state.bspline->eval( t, 1 );
		double z = gr_state.bspline->eval( t, 2 );

		double dx = gr_state.bspline->eval( t+DELTA_T, 0 );
		double dy = gr_state.bspline->eval( t+DELTA_T, 1 );
		double dz = gr_state.bspline->eval( t+DELTA_T, 2 );

		app_state->anim_state.speed
			= sqrt( (dx-x)*(dx-x) + (dy-y)*(dy-y) + (dz-z)*(dz-z) );

		//
		//	look-at vector
		//
		if( app_state->anim_state.lookat == 0 )
			{
			ta[0] = gr_state.bspline->eval_tan( t, 0 );
			ta[1] = l[1];
			ta[2] = gr_state.bspline->eval_tan( t, 2 );
			ta.normalize();

			double cosa	= l.dot( ta );
			double roty = acos( cosa );

			Vector3d cross = l.cross( ta );
			if( cross[1] < 0.0 )
				roty = -roty;

			gr_state.camera->Place( x, y, z );
			gr_state.camera->Rotate( roty );
			}
		else if( app_state->anim_state.lookat == 1 ) 
			{
			ta[0] = gr_state.bspline->eval_tan( t, 0 );
			ta[1] = gr_state.bspline->eval_tan( t, 1 );
			ta[2] = gr_state.bspline->eval_tan( t, 2 );
			ta.normalize();

			Vector3d up( 0,1,0 );
			Vector3d bi;

			bi = ta.cross( up );
			bi.normalize();

			gr_state.camera->Place( x, y, z );
			gr_state.camera->Look( bi[0], bi[1], bi[2] );
			}
		else if(  app_state->anim_state.lookat == 2 )
			{
			ta[0] = gr_state.bspline->eval_tan( t, 0 );
			ta[1] = l[1];
			ta[2] = gr_state.bspline->eval_tan( t, 2 );
			ta.normalize();

			Vector3d up( 0,1,0 );
			Vector3d bi;

			bi = -ta.cross( up );
			bi.normalize();

			Vector3d slant( -0.42, 0, -0.91 );	// 25 degrees
			slant.normalize();

			Vector3d gaze;
			if( t < 0.30 )	gaze = bi;

			else if( t <= 0.60 )
				{
				double u = (t-0.30) / 0.30;
				gaze = u * slant + (1.0 - u) * bi;
				gaze.normalize();
				}
			else if( t <= 0.70 )
				{
				gaze = slant;
				}
			else if( t <= 1.0 )
				{
				double u = (t-0.70) / 0.35;		// .35 instead of .30
				gaze = u * ta + (1.0 - u) * slant;
				gaze.normalize();
				}
			else
				gaze = ta;

			double cosa	= l.dot( gaze );
			double roty = acos( cosa );

			Vector3d cross = l.cross( gaze );
			if( cross[1] < 0.0 )
				roty = -roty;

			gr_state.camera->Place( x, y, z );
			
			if( t<=0.60 || t>0.70 )
				gr_state.camera->Rotate( roty );
			}

		gr_state.camera->Update();

		app_state->anim_state.param = t;

		//
		//	Loop through all the windows
		//
		for ( int i=0 ; i<3 ; i++ )
			{
			if( app_state->anim_state.win_list[i] )
				{
				ren_win = app_state->uiWindowList[i]; 

				uiRenderImmediately( ren_win );

				//
				//	Check Constraint List for other Windows to Render
				//
				for( int j=0 ; j<ren_win->num_constraints ; j++ )
					{
					uiRenderWindow *cstr = &(ren_win->constraint_list[j]);

					if( app_state->activeWindowList[cstr->window_id] ) 
						uiRenderImmediately( cstr );
					}

				if(  app_state->anim_state.record )
				//	(app_state->anim_state.current_frame%40) == 0 )
					{
#if 0
					glReadPixels( 	0, 0,
									640,
									480,
									//ren_win->glxarea_width,
									//ren_win->glxarea_height,
									GL_RGB,
									GL_UNSIGNED_BYTE, screen_buffer );

					//ilSize    size( ren_win->glxarea_width,
					//				ren_win->glxarea_height,
					//				3 );

					ilSize    size( 640,480,3 );

					char str[256];
					sprintf(str,"%s/img%.4d.rgb",
						app_state->anim_state.movie_dir,
						app_state->anim_state.current_frame);


					ilFileImg *out_file = ilCreateImgFile( str, size, ilUChar,
															ilInterleaved,
															ilSGI_IMG ); 


					ilMemoryImg *mem_img = new ilMemoryImg( screen_buffer,
															size,
															ilUChar,
															ilInterleaved );


					out_file->setCompression( ilSGIRLE );
					out_file->copyTile(0,0,size.x,size.y,mem_img,0,0,0,TRUE);

					delete out_file;
					delete mem_img;
#else
     // Create memory image.
     iflSize size(640, 480, 3);
     ilMemoryImg mem_img(screen_buffer, size, iflUChar, iflInterleaved);
     mem_img.setOrientation(iflLowerLeftOrigin);
     mem_img.setColorModel(iflRGB);

     // Install silent error handler.
     ilErrorHandlerType handler;
     void *arg;
     ilGetErrorHandler(&handler, &arg);
     ilSetErrorHandler(ilSilentErrorHandler, NULL);

     // Create file image.
 
	 char str[256];
	 sprintf(str,"%s/img%.4d.rgb",
			app_state->anim_state.movie_dir,
			app_state->anim_state.current_frame);

     iflFileConfig file_config(&size);
     ilFileImg file_img(str, NULL, &file_config);

     // Restore previous error handler.
     ilSetErrorHandler(handler, arg);

     // Return if there was an error.
     if (file_img.getStatus() != ilOKAY) return FALSE;

     // Copy memory image to file image.
     ilStatus status = file_img.copy(&mem_img);
     file_img.flush();
     //return (status == ilOKAY);

#endif
					sprintf(str,"gzip -9 %s/img%.4d.rgb",
						app_state->anim_state.movie_dir,
						app_state->anim_state.current_frame);
					system(str);
			
					} // if record
				} // if animate this window
			} // for

		FILE *fp = app_state->stats.fp;

		if( app_state->anim_state.current_frame ==
			app_state->anim_state.starting_frame )
			{
			fprintf( fp, "%d\n\n", app_state->stats.num_frames );
			fprintf( fp, "%d\n",   app_state->stats.read_time );
			fprintf( fp, "%d\n",   app_state->stats.neighbor_time );
			fprintf( fp, "%d\n",   app_state->stats.size );
			fprintf( fp, "%d\n",   app_state->stats.size_extra );
			fflush ( fp );
			}

		fprintf( fp,"\n");
		print_FrameStat( fp, app_state->stats.frame[0] );
		fprintf( fp,"\n");
		fflush ( fp );

		app_state->anim_state.current_frame += app_state->anim_state.frame_step;

		return False;
		}
	else
		{
		if( app_state->stats.fp != NULL )
		fclose( app_state->stats.fp );
		app_state->stats.fp = NULL;
		app_state->anim_state.speed = app_state->gr_state.zoomSpeed;

		fprintf(stderr,"DONE DONE DONE DONE\n");
		fprintf(stderr,"--------------------------------------------------\n");

		if( app_state->anim_state.terminate_on_finish )
			exit(0);

		return True;
		}
	}
