//============================================================================
//
//		callbacks.c
// 
//
//		Misc Callbacks
//
//			Exit
//			Reset
//
//			Choose Superman
//			Choose Trackball
//			Toggle Bounding Boxes
//
//			Update Render
//			Grab   Image
//			Paste  Image
//
//
//
//			Jonathan Shade
//
//============================================================================

#include <stdlib.h>
#include <stdio.h>

#include <Xm/Xm.h>
#include <X11/keysym.h>

#include <GL/glx.h>
#ifdef __sgi
#include <GL/GLwMDrawA.h>
#else
#include <X11/GLw/GLwDrawA.h>
#endif


#include "ui.h"
#include "state.h"
#include "input.h"
#include "scene.h"
#include "callbacks.h"




//----------------------------------------------------------------------------
//
//		exit_cb
//
//----------------------------------------------------------------------------
void
exit_cb( Widget, XtPointer, XtPointer )
	{
	exit(0);
	}

//----------------------------------------------------------------------------
//
//		choose_trackball
//
//----------------------------------------------------------------------------
void
choose_trackball( Widget, XtPointer client_data, XtPointer )
	{
	((AppState *)
	((uiRenderWindow *) client_data )->uiData)->fly_mode = INPUT_FLYMODE_TRACKBALL;
	}
 
//----------------------------------------------------------------------------
//
//		choose_superman
//
//----------------------------------------------------------------------------
void
choose_superman( Widget, XtPointer client_data, XtPointer )
	{
	((AppState *)
	((uiRenderWindow *) client_data )->uiData)->fly_mode = INPUT_FLYMODE_SUPERMAN;
	}

//----------------------------------------------------------------------------
//
//		choose_drive
//
//----------------------------------------------------------------------------
void
choose_drive( Widget, XtPointer client_data, XtPointer )
	{
	((AppState *)
	((uiRenderWindow *) client_data )->uiData)->fly_mode = INPUT_FLYMODE_DRIVE;
	}
 
//----------------------------------------------------------------------------
//
//		reset_cb
//
//----------------------------------------------------------------------------
void
reset_cb( Widget, XtPointer client_data, XtPointer )
	{
	int i;

	uiRenderWindow	*ren_win	= (uiRenderWindow *) client_data;
	AppState		*app_state	= (AppState *) 		ren_win->uiData; 

	scene_Init_Camera( app_state );

	for( i=0 ; i<STATE_NUM_WINDOWS ; i++ )
		if( app_state->activeWindowList[i] )
			uiPostRender( app_state->uiWindowList[i] );
	}
//----------------------------------------------------------------------------
//
//		toggle_fog
//
//----------------------------------------------------------------------------
void
toggle_fog( Widget, XtPointer client_data, XtPointer )
	{
	AppState *app_state = (AppState *) ((uiRenderWindow *) client_data )->uiData;

	if( app_state->gr_state.drawFog == 1 )
		app_state->gr_state.drawFog = 0;
	else
		app_state->gr_state.drawFog = 1;
	}

//----------------------------------------------------------------------------
//
//		toggle_timings
//
//----------------------------------------------------------------------------
void
toggle_timings( Widget, XtPointer client_data, XtPointer )
	{
	AppState *app_state = (AppState *) ((uiRenderWindow *) client_data )->uiData;

	if( app_state->gr_state.printTimings == 1 )
		app_state->gr_state.printTimings = 0;
	else
		app_state->gr_state.printTimings = 1;
	}

//----------------------------------------------------------------------------
//
//		update_render
//
//----------------------------------------------------------------------------
void
update_render( Widget, XtPointer client_data, XtPointer )
	{
	uiPostRender( (uiRenderWindow *) client_data );
	}

//----------------------------------------------------------------------------
//
//		one_step
//
//----------------------------------------------------------------------------
void
one_step( Widget, XtPointer client_data, XtPointer )
	{
	uiRenderWindow 	*ren_win  = (uiRenderWindow *) client_data;
	AppState*		app_state = (AppState *) ren_win->uiData;

	app_state->gr_state.camera->Dolly( 1.0 );
	app_state->gr_state.camera->Update();

	uiPostRender( (uiRenderWindow *) client_data );

	//
	//	Check Constraint List for other Windows to Render
	//
	for( int i=0 ; i<ren_win->num_constraints ; i++ )
		{
		uiRenderWindow *cstr = &(ren_win->constraint_list[i]);

		if( app_state->activeWindowList[cstr->window_id] ) 
		uiPostRender( cstr );
		}

	}


//----------------------------------------------------------------------------
//
//		Uses the global array:	WindowList
//
//			to manage/unmanage X windows.
//
//
//----------------------------------------------------------------------------
void
update_window_config( Widget, XtPointer client_data, XtPointer )
	{
	int	window_id	= ((uiRenderWindow *) client_data)->window_id;
	AppState *app_state = (AppState *) ((uiRenderWindow *) client_data)->uiData;

	if ( app_state->activeWindowList[window_id] )
		{
		XtPopdown( ((uiRenderWindow *) client_data)->shell);
		app_state->activeWindowList[window_id] = 0;
		}
	else
		{
		XtPopup( ((uiRenderWindow *) client_data)->shell,
				 XtGrabNone );
		app_state->activeWindowList[window_id] = 1;
		}
	}

//----------------------------------------------------------------------------
//
//		toggle_bb
//
//		In the Geometry window button bar.
//
//----------------------------------------------------------------------------
void
toggle_bb( Widget, XtPointer client_data, XtPointer )
	{
	AppState *app_state = (AppState *) ((uiRenderWindow *) client_data)->uiData;

	if( app_state->gr_state.drawBoundingBoxes == 1 )
		app_state->gr_state.drawBoundingBoxes = 0;
	else
		app_state->gr_state.drawBoundingBoxes = 1;

	}

//----------------------------------------------------------------------------
//
//		toggle_frames
//
//
//		In the Image window button Bar.
//----------------------------------------------------------------------------
void
toggle_frames( Widget, XtPointer client_data, XtPointer )
	{
	AppState *app_state = (AppState *) ((uiRenderWindow *) client_data)->uiData;

	if( app_state->gr_state.drawFrames == 1 )
		app_state->gr_state.drawFrames = 0;
	else
		app_state->gr_state.drawFrames = 1;
	}

//----------------------------------------------------------------------------
//
//		toggle_leaf_textures
//
//
//		In the Image window button Bar.
//----------------------------------------------------------------------------
void
toggle_leaf_textures( Widget, XtPointer client_data, XtPointer )
	{
	AppState *app_state = (AppState *) ((uiRenderWindow *) client_data)->uiData;

	if( app_state->gr_state.drawLeafTextures == 1 )
		app_state->gr_state.drawLeafTextures = 0;
	else
		app_state->gr_state.drawLeafTextures = 1;
	}

//----------------------------------------------------------------------------
//
//		toggle_image_geom
//
//
//		In the Image window button Bar.
//----------------------------------------------------------------------------
void
toggle_image_geom( Widget, XtPointer client_data, XtPointer )
	{
	AppState *app_state = (AppState *) ((uiRenderWindow *) client_data)->uiData;

	if( app_state->gr_state.drawGeom == 1 )
		app_state->gr_state.drawGeom = 0;
	else
		app_state->gr_state.drawGeom = 1;
	}

//----------------------------------------------------------------------------
//
//		expose_callback:
//
//			This function is called when the X window is exposed.
//
//----------------------------------------------------------------------------
void
expose_callback( Widget, XtPointer client_data, XtPointer )
	{
	uiPostRender( (uiRenderWindow *) client_data );
	}



//----------------------------------------------------------------------------
//
//		resize_callback:
//
//			This function is called when the X window is resized.
//
//----------------------------------------------------------------------------
void
resize_callback( Widget widget, XtPointer client_data, XtPointer )
    {
	Dimension width;
	Dimension height;

	XtVaGetValues(  widget,
					XmNwidth,  &width,
					XmNheight, &height,
					NULL );

	((uiRenderWindow *) client_data)->dirtyViewport		= 1;
	((uiRenderWindow *) client_data)->glxarea_width		= width;
	((uiRenderWindow *) client_data)->glxarea_height	= height;
    }


