//============================================================================
//
//		main.c
//
//
//			Jonathan Shade
//
//============================================================================

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeBG.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <X11/keysym.h>

#include <GL/glx.h>
#ifdef __sgi
#include <GL/GLwMDrawA.h>
#else
#include <X11/GLw/GLwDrawA.h>
#endif

#include "ui.h"
#include "glx_resources.h"
#include "scene.h"
#include "Camera.h"
#include "callbacks.h"
#include "input.h" 
#include "menu.h"
#include "state.h"
#include "animate.h"

#define APP_CLASS	"WalkthruViewer"
#define APP_NAME	"WalkthruViewer"

#define	WIN_HEIGHT	640
#define WIN_WIDTH	640

/*
 *	Motif options
 */
static String   fallbackResources[] =
{
	"*sgiMode:	  True",
	"*useSchemes: All",
	NULL
};



//----------------------------------------------------------------------------
//
//		List of "Visuals" to try when creating
//		the windows of this application
//
//
//----------------------------------------------------------------------------
uiVisualList	visual_list_plain[] =
								{
								VR0,VR0_desc,
								VR1,VR1_desc,
								VR2,VR2_desc,
								VR3,VR3_desc,
								VR4,VR4_desc,
								VR5,VR5_desc,
								NULL,
								};

uiVisualList	visual_list_render[10];

//----------------------------------------------------------------------------
//
//	main_complex:
//
//		Sets up the X windows stuff, and launches into 
//		the main loop of the program.
//
//----------------------------------------------------------------------------
void
main_interactive(int argc, char *argv[], AppState &app_state )
    {
    Arg             args[20];

	Widget  	    toplevel;

	Display			*dpy;

    XtAppContext    app;

	XtWorkProcId	renderWorkId;

	uiRenderWindow	geom_win;
	uiRenderWindow	image_win;
	uiRenderWindow	preview_win;
	uiRenderWindow	skycam_win;
	uiRenderWindow	sidecam_win;

	Dimension		width,height;

	Camera			camera;

	int				vis_index;

	//
	//	Build up the Visual Request List for the rendering window
	//
	if( app_state.gr_state.grhardware == GH_REALITY_ENGINE )
		{
		if( app_state.gr_state.aa )
			{
			visual_list_render[0].reqstr  = VR_RE0;
			visual_list_render[0].descstr = VR_RE0_desc;
			}
		else
			{
			visual_list_render[0].reqstr  = VR_RE1;
			visual_list_render[0].descstr = VR_RE1_desc;
			}

		visual_list_render[1].reqstr  = VR_RE2;
		visual_list_render[1].descstr = VR_RE2_desc;

		visual_list_render[2].reqstr  = NULL;
		visual_list_render[2].descstr = NULL;
		}
	else
		{
		if( app_state.gr_state.aa )
			{
			visual_list_render[0].reqstr  = VR_IM0;
			visual_list_render[0].descstr = VR_IM0_desc;
			visual_list_render[1].reqstr  = VR_IM1;
			visual_list_render[1].descstr = VR_IM1_desc;
			}

		else
			{
			visual_list_render[0].reqstr  = VR_IM2;
			visual_list_render[0].descstr = VR_IM2_desc;

			if( app_state.gr_state.db )
				{
				visual_list_render[1].reqstr  = VR2;
				visual_list_render[1].descstr = VR2_desc;
				}
			else
				{
				visual_list_render[1].reqstr  = VR_IM3;
				visual_list_render[1].descstr = VR_IM3_desc;
				}
			}

		visual_list_render[2].reqstr  = VR4;
		visual_list_render[2].descstr = VR4_desc;
		visual_list_render[3].reqstr  = NULL;
		visual_list_render[3].descstr = NULL;
		}

	app_state.fly_mode = INPUT_FLYMODE_DRIVE;

	app_state.activeWindowList[0] = 1;
	app_state.activeWindowList[1] = 0;
	app_state.activeWindowList[2] = 0;
	app_state.activeWindowList[3] = 0;
	app_state.activeWindowList[4] = 0;

	app_state.gr_state.numSamples		 = 8;  // Should be command line param
	app_state.gr_state.drawFog			 = 0;
	app_state.gr_state.drawLeafTextures	 = 0;
	app_state.gr_state.drawGeom			 = 0;
	app_state.gr_state.drawBoundingBoxes = 0;
	app_state.gr_state.printTimings		 = 0;
	app_state.gr_state.camera			 = &camera;

	app_state.anim_state.terminate_on_finish = 0;

	//
	//	Create the Application
	//
	//
    toplevel	= XtAppInitialize(&app, "WalkthruViewer",
					NULL, 0, &argc, argv, fallbackResources, NULL, 0);
    dpy			= XtDisplay(toplevel);



	//------------------------------------------------------------
	//
	//	Create the window for geometry rendering
	//
	//		This window is the "main" app window.
	//
	//------------------------------------------------------------
	XtSetArg(args[0],XmNx, 0);
	XtSetArg(args[1],XmNy, 0);
	XtSetArg(args[2],XmNtitle, "Preview" );
	XtSetArg(args[3],XmNiconName, "Preview" );

	preview_win.parent	= toplevel;

//	preview_win.shell	= XtAppCreateShell(APP_NAME,APP_CLASS,
//						  topLevelShellWidgetClass, dpy, args, 4 );	

	preview_win.shell = toplevel;
	XtSetValues( preview_win.shell,args,4);

	preview_win.app					= app; 
	preview_win.dpy					= dpy; 
	preview_win.draw				= scene_Draw_Preview;
	preview_win.init				= scene_Init_GL;
	preview_win.needsInit			= 1;
	preview_win.dirtyViewport		= 1;
	preview_win.onRenderList		= 0;
	preview_win.nextRenderWindow	= NULL;

	preview_win.num_constraints		= 0;
	preview_win.constraint_list		= NULL;

	preview_win.uiData				= (AppState *)	&app_state;

	preview_win.glxarea_width		= 300;
	preview_win.glxarea_height		= 300;

	preview_callbacks.exposeData	= &preview_win;
	preview_callbacks.resizeData	= &preview_win;
	preview_callbacks.inputData	= &preview_win;

	//
	//	"Admin"
	//		"Exit"
	//
	preview_menu[0].menu[0].callback_data = &preview_win;

	//
	//	"Control"
	//		"Trackball"
	//		"Superman"
	//		"Drive"
	//		"Reset"
	//
	preview_menu[1].menu[0].callback_data = &preview_win;
	preview_menu[1].menu[1].callback_data = &preview_win;
	preview_menu[1].menu[2].callback_data = &preview_win;
	preview_menu[1].menu[3].callback_data = &preview_win;

	//
	//	"Animation"
	//		"Preview"
	//		"Geometry"
	//		"Image"
	//		"Record"
	//		"Print Camera"
	//		"Reset Camera"
	//		"Start Animation"
	//		"Stop Animation"
	//		"Read Curve File"
	//
	preview_menu[2].menu[0].callback_data = &preview_win;
	preview_menu[2].menu[1].callback_data = &preview_win;
	preview_menu[2].menu[2].callback_data = &preview_win;
	preview_menu[2].menu[3].callback_data = &preview_win;
	preview_menu[2].menu[4].callback_data = &preview_win;
	preview_menu[2].menu[5].callback_data = &preview_win;
	preview_menu[2].menu[6].callback_data = &preview_win;
	preview_menu[2].menu[7].callback_data = &preview_win;
	preview_menu[2].menu[8].callback_data = &preview_win;

	//
	//	"Other"
	//		"Fog"
	//		"Print Timings"
	//
	preview_menu[3].menu[0].callback_data = &preview_win;
	preview_menu[3].menu[1].callback_data = &preview_win;


	//	
	//		Button Bar
	//
	//	"Geometry	Window"
	//	"Image		Window"
	//	"Skycam		Window"
	//	"SideCam	Window"
	//
	preview_ht_bar[0].callback_data	= &geom_win;
	preview_ht_bar[1].callback_data	= &image_win;
	preview_ht_bar[2].callback_data	= &skycam_win;
	preview_ht_bar[3].callback_data	= &sidecam_win;

	vis_index = ui_Create_GL_Rendering_Window( &preview_win, preview_callbacks,
												preview_menu, preview_ht_bar,
												visual_list_plain,
												None );

fprintf(stderr,"Preview Win:\t%s\n", visual_list_plain[vis_index].descstr );

	if ( app_state.activeWindowList[preview_win.window_id] )
		{
		XtPopup( preview_win.shell, XtGrabNone );

		XtVaGetValues(preview_win.glxarea,
						XmNwidth,  &width,
					 	XmNheight, &height,
					 	NULL );

		preview_win.glxarea_height = height;
		preview_win.glxarea_width  = width;
		}



	//------------------------------------------------------------
	//
	//	Create the window for geometry rendering
	//
	//
	//------------------------------------------------------------
	XtSetArg(args[0],XmNx, 0);
	XtSetArg(args[1],XmNy, 0);
	XtSetArg(args[2],XmNtitle, "Geometry Render" );
	XtSetArg(args[3],XmNiconName, "Geometry" );

	geom_win.parent	= toplevel;

	geom_win.shell	= XtAppCreateShell(APP_NAME,APP_CLASS,
						  topLevelShellWidgetClass, dpy, args, 4 );	

	geom_win.app				= app; 
	geom_win.dpy				= dpy; 
	geom_win.draw				= scene_Draw_Geom;
	geom_win.init				= scene_Init_GL;
	geom_win.needsInit			= 1;
	geom_win.dirtyViewport		= 1;
	geom_win.onRenderList		= 0;
	geom_win.nextRenderWindow	= NULL;

	geom_win.num_constraints	= 0;
	geom_win.constraint_list	= NULL;

	geom_win.uiData				= (AppState *) 		&app_state;

	geom_win.glxarea_width		= WIN_WIDTH;
	geom_win.glxarea_height		= WIN_HEIGHT;

	geom_callbacks.exposeData	= &geom_win;
	geom_callbacks.resizeData	= &geom_win;
	geom_callbacks.inputData	= &geom_win;

	//
	//	"Update Render"
	//	"One Step"
	//	"Bounding Boxes"
	//
	geom_ht_bar[0].callback_data		= &geom_win;
	geom_ht_bar[1].callback_data		= &geom_win;
	geom_ht_bar[2].callback_data		= &geom_win;

	vis_index = ui_Create_GL_Rendering_Window( &geom_win, geom_callbacks,
												geom_menu, geom_ht_bar,
												visual_list_render,
												None );

fprintf(stderr,"Geom win:\t%s\n", visual_list_render[vis_index].descstr );

	if ( app_state.activeWindowList[geom_win.window_id] )
		XtPopup( geom_win.shell, XtGrabNone );


	//------------------------------------------------------------
	//
	//	Create the window for image composition rendering
	//
	//
	//------------------------------------------------------------
	XtSetArg(args[0],XmNx, 0);
	XtSetArg(args[1],XmNy, 0);
	XtSetArg(args[2],XmNtitle, "Hierarchical Image Caching" );
	XtSetArg(args[3],XmNiconName, "Image" );

	image_win.parent	= toplevel;

	image_win.shell		= XtAppCreateShell(APP_NAME,APP_CLASS,
						  topLevelShellWidgetClass, dpy, args, 4 );	

	image_win.app				= app; 
	image_win.dpy				= dpy; 
	image_win.draw				= scene_Draw_Image;
	image_win.init				= scene_Init_GL;
	image_win.needsInit			= 1;
	image_win.dirtyViewport		= 1;
	image_win.onRenderList		= 0;
	image_win.nextRenderWindow	= NULL;

	image_win.num_constraints	= 0;
	image_win.constraint_list	= NULL;

	image_win.uiData			= (AppState *) 		&app_state;

	image_win.glxarea_width		= WIN_WIDTH;
	image_win.glxarea_height	= WIN_HEIGHT;

	image_callbacks.exposeData	= &image_win;
	image_callbacks.resizeData	= &image_win;
	image_callbacks.inputData	= &image_win;

	//
	//	"Update Render"
	//	"One Step"
	//	"Show Frames"			toggle button
	//	"Draw Leaf Textures"	toggle button
	//	"Draw Geometry"			toggle button
	//
	image_ht_bar[0].callback_data		= &image_win;
	image_ht_bar[1].callback_data		= &image_win;
	image_ht_bar[2].callback_data		= &image_win;
	image_ht_bar[3].callback_data		= &image_win;
	image_ht_bar[4].callback_data		= &image_win;

	vis_index = ui_Create_GL_Rendering_Window( &image_win, image_callbacks,
												image_menu, image_ht_bar,
												visual_list_render,
												geom_win.cx );

fprintf(stderr,"Image Win:\t%s\n", visual_list_render[vis_index].descstr );

	if ( app_state.activeWindowList[image_win.window_id] )
		XtPopup( image_win.shell, XtGrabNone );

	//------------------------------------------------------------
	//
	//	Create the window for skycam rendering
	//
	//		This window is the "skycam" window.
	//
	//------------------------------------------------------------
	XtSetArg(args[0],XmNx, 0);
	XtSetArg(args[1],XmNy, 0);
	XtSetArg(args[2],XmNtitle, "Sky Cam" );
	XtSetArg(args[3],XmNiconName, "Sky Cam" );

	skycam_win.parent	= toplevel;

	skycam_win.shell	= XtAppCreateShell(NULL,"Class",
						  topLevelShellWidgetClass, dpy, args, 4 );	

	skycam_win.app				= app; 
	skycam_win.dpy				= dpy; 
	skycam_win.draw				= scene_Draw_SkyCam;
	skycam_win.init				= scene_Init_GL;
	skycam_win.needsInit		= 1;
	skycam_win.dirtyViewport	= 1;
	skycam_win.onRenderList		= 0;
	skycam_win.nextRenderWindow	= NULL;

	skycam_win.num_constraints	= 0;
	skycam_win.constraint_list	= NULL;
	
	skycam_win.uiData			= (AppState *)		&app_state;

	skycam_win.glxarea_width	= 600;
	skycam_win.glxarea_height	= 600;

	skycam_callbacks.exposeData	= &skycam_win;
	skycam_callbacks.resizeData	= &skycam_win;
	skycam_callbacks.inputData	= &skycam_win;

	vis_index = ui_Create_GL_Rendering_Window( &skycam_win, skycam_callbacks,
												skycam_menu, skycam_ht_bar,
												visual_list_plain,
												None );

fprintf(stderr,"Sky Cam Win:\t%s\n\n", visual_list_plain[vis_index].descstr );


	if ( app_state.activeWindowList[skycam_win.window_id] )
		XtPopup( skycam_win.shell, XtGrabNone );

	//------------------------------------------------------------
	//
	//	Create the window for SideCam rendering
	//
	//		This window is the "SideCam" window.
	//
	//------------------------------------------------------------
	XtSetArg(args[0],XmNx, 0);
	XtSetArg(args[1],XmNy, 0);
	XtSetArg(args[2],XmNtitle, "SideCam" );
	XtSetArg(args[3],XmNiconName, "SideCam" );

	sidecam_win.parent	= toplevel;

	sidecam_win.shell	= XtAppCreateShell(NULL,"Class",
						  topLevelShellWidgetClass, dpy, args, 4 );	

	sidecam_win.app					= app; 
	sidecam_win.dpy					= dpy; 
	sidecam_win.draw				= scene_Draw_SideCam;
	sidecam_win.init				= scene_Init_GL;
	sidecam_win.needsInit			= 1;
	sidecam_win.dirtyViewport		= 1;
	sidecam_win.onRenderList		= 0;
	sidecam_win.nextRenderWindow	= NULL;

	sidecam_win.num_constraints	= 0;
	sidecam_win.constraint_list	= NULL;
	
	sidecam_win.uiData			= (AppState *)		&app_state;

	sidecam_win.glxarea_width	= 640;
	sidecam_win.glxarea_height	= 640;

	sidecam_callbacks.exposeData	= &sidecam_win;
	sidecam_callbacks.resizeData	= &sidecam_win;
	sidecam_callbacks.inputData		= &sidecam_win;


	vis_index = ui_Create_GL_Rendering_Window( &sidecam_win, sidecam_callbacks,
												sidecam_menu, sidecam_ht_bar,
												visual_list_plain,
												None );

fprintf(stderr,"SideCam Win:\t%s\n\n", visual_list_plain[vis_index].descstr );


	if ( app_state.activeWindowList[sidecam_win.window_id] )
		XtPopup( sidecam_win.shell, XtGrabNone );

	//======================================================
	//======================================================

	//
	//  Realize the toplevel application widget
	//
	//XtRealizeWidget(toplevel);

	//
	//	Setup Constraints between windows
	//
	preview_win.num_constraints = 2;
	preview_win.constraint_list = new uiRenderWindow[2];
	preview_win.constraint_list[0] = skycam_win;
	preview_win.constraint_list[1] = sidecam_win;

	image_win.num_constraints = 1;
	image_win.constraint_list = new uiRenderWindow[1];
	image_win.constraint_list[0] = skycam_win;
//	image_win.constraint_list[0] = preview_win;
//	image_win.constraint_list[2] = sidecam_win;

	geom_win.num_constraints = 1;
	geom_win.constraint_list = new uiRenderWindow[1];
	geom_win.constraint_list[0] = skycam_win;
//	geom_win.constraint_list[0] = preview_win;
//	geom_win.constraint_list[2] = sidecam_win;

	//
	//	Init scene database
	//
	//	Make sure it is the Image window that actually
	//	inits the db.
	//
	animate_init( app_state );

	scene_Init_DB( app_state.db_filename, image_win.window_id  );

	//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	//	Hack for Animation
	//
	app_state.uiWindowList[0] = &preview_win;
	app_state.uiWindowList[1] = &geom_win;
	app_state.uiWindowList[2] = &image_win;
	app_state.uiWindowList[3] = &skycam_win;
	app_state.uiWindowList[4] = &sidecam_win;

	app_state.anim_state.win_list[0] = 0;
	app_state.anim_state.win_list[1] = 0;
	app_state.anim_state.win_list[2] = 0;
	app_state.anim_state.win_list[3] = 0;
	app_state.anim_state.win_list[4] = 0;
	//
	//
	//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

	renderWorkId = XtAppAddWorkProc( app, uiRenderIdleProc, NULL );

	XtAppMainLoop(app);
    }


//----------------------------------------------------------------------------
//
//	main_batch:
//
//		Sets up the X windows stuff, and launches into 
//		the main loop of the program.
//
//----------------------------------------------------------------------------
void
main_batch(int argc, char *argv[], AppState & app_state )
    {
    Arg             args[20];
	Widget  	    toplevel;
	Display			*dpy;
    XtAppContext    app;
	XtWorkProcId	renderWorkId;

	uiRenderWindow	batch_win;
	Camera			camera;

	int				vis_index;

	//
	//	Build up the Visual Request List for the rendering window
	//
	if( app_state.gr_state.grhardware == GH_REALITY_ENGINE )
		{
		if( app_state.gr_state.aa )
			{
			visual_list_render[0].reqstr  = VR_RE0;
			visual_list_render[0].descstr = VR_RE0_desc;
			}
		else
			{
			visual_list_render[0].reqstr  = VR_RE1;
			visual_list_render[0].descstr = VR_RE1_desc;
			}

		visual_list_render[1].reqstr  = VR_RE2;
		visual_list_render[1].descstr = VR_RE2_desc;

		visual_list_render[2].reqstr  = NULL;
		visual_list_render[2].descstr = NULL;
		}
	else
		{
		if( app_state.gr_state.aa )
			{
			visual_list_render[0].reqstr  = VR_IM0;
			visual_list_render[0].descstr = VR_IM0_desc;
			visual_list_render[1].reqstr  = VR_IM1;
			visual_list_render[1].descstr = VR_IM1_desc;
			}

		else
			{
			visual_list_render[0].reqstr  = VR_IM2;
			visual_list_render[0].descstr = VR_IM2_desc;

			if( app_state.gr_state.db )
				{
				visual_list_render[1].reqstr  = VR2;
				visual_list_render[1].descstr = VR2_desc;
				}
			else
				{
				visual_list_render[1].reqstr  = VR_IM3;
				visual_list_render[1].descstr = VR_IM3_desc;
				}
			}

		visual_list_render[2].reqstr  = VR4;
		visual_list_render[2].descstr = VR4_desc;
		visual_list_render[3].reqstr  = NULL;
		visual_list_render[3].descstr = NULL;
		}

	app_state.fly_mode = INPUT_FLYMODE_DRIVE;

	app_state.activeWindowList[0] = 0;
	app_state.activeWindowList[1] = 0;
	app_state.activeWindowList[2] = 0;
	app_state.activeWindowList[3] = 0;
	app_state.activeWindowList[4] = 0;

	app_state.activeWindowList[0] = 1;
	app_state.anim_state.win_list[0] = 1;

	app_state.gr_state.numSamples		 = 8;  // Should be command line param
	app_state.gr_state.drawFog			 = 0;
	app_state.gr_state.drawLeafTextures	 = 0;
	app_state.gr_state.drawGeom			 = 0;
	app_state.gr_state.drawBoundingBoxes = 0;
	app_state.gr_state.printTimings		 = 0;
	app_state.gr_state.camera			 = &camera;

	app_state.anim_state.terminate_on_finish = 1;

	//
	//	Create the Application
	//
	//
    toplevel	= XtAppInitialize(&app, "WalkthruViewer",
					NULL, 0, &argc, argv, fallbackResources, NULL, 0);
    dpy			= XtDisplay(toplevel);


	//------------------------------------------------------------
	//
	//	Create the window for image composition rendering
	//
	//
	//------------------------------------------------------------
	XtSetArg(args[0],XmNx, 0);
	XtSetArg(args[1],XmNy, 0);
	XtSetArg(args[2],XmNtitle, "Hierarchical Image Caching" );
	XtSetArg(args[3],XmNiconName, "Batch Mode Renderer" );

	batch_win.parent	= toplevel;

	batch_win.shell		= XtAppCreateShell(APP_NAME,APP_CLASS,
						  topLevelShellWidgetClass, dpy, args, 4 );	

	batch_win.app				= app; 
	batch_win.dpy				= dpy; 

	switch( app_state.anim_state.batch_renderer )
		{
		case BR_IMAGE:
		batch_win.draw				= scene_Draw_Image;
		break;

		case BR_GEOM:
		batch_win.draw				= scene_Draw_Geom;
		break;

		case BR_SIDECAM:
		batch_win.draw				= scene_Draw_SideCam;
		break;

		default:
		batch_win.draw				= scene_Draw_Image;
		break;
		}

	batch_win.init				= scene_Init_GL;
	batch_win.needsInit			= 1;
	batch_win.dirtyViewport		= 1;
	batch_win.onRenderList		= 0;
	batch_win.nextRenderWindow	= NULL;

	batch_win.uiData			= (AppState *) 		&app_state;

	batch_win.glxarea_width		= WIN_WIDTH;
	batch_win.glxarea_height	= WIN_HEIGHT;

	image_callbacks.exposeData	= &batch_win;
	image_callbacks.resizeData	= &batch_win;
	image_callbacks.inputData	= &batch_win;

	//
	//	"Update Render"
	//	"One Step"
	//	"Show Frames"		 toggle button
	//	"Draw Leaf Textures" toggle button
	//	"Draw Geometry"      toggle button
	//
	image_ht_bar[0].callback_data		= &batch_win;
	image_ht_bar[1].callback_data		= &batch_win;
	image_ht_bar[2].callback_data		= &batch_win;
	image_ht_bar[3].callback_data		= &batch_win;
	image_ht_bar[4].callback_data		= &batch_win;

	vis_index = ui_Create_GL_Rendering_Window( &batch_win, image_callbacks,
												image_menu, image_ht_bar,
												visual_list_render,
												None );

fprintf(stderr,"Batch Mode Win:\t%s\n",visual_list_render[vis_index].descstr);

	if ( app_state.activeWindowList[batch_win.window_id] )
		XtPopup( batch_win.shell, XtGrabNone );


	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~


	app_state.uiWindowList[0] = &batch_win;
	app_state.uiWindowList[1] = NULL;
	app_state.uiWindowList[2] = NULL;
	app_state.uiWindowList[3] = NULL;
	app_state.uiWindowList[4] = NULL;

	batch_win.num_constraints = 0;
	batch_win.constraint_list = NULL;

	animate_init( app_state );

	scene_Init_DB( app_state.db_filename, batch_win.window_id  );

	renderWorkId = XtAppAddWorkProc( app, uiRenderIdleProc, NULL );

	animate_start_batch( &batch_win );

	XtAppMainLoop(app);
	}

//----------------------------------------------------------------------------
//
//	main:
//
//		Process the command line arguments
//
//		-metric1	Use Angle/Distance Metric
//		-metric2	Use Corners Metric
//
//		-h			List options
//
//----------------------------------------------------------------------------
void
print_options()
	{
	fprintf(stderr,"\nOptions:\n========\n");

	fprintf(stderr,"-h\tThis Menu\n");
	fprintf(stderr,"-m1\tAngle/Distance\n");
	fprintf(stderr,"-m2\tCorners\n");
	fprintf(stderr,"-m3\tBrick\n");

	fprintf(stderr,"\n");
	fprintf(stderr,"-aa\tAnti-Aliasing\n");
	fprintf(stderr,"-db\tDouble Buffering\n");
	fprintf(stderr,"-re\tRealityEngine Graphics\n");
	fprintf(stderr,"-max\tMaximum IMPACT Graphics\n");
	fprintf(stderr,"-high\tHigh IMPACT Graphics\n");
	fprintf(stderr,"-noenv\tDon't draw the Environment(sky,etc)\n");

	fprintf(stderr,"\n");
	fprintf(stderr,"-batch\tBatch Mode Animation\n");
	fprintf(stderr,"-image\tAnimate Image Caching\n");
	fprintf(stderr,"-geom\tAnimate Geometry\n");
	fprintf(stderr,"-sidecam Animate SideCam\n");
	fprintf(stderr,"-record\tStore Frames to Disk\n");
	fprintf(stderr,"-frames\tDraw a frame around the caches\n");
	fprintf(stderr,"-psuedo\tHighlight the changing caches\n");

	fprintf(stderr,"\n");
	fprintf(stderr,"-eps #\t\t e.g. 2.0\n");
	fprintf(stderr,"-inflation #\t e.g. 1.03\n");
	fprintf(stderr,"-noinf\tDon't inflate the Nodes\n");

	fprintf(stderr,"\n");
	fprintf(stderr,"-f  database filename\tThis option is required\n");
	fprintf(stderr,"-cf config   filename\n");
	fprintf(stderr,"\n");
	}

void
main(int argc, char *argv[] )
	{
	char		config_filename[256];
	int			batch = 0;
	AppState	app_state;

	app_state.gr_state.aa = 0;
	app_state.gr_state.db = 0;
	app_state.gr_state.grhardware		= GH_HIGH_IMPACT;
	app_state.gr_state.metric_type		= Brick;
	app_state.gr_state.pixel_eps		= 2.0;
	app_state.gr_state.inflation		= 1.0;
	app_state.gr_state.inflate			= 1;
	app_state.gr_state.drawHz			= 0;
	app_state.gr_state.drawEnvironment	= 1;
	app_state.gr_state.drawFrames		= 0;
	app_state.gr_state.drawPsuedoColor	= 0;

	app_state.anim_state.batch_renderer	= BR_IMAGE;
	app_state.anim_state.record			= 0;

	app_state.stats.num_frames = 0;
	app_state.stats.read_time = 0;
	app_state.stats.neighbor_time = 0;
	app_state.stats.size = 0;
	app_state.stats.size_extra = 0;
	app_state.stats.fp    = NULL;
	app_state.stats.frame = NULL;

	strcpy( config_filename, "walkthru.conf" );

	if( argc == 1 )
		{
		print_options();
		exit(0);
		}

	for (int i = 1; i < argc; i++)
		{
		if      (strcmp(argv[i], "-m1") == 0)
			app_state.gr_state.metric_type = AngleDistance;
		else if (strcmp(argv[i], "-m2") == 0)
			app_state.gr_state.metric_type = Corners;
		else if (strcmp(argv[i], "-m3") == 0)
			app_state.gr_state.metric_type = Brick;

		else if (strcmp(argv[i], "-aa") == 0)
			{
			app_state.gr_state.aa = 1;
			}	

		else if (strcmp(argv[i], "-db") == 0)
			{
			app_state.gr_state.db = 1;
			}	

		else if (strcmp(argv[i], "-re") == 0)
			{
			app_state.gr_state.grhardware = GH_REALITY_ENGINE;
			}	

		else if (strcmp(argv[i], "-max") == 0)
			{
			app_state.gr_state.grhardware = GH_MAXIMUM_IMPACT;
			}	

		else if (strcmp(argv[i], "-high") == 0)
			{
			app_state.gr_state.grhardware = GH_HIGH_IMPACT;
			}	

		else if (strcmp(argv[i], "-noenv") == 0)
			{
			app_state.gr_state.drawEnvironment = 0;
			}	

		else if (strcmp(argv[i], "-batch") == 0)
			{
			batch = 1;
			}	

		else if (strcmp(argv[i], "-geom") == 0)
			{
			app_state.anim_state.batch_renderer = BR_GEOM;
			}	

		else if (strcmp(argv[i], "-image") == 0)
			{
			app_state.anim_state.batch_renderer = BR_IMAGE;
			}	

		else if (strcmp(argv[i], "-sidecam") == 0)
			{
			app_state.anim_state.batch_renderer = BR_SIDECAM;
			}	

		else if (strcmp(argv[i], "-record") == 0)
			{
			app_state.anim_state.record = 1;
			}	

		else if (strcmp(argv[i], "-frames") == 0)
			{
			app_state.gr_state.drawFrames = 1;
			}	

		else if (strcmp(argv[i], "-psuedo") == 0)
			{
			app_state.gr_state.drawPsuedoColor = 1;
			}	

		else if (strcmp(argv[i], "-eps") == 0)
			{
			if ( i != argc-1 )
				app_state.gr_state.pixel_eps = atof( argv[i+1] );
			else
				{
				fprintf(stderr,"\nNo Pixel Epsilon Given\n");
				exit(0);
				}
			i++;
			}

		else if (strcmp(argv[i], "-inflation") == 0)
			{
			if ( i != argc-1 )
				app_state.gr_state.inflation = atof( argv[i+1] );
			else
				{
				fprintf(stderr,"\nNo Bounding Box Inflation Given\n");
				exit(0);
				}
			i++;
			}

		else if (strcmp(argv[i], "-noinf") == 0)
			{
			app_state.gr_state.inflate = 0;
			}	


		else if (strcmp(argv[i], "-f") == 0)
			{
			if ( i != argc-1 )
				strcpy( app_state.db_filename, argv[i+1] );
			else
				{
				fprintf(stderr,"\nNo Database File Name Given\n");
				exit(0);
				}
			i++;
			}

		else if (strcmp(argv[i], "-cf") == 0)
			{
			if ( i != argc-1 )
				strcpy( config_filename, argv[i+1] );
			else
				{
				fprintf(stderr,"\nNo Config File Name Given\n");
				exit(0);
				}
			i++;
			}

		else if ( (strcmp(argv[i], "-h") == 0)    ||
				  (strcmp(argv[i], "-help") == 0) )
			{
			print_options();
			exit(0);
			}

		else
			{
			fprintf(stderr,"Dont recognize option: %s \n", argv[i] );
			}
		}

	fprintf(stderr,"Reading config file: %s\n",config_filename);  

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//	Read Configuration File
	//
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	FILE *fp = fopen( config_filename, "r" );
	if( fp == NULL )
		{
		fprintf(stderr,"Can't open config file: %s\n", config_filename );
		exit(1);
		}

	fscanf( fp, "%s", app_state.anim_state.curve_file );
	fscanf( fp, "%s", app_state.anim_state.movie_dir );
	fscanf( fp, "%d", &app_state.anim_state.starting_frame );
	fscanf( fp, "%d", &app_state.anim_state.ending_frame );
	fscanf( fp, "%d", &app_state.anim_state.frame_step );
	fscanf( fp, "%d", &app_state.anim_state.num_frames );
	fscanf( fp, "%d", &app_state.anim_state.lookat );
	fscanf( fp, "%lf", &app_state.gr_state.ic_x );
	fscanf( fp, "%lf", &app_state.gr_state.ic_y );
	fscanf( fp, "%lf", &app_state.gr_state.ic_z );

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	// Read in the "Environment" object names
	//
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	int num_env_objs;
	fscanf( fp, "%d", &num_env_objs );
	app_state.gr_state.num_env_objs = num_env_objs;

	if( num_env_objs > 0 )
		{
		app_state.gr_state.env_objs = new TriangleList[num_env_objs];
		app_state.gr_state.env_obj_names = new char* [num_env_objs];
		for( i=0 ; i<num_env_objs ; i++ )
			{
			app_state.gr_state.env_obj_names[i] = new char [256];
			fscanf( fp, "%s", app_state.gr_state.env_obj_names[i] );
			}
		}


	if( batch )
		main_batch( argc,argv,app_state );
	else
		main_interactive( argc,argv,app_state );
	}
