//============================================================================
//
//		ui.c
//
//
//			Jonathan Shade
//
//============================================================================

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/Frame.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <Xm/RowColumn.h>
#include <Xm/CascadeBG.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <X11/keysym.h>

#include <GL/glx.h>
#ifdef __sgi
#include <GL/GLwMDrawA.h>
#else
#include <X11/GLw/GLwDrawA.h>
#endif

#include "ui.h"


uiRenderWindow*	RenderList = NULL;

//----------------------------------------------------------------------------
//
//		ui_Create_GL_Rendering_Window
//
//----------------------------------------------------------------------------
int		ui_Create_GL_Rendering_Window(	uiRenderWindow	*renderWindow,
										uiCallbackList	callback_list,
										uiMenuBarItem	*menu_bar_items,
										uiHotkeyItem	*hk_items,
										uiVisualList	*v_list,
										GLXContext		share_list )
	{
	static	current_id	= 0;
	int		i,retVisFnd;

	renderWindow->window_id = current_id;
	current_id++;

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//	Get visual for GL window
	//
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	for( i=0 ; v_list[i].reqstr != NULL; i++ )
		{
		//fprintf(stderr,"Trying: %s\n", v_list[i].descstr );

		renderWindow->vi = glXChooseVisual( renderWindow->dpy,
										DefaultScreen(renderWindow->dpy),
										v_list[i].reqstr );
		if( renderWindow->vi != NULL )
			{
			retVisFnd = i;
			break;
			}
		}

	if ( renderWindow->vi == NULL )
		XtAppError(renderWindow->app, "No visual was found");
	
	
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//	Create GL context for GL window
	//
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	renderWindow->cx = glXCreateContext( renderWindow->dpy,
										 renderWindow->vi,
										 share_list,
										 GL_TRUE );

	if ( renderWindow->cx == NULL )
		XtAppError(renderWindow->app, "could not create rendering context");


	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//	Create the Main Window
	//
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	renderWindow->mainw = XtVaCreateManagedWidget( "main_window",
		xmMainWindowWidgetClass, renderWindow->shell,
		NULL);

	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//	Menu stuff
	//
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	renderWindow->menuBar = NULL;
	if( menu_bar_items[0].label != NULL )
		{
		renderWindow->menuBar = XmCreateMenuBar( renderWindow->mainw,
												 "menubar", NULL, 0 );

		for ( i=0 ; menu_bar_items[i].label != NULL ; i++ )
			{
			uiBuildPulldownMenu(	renderWindow->menuBar,
									menu_bar_items[i].label,
									(char) NULL,
									True,
									menu_bar_items[i].menu );
			}

		XtManageChild( renderWindow->menuBar	);
		}


	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//	Build the "bar-o-buttons"
	//
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	renderWindow->hotkeyBar = NULL;
	uiBuildHotKeyBar( renderWindow, hk_items );


	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	//	Create the rendering area (with frame)
	//
	//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	renderWindow->glxarea_frame = 

		XtVaCreateManagedWidget( "glxarea_frame",
			xmFrameWidgetClass,	renderWindow->mainw,
			XmNtraversalOn,		True,
			NULL);


	renderWindow->glxarea = 

		XtVaCreateManagedWidget("glxarea",
			glwMDrawingAreaWidgetClass, renderWindow->glxarea_frame,
			GLwNvisualInfo, 			renderWindow->vi,
			XmNheight,					renderWindow->glxarea_height,
			XmNwidth,					renderWindow->glxarea_width,
			XmNtraversalOn,				True,
			NULL);


	XtAddCallback(renderWindow->glxarea, XmNexposeCallback,
					callback_list.exposeCB,	callback_list.exposeData);

	XtAddCallback(renderWindow->glxarea, XmNresizeCallback,
					callback_list.resizeCB,	callback_list.exposeData);

	XtAddCallback(renderWindow->glxarea, XmNinputCallback,
					callback_list.inputCB, 	callback_list.exposeData);


	XtVaSetValues( renderWindow->mainw,
		XmNmenuBar,		  renderWindow->menuBar,
		XmNcommandWindow, renderWindow->hotkeyBar,
		XmNworkWindow,	  renderWindow->glxarea_frame,
		XmNinitialFocus,  renderWindow->glxarea_frame,
		NULL);


	return( retVisFnd );
	}


//----------------------------------------------------------------------------
//
//		uiBuildHotKeyBar
//
//----------------------------------------------------------------------------
void	uiBuildHotKeyBar( uiRenderWindow *renderWindow,
						  uiHotkeyItem *hotkeyItems
						 )
	{
	int	i;

    Arg             args[20];
	XmString		str;

	Screen	*screen;
	Pixel	foreground;
	Pixel	background;
	Pixmap	pixmap;

	Widget	button;

	if ( hotkeyItems[0].label == NULL )
		return;

	//
	//	Could add in stuff for a pixmap button here
	//
	//
	renderWindow->hotkeyBar =

		XtVaCreateManagedWidget("rowcol",
			xmRowColumnWidgetClass,	renderWindow->mainw,
			XmNorientation,			XmHORIZONTAL,
			XmNpacking,				XmPACK_TIGHT,
			XmNindicatorType,		XmN_OF_MANY,
			NULL);

	XtManageChild( renderWindow->hotkeyBar	);

	for ( i=0 ; hotkeyItems[i].label != NULL ; i++ )
		{
		str = XmStringCreateLocalized (hotkeyItems[i].label);
		XtSetArg(args[0],XmNlabelString, str );
		XtSetArg(args[1],XmNwidth, 30);


    	if (hotkeyItems[i].callback)
			{
			//
			//	Toggle Button
			//
			if ( hotkeyItems[i].widget_class == &xmToggleButtonWidgetClass )
				{
				button = XmCreateToggleButton( renderWindow->hotkeyBar, 
												hotkeyItems[i].label, args, 2);
				XtManageChild(button);
				XtAddCallback(button,
								XmNvalueChangedCallback,
								hotkeyItems[i].callback, 
								hotkeyItems[i].callback_data);
				}
			//
			//	Push Button
			//
			else
				{
				button = XmCreatePushButton( renderWindow->hotkeyBar,
											 hotkeyItems[i].label, args, 2);
				XtManageChild(button);
				XtAddCallback(button,
								XmNactivateCallback,
								hotkeyItems[i].callback,
								hotkeyItems[i].callback_data);
				}
			}
		}

	return;

	//
	//	Hack ABOVE
	//

	foreground = BlackPixel( renderWindow->dpy, 0 );
	background = WhitePixel( renderWindow->dpy, 0 );

	screen = XtScreen( renderWindow->mainw );
	pixmap = XmGetPixmap( screen,
							"sgilogo64",
							foreground,
							background );

	if ( pixmap == XmUNSPECIFIED_PIXMAP )
		fprintf(stderr,"Couldn't find pixmap\n");

	XtSetArg(args[0],XmNlabelType, XmPIXMAP);
	XtSetArg(args[1],XmNarmPixmap, pixmap);

	button = XmCreatePushButton( renderWindow->hotkeyBar, "button1", args, 2);
	XtManageChild(button);

//
//	----------------------
//
	pixmap = XmGetPixmapByDepth( screen,
								"marble3small.xpm",
								foreground,
								background,
								2 );

	if ( pixmap == XmUNSPECIFIED_PIXMAP )
		fprintf(stderr,"Couldn't find pixmap\n");

	XtSetArg(args[0],XmNlabelType, XmPIXMAP);
	XtSetArg(args[1],XmNarmPixmap, pixmap);

	button = XmCreatePushButton( renderWindow->hotkeyBar, "button2", args, 2);
	XtManageChild(button);
	}


//----------------------------------------------------------------------------
//
//		uiBuildPullDownMenu
//
//
//----------------------------------------------------------------------------
Widget	uiBuildPulldownMenu( Widget parent,
								char		*menu_title,
								char		menu_mnemonic,
								Boolean		tear_off,
								uiMenuItem	*items)
    {
    Widget  PullDown;
    Widget  cascade;
    Widget  widget;

    int i;
    XmString str;

    PullDown = XmCreatePulldownMenu (parent, "_pulldown", NULL, 0);


    if (tear_off)
    	XtVaSetValues (PullDown, XmNtearOffModel,XmTEAR_OFF_ENABLED, NULL);


    str = XmStringCreateLocalized (menu_title);

    cascade = XtVaCreateManagedWidget (menu_title,
    	    		xmCascadeButtonGadgetClass,	parent,
    	    		XmNsubMenuId,   			PullDown,
    	    		XmNlabelString, 			str,
    	    		XmNmnemonic,    			menu_mnemonic,
    	    		NULL);

    XmStringFree (str);

    
    //
	//	Now add the menu items
	//
    for (i = 0; items[i].label != NULL; i++)
    	{
		//
		// If subitems exist, create the pull-right menu by calling this
		//	function recursively.  Since the function returns a cascade
		//	button, the widget returned is used..
    	//
    	if (items[i].subitems)
    	    widget = uiBuildPulldownMenu (PullDown,
											items[i].label, 
    	    								items[i].mnemonic,
											tear_off,
											items[i].subitems);
    	else
    	    widget = XtVaCreateManagedWidget (items[i].label,
    	    									*items[i].item_class,
												PullDown,
    	    									NULL);

    	//	
		//	Whether the item is a real item or a cascade button with a
    	//	menu, it can still have a mnemonic.
    	//
    	if (items[i].mnemonic)
    	    XtVaSetValues (widget, XmNmnemonic, items[i].mnemonic, NULL);
    	
    	// any item can have an accelerator, except cascade menus. But,
    	// we do not worry about that; we know better in our declarations.
    	//
    	if (items[i].accelerator)
    	    {
    	    str = XmStringCreateLocalized (items[i].accel_text);
    	    XtVaSetValues (widget,
    	    				XmNaccelerator, items[i].accelerator,
    	    				XmNacceleratorText, str,
    	    				NULL);
    	    XmStringFree (str);
    	    }

    	if (items[i].callback)
    	    XtAddCallback(widget,
    	    	(items[i].item_class == &xmToggleButtonWidgetClass ||
    	    	items[i].item_class == &xmToggleButtonGadgetClass) ?
				XmNvalueChangedCallback : // ToggleButton class
    	    	XmNactivateCallback,      // PushButton class
     	    	(XtCallbackProc) items[i].callback,
				items[i].callback_data);
    	}

    return cascade;
    }



//----------------------------------------------------------------------------
//
//		uiRenderIdleProc
//
//			Rendering Idle Proc
//
//----------------------------------------------------------------------------
Boolean	uiRenderIdleProc( XtPointer )
	{
	uiRenderWindow*	tmp_ptr;

	if( RenderList == NULL )	sginap(1);
	else
		while ( RenderList != NULL )
		{
		//fprintf( stderr, "Rendering Window: %d\n", RenderList->window_id );

		glXMakeCurrent( RenderList->dpy,
						XtWindow(RenderList->glxarea),
						RenderList->cx);

		if ( RenderList->needsInit )
			{
			(RenderList->init)( RenderList );
			RenderList->needsInit = 0;
			}

		(RenderList->draw)( RenderList->glxarea,
							RenderList->dpy,
							&(RenderList->dirtyViewport),
							RenderList->uiData );

		RenderList->onRenderList = 0;
		
		tmp_ptr = RenderList->nextRenderWindow;
		RenderList->nextRenderWindow = NULL;
		RenderList = tmp_ptr;
		}

	return False;
	}

//----------------------------------------------------------------------------
//
//		uiPostRender
//
//			Post a window to the Rendering list.
//
//----------------------------------------------------------------------------
void	uiPostRender( uiRenderWindow* window_to_render ) 
	{
	uiRenderWindow* tmp_ptr;

	if ( window_to_render->onRenderList == 1 )
		{
		return;
		}

	if ( !(XtIsRealized(window_to_render->glxarea)) )
		{
		return;
		}

	if ( RenderList != NULL )
		{
		tmp_ptr = RenderList;
		while ( tmp_ptr->nextRenderWindow != NULL )
			{
			tmp_ptr = tmp_ptr->nextRenderWindow;
			}

		tmp_ptr->nextRenderWindow = window_to_render;
		window_to_render->onRenderList = 1;
		}
	else
		{
		RenderList = window_to_render;
		window_to_render->onRenderList = 1;
		}
	}

//----------------------------------------------------------------------------
//
//		uiRenderImmediately
//
//			Render the requested window NOW !
//
//----------------------------------------------------------------------------
void	uiRenderImmediately( uiRenderWindow* window_to_render ) 
	{
	glXMakeCurrent( window_to_render->dpy,
					XtWindow( window_to_render->glxarea ),
					window_to_render->cx );

	(window_to_render->draw) (	window_to_render->glxarea,
								window_to_render->dpy,
								&(window_to_render->dirtyViewport ),
								window_to_render->uiData );
	}		
