//============================================================================
//
//		ui.h
//
//
//			Jonathan Shade
//
//============================================================================

#include <Xm/Xm.h>
#include <GL/glx.h>

#ifndef	UI_H
#define UI_H

struct _uiRenderWindow;

typedef void (* uiInitFunc) (struct _uiRenderWindow *ren_win);
typedef	void (* uiGenericFunc) (void);
typedef void (* uiDrawFunc) (Widget		w,
							 Display	*dpy,
							 int		*dirtyViewport,
							 void		*ui_data);

typedef struct _uiRenderWindow
	{
	int		window_id;

	Widget	parent;
	Widget	shell;
	Widget	mainw;

	Widget	menuBar;
	Widget	hotkeyBar;
	Widget	glxarea_frame;
	Widget	glxarea;
	
	int		glxarea_width;
	int		glxarea_height;

	int				onRenderList;
	int				dirtyViewport;
	int				needsInit;

	uiDrawFunc		draw;
	uiInitFunc		init;

	int				num_constraints;
	struct _uiRenderWindow *constraint_list;

	void			*uiData;

	Display			*dpy;
	GLXContext		cx;
	XVisualInfo		*vi;
	XtAppContext	app;

	struct _uiRenderWindow *nextRenderWindow;

	}	uiRenderWindow;	



typedef struct _ui_menu_item
	{
	char        *label;
	WidgetClass *item_class;
	char        mnemonic;
	char        *accelerator;
	char        *accel_text;
	//void        (*callback)();
	XtCallbackProc	callback;
	XtPointer   callback_data;
	struct _ui_menu_item *subitems;
	}   uiMenuItem;

typedef struct _ui_menu_bar_item
	{
	char		*label;
	uiMenuItem	*menu;
	}	uiMenuBarItem;

	
typedef struct _hotkey_item
	{
	char		*label;
	char		*pixmap;
	WidgetClass *widget_class;
	XtCallbackProc	callback;
	XtPointer   callback_data;
	} uiHotkeyItem;
	

typedef struct _ui_callback_list
	{
	XtCallbackProc	exposeCB;
	XtCallbackProc	resizeCB;
	XtCallbackProc	inputCB;

	uiRenderWindow	*exposeData;
	uiRenderWindow	*resizeData;
	uiRenderWindow	*inputData;
	}	uiCallbackList;

typedef struct _ui_visual_list
	{
	int  *reqstr;
	char *descstr;
	} uiVisualList;


int		ui_Create_GL_Rendering_Window(	uiRenderWindow	*renderWindow,
										uiCallbackList	callback_list,
										uiMenuBarItem	*menu_items,
										uiHotkeyItem	*hk_Items,
										uiVisualList	*v_list,
										GLXContext		share_list );

void	uiBuildHotKeyBar( uiRenderWindow *renderWindow,
						  uiHotkeyItem *hotkeyItems );

Widget	uiBuildPulldownMenu(	Widget parent,
								char *menu_title,
								char menu_mnemonic,
								Boolean tear_off,
								uiMenuItem *items);


Boolean	uiRenderIdleProc( XtPointer data );

void	uiPostRender( uiRenderWindow* window_to_render );
void	uiRenderImmediately( uiRenderWindow* window_to_render );

#endif





